/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CEthCommSvr.c   :                                                 //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CEthCommSvr.h"
#include "SysUtil.h"
#include "Mem.h"
#include "Wacp.h"
#include <stdio.h>

/*********************************************************************************
CEthCommSvr_Init: Initializes the CEthCommSvr settings.

Parameters:
    pObject - Pointer to the CEthCommSvr structure.
*********************************************************************************/
void CEthCommSvr_Init(CEthCommSvr* pObject)
{
    if (pObject)
    {
        /* Zero out structure */
        memset(&pObject->DestAddr, 0, sizeof(pObject->DestAddr));           
        memset((void*)&(pObject->SourceID[0]), 0, sizeof(pObject->SourceID));           
        
        /* Initialize the socket handles */
        pObject->TxSocket = INVALID_SOCKET;

        /* Set the stack type */
        pObject->StackType = ETHERNET;

        /* Default Protocol is TCP */
        pObject->Protocol = IPPROTO_TCP;

        /* Default Port is 0 */
        pObject->Port = 0;

        /* Initialize the WIN32 dll flag */
        pObject->WSAStartFlag = FALSE;

        /* Create shutdown mutex */
	    pObject->ClientExitMutex = CreateMutex(NULL, FALSE, NULL);
    }
}

/*********************************************************************************
CEthCommSvr_Destroy: Closes the TxSocket and destroys the shutdown mutex.
    This method waits until the StartClient thread terminates.

Parameters:
    pObject - Pointer to the CEthCommSvr structure.

*********************************************************************************/
void CEthCommSvr_Destroy(CEthCommSvr* pObject)
{
    if (pObject)
    {
        if (pObject->TxSocket != INVALID_SOCKET)
        {
            closesocket(pObject->TxSocket);
            pObject->TxSocket = INVALID_SOCKET;
        }

        if (pObject->WSAStartFlag == TRUE)
        {
            WSACleanup( );
        }

        WaitForSingleObject(pObject->ClientExitMutex, INFINITE);
        ReleaseMutex(pObject->ClientExitMutex);
    }
    CloseHandle(pObject->ClientExitMutex);
}

/*================================================================================
CCommSvr SetSourceID: Copies the pSourceID.
================================================================================*/
boolean CCommSvr_SetSourceID(char *pSourceID, const char* pNewSourceID)
{   
    boolean RetVal = FALSE;
    if ((pSourceID != NULL) && (pNewSourceID != NULL))
    {
        memset(pSourceID, 0, ADDR_BUF_SIZE);
        memcpy(pSourceID, pNewSourceID, strlen(pNewSourceID));
        RetVal = TRUE;
    }
    return (RetVal);
}

/*********************************************************************************
CEthCommSvr_SetConfig: Updates the addressing information of the socket 
    connection.

Parameters:
    pObject - Pointer to the CEthCommSvr structure.

    pAddr - Pointer to a EthAddr structure that contains the socket information
            for the connection.
*********************************************************************************/
void CEthCommSvr_SetConfig(CEthCommSvr* pObject, const EthAddr* pAddr)
{
    if (pObject)
    {
        if (pAddr != NULL)
        {
            memcpy(&pObject->Config, pAddr, sizeof(EthAddr));
            pObject->Protocol = pObject->Config.Protocol;
            pObject->TxSocket = pObject->Config.Socket;
            pObject->Port = pObject->Config.Port;
            CCommSvr_SetSourceID(pObject->SourceID, pObject->Config.pSourceID);
        }
    }
}

/*********************************************************************************
CEthCommSvr_Connect: Creates a socket that connects to the passed in SourceID 
    and Port. Updates the Port and SourceID once a connection is created.
    
Note: UDP does not perform a connection as it is a connection-less protocol.

Parameters:
    pObject - Pointer to the CEthCommSvr structure.

    pSourceID - String identifier(IP address) of the connection 

    PortNum - Port value for the socket connection. 

Returns: 
    TRUE, if the socket is created and the connection is made.
    FALSE, if the socket is not created, or the connection fails.

*********************************************************************************/
boolean CEthCommSvr_Connect(CEthCommSvr* pObject, uint8* pSourceID, int PortNum)
{
    struct WSAData Data;
    boolean Result = FALSE;

    if (pObject)
    {
        if (WSAStartup(MAKEWORD(2,1),&Data) == 0)
        {
            pObject->WSAStartFlag = TRUE;
        }
#ifdef DEBUG_PRINT_STATUS
        else
        {
            SYS_PRINTF("Init_Server: WSAStartup failed: %d\n", WSAGetLastError());
        }
#endif

        /* Construct the server Address structure */
        memset(&pObject->DestAddr, 0, sizeof(pObject->DestAddr));       /* Zero out structure */
        pObject->DestAddr.sin_family      = AF_INET;                    /* Internet Address family */
        pObject->DestAddr.sin_addr.s_addr = inet_addr((char*)pSourceID);/* Server IP Address */
        pObject->DestAddr.sin_port        = htons((u_short)PortNum);    /* Server Port */

        if (pObject->Protocol == IPPROTO_TCP)
        {
            /* Create a reliable, Stream socket using TCP */
            pObject->TxSocket = socket(PF_INET, SOCK_STREAM, pObject->Protocol);
            if (pObject->TxSocket != INVALID_SOCKET)
            {
                /* Establish the connection to the server */
                if (connect(pObject->TxSocket, (struct sockaddr*)(&pObject->DestAddr), sizeof(pObject->DestAddr)) >= 0)
                {
                    Result = TRUE;
                }
#ifdef DEBUG_PRINT_STATUS
                else
                {
                    SYS_PRINTF("ERROR!!! Unable to connect %d\n", WSAGetLastError());
                }
#endif
            }/* if (TxSocket >= 0) */

#ifdef DEBUG_PRINT_STATUS
            else
            {
                SYS_PRINTF("ERROR!!! Unable to create a TCP socket %d\n", WSAGetLastError());
            }
#endif
        }
        else if (pObject->Protocol == IPPROTO_UDP)
        {
            /* Create a UDP packet based connection */
            pObject->TxSocket = socket(PF_INET, SOCK_DGRAM, pObject->Protocol);

            if (pObject->TxSocket != INVALID_SOCKET)
            {
                pObject->Config.Port = (uint16)PortNum;
                pObject->Config.Protocol = pObject->Protocol;
                pObject->Config.Socket = pObject->TxSocket;
                pObject->Config.pSourceID = (char*)pSourceID;
                memcpy(&(pObject->Config.ClntAddr), &pObject->DestAddr, sizeof(pObject->DestAddr));
                Result = TRUE;
            }
#ifdef DEBUG_PRINT_STATUS
            else
            {
                SYS_PRINTF("ERROR!!! Unable to create UDP socket\n");
            }
#endif
        }  

        /* Update the Port */
        pObject->Port = (uint16)PortNum;

        /* Set the destination IP to be the SourceID */
        CCommSvr_SetSourceID(pObject->SourceID, (char*)pSourceID);
        
#ifdef DEBUG_PRINT
        if ( Result == TRUE)
        {
            SYS_PRINTF("Connection open to %s to Port %d\n", 
                    inet_ntoa(pObject->DestAddr.sin_addr), PortNum);
        }
        else
        {
            SYS_PRINTF("Unable to connect to %s to Port %d\n",
                    inet_ntoa(pObject->DestAddr.sin_addr), PortNum);
        }
#endif
    }
    return Result;

}

/*********************************************************************************
CEthCommSvr_SendMessage: Uses the current socket descriptor to send a message.
    
Parameters:
    pObject - Pointer to the CEthCommSvr structure.

    pBuf - Pointer to a buffer that contains the data to be sent.

    pBufSize - Size of the buffer pointed to by pBuf.

Returns: 
    TRUE, if the number of bytes sent matches BufSize
    FALSE, if the object is invalid, Socket is invalid, or the number of bytes
        sent does not match the BufSize.

*********************************************************************************/
boolean CEthCommSvr_SendMessage(const CEthCommSvr* pObject, const uint8* pBuf, uint32 BufSize)
{
    boolean RetVal = FALSE;
    int Bytes;

    if (pObject)
    {
        if (pObject->TxSocket != INVALID_SOCKET)
        {
            if (pObject->Protocol == IPPROTO_TCP)
            {
                Bytes = send(pObject->TxSocket, (char*)pBuf, (int)BufSize, 0);
            }
            else
            {
                Bytes = sendto(pObject->TxSocket, (char*)pBuf, (int)BufSize, 0, 
                                (struct sockaddr *) &(pObject->Config.ClntAddr), 
                                sizeof(pObject->Config.ClntAddr));
            }

            if (Bytes == (int)BufSize)
            {
                RetVal = TRUE;
#ifdef DEBUG_PRINT
                SYS_PRINTF("SendMessage: Sent %d Bytes\n", Bytes);
                {
                    uint32 i;
                    for (i=0; i < BufSize;i++)
                    {
                        if ( i % 25 == 0)
                            SYS_PRINTF("\n");
                        SYS_PRINTF("%.2x ", (uint8)pBuf[i]);
                    }
                    SYS_PRINTF("\n\n");
                }
#endif  
            }/* if (Bytes == (int)BufSize) */
#ifdef DEBUG_PRINT
            else
            {
                SYS_PRINTF("\n\nERROR!!! SendMessage: %d Bytes were sent out of %d\n\n", 
                        Bytes, BufSize);
            }
#endif
        }/* if (pObject->TxSocket != INVALID_SOCKET) */

    }/* if (pObject) */

    return RetVal;
}   

/*********************************************************************************
CEthCommSvr_StartClient: Handles the start of the new client connection 
    thread. The function loops continously reading data from the socket and 
    passing it into the Parser to handle valid WACP messages. This thread 
    terminates once the Socket handle has been closed by either side of the
    connection. 
    
Parameters:
    pData - Pointer to the CEthCommSvr structure that contains the addressing
            information for the socket. 

Returns: 
    Always returns TRUE.

*********************************************************************************/
uint32 WACPAPI CEthCommSvr_StartClient(void* pData)
{
    CEthCommSvr* pCommSvr = (CEthCommSvr*)pData;
    char         Buffer[RCVBUFSIZE];
    int          MsgSize;
    uint32       PacketLen;
    uint32       BytesProcessed, Index = 0;

    static int TestCount;

    if (pCommSvr)
    {
        /* Grab the Exit Mutex */
        WaitForSingleObject(pCommSvr->ClientExitMutex, INFINITE);

        if (pCommSvr->Protocol == IPPROTO_TCP)
        {
            /* Loop while there is data available */
            do
            {
                /* Receive the next message from client */
                MsgSize = recv(pCommSvr->TxSocket, &Buffer[Index], RCVBUFSIZE - Index, 0);
                if (MsgSize > 0)
                {
                    /* Pass the data to the WACP Parser */
                    BytesProcessed = WACPParser(pCommSvr->SourceID, (uint8*)Buffer, MsgSize + Index, &PacketLen);
                    if (BytesProcessed == 0)
                    {
                        /* Partial packet, increment the Index and continue to read data */
                        Index += MsgSize;
                    }
                    else if (BytesProcessed == (uint32)MsgSize + Index)
                    {
                        /* Full packet received, reset Index to 0 */
                        Index = 0;
                        memset(Buffer, 0, RCVBUFSIZE);
                    }
                    else if (BytesProcessed < (uint32)MsgSize + Index)
                    {
                        /* Multiple packets, with partial packet found, copy over processed packet 
                        * and set Index to the first unprocessed packet byte
                        */
                        Index = MsgSize + Index - BytesProcessed;
#if defined(DEBUG_PRINT)
                        SYS_PRINTF("Index = %d ,MsgSize = %d, BytesProcessed = %d\n", Index, MsgSize, BytesProcessed);
#endif
                        memcpy(Buffer, &Buffer[BytesProcessed], Index);
                    }
                }
            }while (MsgSize > 0);

            /* Check to see if this was a normal termination or break in the communication 
             * In a normal termination, the TxSocket is set to INVALID_SOCKET.
             * In a break in communication, the call to recv() will return -1.
             */
            if (pCommSvr->TxSocket != INVALID_SOCKET)
            {
                /* Notify the application a break in the connection has occurred */
                //CCommSvr_Removal(pCommSvr->SourceID, pCommSvr->Port);
            }

        }/* if (Protocol == IPPROTO_TCP) */
        else if (pCommSvr->Protocol == IPPROTO_UDP)
        {
            for(;;) 
            {
                if ((MsgSize = recvfrom(pCommSvr->TxSocket, Buffer, RCVBUFSIZE, 0,
                                        NULL, NULL)) > 0)
                {
                    /* UDP insures message boundaries so we don't have to worry about partial packets */
                    WACPParser(pCommSvr->SourceID, (uint8*)Buffer, MsgSize, &PacketLen);
                }

                /* Check to see if this is a fatal error */
                if (MsgSize < 0)
                {
                    /* For UDP, it's possible the socket has not been bounded
                     * which doesn't occur until a datagram is sent. In this case
                     * we can just look for WSAEINVAL and try again, otherwise
                     * we break out of the loop and terminate the reader thread.
                     */
                    if ((MsgSize == -1) && (GetLastError() == WSAEINVAL))
                    {
                        SYS_SLEEP(WAIT_COUNT);
                    }
                    else
                        break;
                }
            }

        }/*else if (Protocol == IPPROTO_UDP) */

        /* Release the Exit lock */
        ReleaseMutex(pCommSvr->ClientExitMutex);
    }
#ifdef DEBUG_PRINT
        SYS_PRINTF("CEthCommSvr HandleClient: Thread exiting\n");
#endif

    return TRUE;
}




