/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CSerialCommSvr.c:                                                 //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/
#include "CSerialCommSvr.h"
#include "SysUtil.h"
#include "stdio.h"
#include "target.h"

const uint8 START_RENDEZVOUS[] = /*(37)*/
{   0x17, 0x01, 0x0c, //envelope preamble
	0x00, 0x00, 0x00, 0x25,//envelope size
	0x01, 0x19, //envelope port
		0x00, 0x1d, 0x0b, 0x01,//message class ID 
		0x00, 0x00, 0x00, 0x12,//message package size 
		0x00, //message enc/Comp
		0x00, 0x00, 0x00, 0x0b,//Message payload data size
			0x52, 0x4e, 0x44, 0x5a, 0x43, 0x4f, 0x4e, 0x4e, 0x45, 0x43, 0x54, 
		0x57, 0xf3, //Message CRC
	0xe9, 0x29 //Envelope CRC
};

/*********************************************************************************
CSerialCommSvr_Init: Initializes the Overlapped IO events and CSerialCommSvr 
    settings.

Parameters:
    pObject - Pointer to the CSerialCommSvr structure.
*********************************************************************************/
void CSerialCommSvr_Init(CSerialCommSvr* pObject)
{
    if (pObject)
    {
        /* Zero out structure */
        memset(&(pObject->SourceID[0]), 0, sizeof(pObject->SourceID));           
        
        /* Set the stack type */
        pObject->hPort = INVALID_HANDLE_VALUE;

        /* Create shutdown mutex */
	    pObject->ExitMutex = CreateMutex(NULL, FALSE, NULL);
    }
}

/*********************************************************************************
CSerialCommSvr_Destroy: Closes the com port and destroys the shutdown mutex.
    This method waits until the StartClient thread terminates.

Parameters:
    pObject - Pointer to the CSerialCommSvr structure.

*********************************************************************************/
void CSerialCommSvr_Destroy(CSerialCommSvr* pObject)
{
    if (pObject)
    {
        if (pObject->hPort != INVALID_HANDLE_VALUE)
        {
            /* Call SetCommMask to cancel any current threads waiting on WaitCommEvent() */
            SetCommMask (pObject->hPort, EV_DSR);

            /* Close the communication Port. */
            CloseHandle(pObject->hPort);

            pObject->hPort = INVALID_HANDLE_VALUE;

            /* Wait for shutdown */
		    WaitForSingleObject(pObject->ExitMutex, INFINITE);
        }

        CloseHandle(pObject->ExitMutex);
    }
}

/*********************************************************************************
CSerialCommSvr_SetSourceID: Sets the SourceID.
    
Parameters:
    pObject - Pointer to the CSerialCommSvr structure.

    pSourceID - String identifier(Com Port) of the connection 
    
    pSourceID - String identifier(Com Port) of the connection 

Returns: 
    TRUE, if the SourceID was copied.
    FALSE, if any parameter was invalid.

*********************************************************************************/
boolean CSerialCommSvr_SetSourceID(CSerialCommSvr* pObject, char *pSourceID, const char* pNewSourceID)
{   
    boolean RetVal = FALSE;
    TOUCH(pObject);
    if ((pSourceID != NULL) && (pNewSourceID != NULL))
    {
        memset(pSourceID, 0, ADDR_BUF_SIZE);
        memcpy(pSourceID, pNewSourceID, strlen(pNewSourceID));
        RetVal = TRUE;
    }
    return (RetVal);
}

/*********************************************************************************
CSerialCommSvr_Connect: Opens a serial com port based on the pSourceID and 
    PortNum. Updates the Port and SourceID once a connection is created.
    
Parameters:
    pObject - Pointer to the CSerialCommSvr structure.

    pSourceID - String identifier(Com Port) of the connection 

    PortNum - Port value used for message identifier 
              i.e. RNDZ vs WACP message.

Returns: 
    TRUE, if the serial port is opened.
    FALSE, if the serial port fails to open or if the port handle is 
        currently not NULL.

*********************************************************************************/
boolean CSerialCommSvr_Connect(CSerialCommSvr* pObject, const uint8* pSourceID, int PortNum)
{
    boolean Result = TRUE;
    DCB PortDCB;                    /* Port Control Settings Structure */
    COMMTIMEOUTS CommTimeouts;      /* Comm Port timeout structure */

    if (pObject)
    {
        pObject->Port = (uint16)PortNum;

        if (pObject->hPort == INVALID_HANDLE_VALUE)
        {
            DWORD Error;
            char TempBuf[ADDR_BUF_SIZE];

            /* Clear the temporary buffer */
            memset(TempBuf, 0, ADDR_BUF_SIZE);

            /* Append "\\.\" to allow port number greater than 9 */
            sprintf(TempBuf, "\\\\.\\%s", pSourceID);

            /*  Open the serial Port. */
            pObject->hPort = CreateFile ((LPCSTR)TempBuf, /* Pointer to the Name of the Port */
                            GENERIC_READ | GENERIC_WRITE,
                                                            /*  Access (read-write) mode */
                            0,                              /* Share mode */
                            NULL,                           /* Pointer to the security attribute */
                            OPEN_EXISTING,                  /* How to open the serial Port */
#if defined(OVERLAPPED_SERIAL)
                            FILE_ATTRIBUTE_NORMAL |
                            FILE_FLAG_OVERLAPPED,           /* Port attributes */
#else
                            FILE_ATTRIBUTE_NORMAL,
#endif                        
                            NULL);                          /* Handle to Port with attribute */
                                                            /* to copy */


            PortDCB.DCBlength = sizeof (DCB);     

            /* Get the default Port setting information. */
            GetCommState (pObject->hPort, &PortDCB);

            /* Change the DCB structure settings. */
            PortDCB.BaudRate = 115200;      /* Current baud */
            PortDCB.fBinary = TRUE;                 /* Binary mode; no EOF check */
            PortDCB.fParity = TRUE;                 /* Enable parity checking */
            PortDCB.fOutxCtsFlow = FALSE;           /* CTS Output flow control */
            PortDCB.fOutxDsrFlow = FALSE;           /* DSR Output flow control */
            PortDCB.fDtrControl = DTR_CONTROL_DISABLE; 
                                                    /* DTR flow control type */
            PortDCB.fDsrSensitivity = FALSE;        /* DSR sensitivity */
            PortDCB.fTXContinueOnXoff = FALSE;       /* XOFF continues Tx */
            PortDCB.fOutX = FALSE;                  /* No XON/XOFF out flow control */
            PortDCB.fInX = FALSE;                   /* No XON/XOFF in flow control */
            PortDCB.fErrorChar = FALSE;             /* Disable error replacement */
            PortDCB.fNull = FALSE;                  /* Disable null stripping */
            PortDCB.fRtsControl = RTS_CONTROL_DISABLE; 
                                                    /* RTS flow control */
            PortDCB.fAbortOnError = FALSE;          /* Do not abort reads/writes on */
                                                    /* error */
            PortDCB.ByteSize = 8;                   /* Number of bits/byte, 4-8 */
            PortDCB.Parity = NOPARITY;              /* 0-4=no,odd,even,mark,space */
            PortDCB.StopBits = ONESTOPBIT;          /* 0,1,2 = 1, 1.5, 2 */

            /* Configure the Port according to the specifications of the DCB  
            * structure. 
            */
            if (!SetCommState (pObject->hPort, &PortDCB))
            {
                Result = FALSE; 
                Error = GetLastError();
#ifdef DEBUG_PRINT_STATUS
                SYS_PRINTF("Error %d!!! Unable to set the com Port settings\n", GetLastError ());
#endif
            }

            /* Retrieve the time-out parameters for all read and write operations
            * on the Port. 
            */
            GetCommTimeouts (pObject->hPort, &CommTimeouts);

            /* Change the COMMTIMEOUTS structure settings. */
            CommTimeouts.ReadIntervalTimeout = MAXDWORD;  
            CommTimeouts.ReadTotalTimeoutMultiplier = 0;  
            CommTimeouts.ReadTotalTimeoutConstant = 0;    
            CommTimeouts.WriteTotalTimeoutMultiplier = 10;  
            CommTimeouts.WriteTotalTimeoutConstant = 1000;    

            /* Set the time-out parameters for all read and write operations 
            * on the Port. 
            */
            if (!SetCommTimeouts (pObject->hPort, &CommTimeouts))
            {
                Result = FALSE; 
#ifdef DEBUG_PRINT_STATUS
                SYS_PRINTF("Error %d!!! Unable to set comm timeouts\n", GetLastError ());
#endif
            }

            CSerialCommSvr_SetSourceID(pObject, pObject->SourceID, (char*)pSourceID);
        }
    }

    return Result;
}

/*********************************************************************************
CSerialCommSvr_SendMessage: Sends the passed in buffer through the com port.
    
Parameters:
    pObject - Pointer to the CSerialCommSvr structure.

    pBuf - Pointer to a buffer that contains the data to be sent.

    BufSize - Size of the buffer pointed to by pBuf.

Returns: 
    TRUE, if the number of bytes sent matches BufSize
    FALSE, if the object is invalid, handle is invalid, or the number of bytes
        sent does not match the BufSize.

*********************************************************************************/
boolean CSerialCommSvr_SendMessage(CSerialCommSvr* pObject, const uint8* pBuf, uint32 BufSize)
{
    boolean Result = TRUE;
    DWORD DwNumBytesWritten;

    if (pObject)
    {
        if (pObject->hPort != INVALID_HANDLE_VALUE)
        {
#if !defined(OVERLAPPED_SERIAL)
            if (!WriteFile (pObject->hPort,            /* Port handle */
                            pBuf,                   /* Pointer to the data to write */
                            BufSize,               /* Number of Bytes to write */
                            &DwNumBytesWritten,     /* Pointer to the number of Bytes */
                                                    /* written */
                            NULL))      /* Must be NULL for Windows CE */
            {
                Result = FALSE;
            }
#else
            if (!WriteFile (pObject->hPort,            /* Port handle */
                            pBuf,                   /* Pointer to the data to write */
                            BufSize,               /* Number of Bytes to write */
                            &DwNumBytesWritten,     /* Pointer to the number of Bytes */
                                                    /* written */
                            &(pObject->OSWrite)))      /* Must be NULL for Windows CE */
            {
                DWORD DwRes;
                if (GetLastError() != ERROR_IO_PENDING) 
                { 
                    /* WriteFile failed, but it isn't delayed. Report error. */
                    Result = FALSE;
                }
                else
                {
                    /* Write is pending. */
                    DwRes = WaitForSingleObject(pObject->OSWrite.hEvent, INFINITE);
                    switch(DwRes)
                    {
                        /* Overlapped event has been signaled. */
                        case WAIT_SUCCESS:
                            if (!GetOverlappedResult(pObject->hPort, &(pObject->OSWrite), &DwNumBytesWritten, FALSE))
                                Result = FALSE;
                            else 
                            {
                                if(DwNumBytesWritten != BufSize) 
                                {
                                    /* The write operation timed out. I now need to 
                                    * decide if I want to abort or retry. If I retry, 
                                    * I need to send only the Bytes that weren't sent. 
                                    * If I want to abort, I would just set fRes to 
                                    * FALSE and return.
                                    */
                                    Result = FALSE;
                                }
                                else
                                    /* Write operation completed successfully. */
                                    Result = TRUE;
                            }
                            break;
                        default:
                            /* An error has occurred in WaitForSingleObject. This usually */
                            /* indicates a problem with the overlapped event handle. */

                            Result = FALSE;
                            break;
                    }/* switch(DwRes) */
                }/* else */
            }/* if (!WriteFile (pObject->hPort...) */
#endif

#ifdef DEBUG_PRINT
            {
                uint32 i;
                SYS_PRINTF("SendMessage: Sent %d of %d Bytes\n", DwNumBytesWritten, BufSize);
                for (i=0; i < DwNumBytesWritten;i++)
                {
                    if (i % 25 == 0)
                        SYS_PRINTF("\n");
                    SYS_PRINTF("%.2x ", (uint8)pBuf[i]);
                }
                SYS_PRINTF("\n\n");
            }
#endif  
        }/* if (pObject->hPort != INVALID_HANDLE) */

    }/* if (pObject) */

    return Result;
}   


/*********************************************************************************
CSerialCommSvr_ReadData: Reads up to BytesToRead number of bytes of data from 
    the serial connection. 

    If Overlapped I/O. We will block until data is ready or the pipe is closed.
    If not overlapped I/O. We will wait SERIAL_POLLTIME before another read.
    
    If DTR line goes low, we will call the Removal callback to terminate the
    connection.

Parameters:
    pObject - Pointer to the CSerialCommSvr structure.

    pBuf - Pointer to a buffer that will receive the data.

    BytesToRead - Number of bytes to read.

    pBytesRead - Pointer to a uint32 used to pass back the number of bytes read.

Returns: 
    TRUE, if ReadFile return without errors. 
    FALSE, if the object is invalid, handle is invalid, or ReadFile returns
        an error.

*********************************************************************************/
boolean CSerialCommSvr_ReadData(CSerialCommSvr* pObject, char* pBuf, uint32 BytesToRead, uint32* pBytesRead)
{
    boolean Result = FALSE;
    DWORD DwCommModemStatus;

    if (pObject)
    {
        if (pObject->hPort != INVALID_HANDLE_VALUE) 
        {
            /* Read the data from the serial Port. */
#if !defined(OVERLAPPED_SERIAL)
            Result = (boolean)ReadFile(pObject->hPort, pBuf, BytesToRead, pBytesRead, NULL);
#else
            Result = (boolean)ReadFile(pObject->hPort, pBuf, BytesToRead, pBytesRead, &pObject->OSRead);
            if (!Result)
            {
                if (GetLastError() == ERROR_IO_PENDING)
                {
                    while((Result = (boolean)GetOverlappedResult(pObject->hPort, &pObject->OSRead, pBytesRead, TRUE)) != TRUE)
                    {
                        if(GetLastError() == ERROR_IO_INCOMPLETE)
                            continue;
                        else
                            break;
                    }
                }
            }
#endif

            /* Get the latest modem status */
            if (GetCommModemStatus(pObject->hPort, &DwCommModemStatus))
            {
                /* Check to see if we are in a low DTR state */ 
                if(!(DwCommModemStatus & MS_DSR_ON))
                {
                    /* Wait CONNECTION_TIMEOUT milleseconds to see if the connection comes back */
                    SYS_SLEEP(CONNECTION_TIMEOUT);
                    if (GetCommModemStatus(pObject->hPort, &DwCommModemStatus))
                    {
                        /* If we are still low, then terminate the connection */
                        if(!(DwCommModemStatus & MS_DSR_ON))
                        {
                            //CCommSvr_Removal(pObject->SourceID, pObject->Port);
                        }
                    }
                }
            }
#if !defined(OVERLAPPED_SERIAL)
            if (*pBytesRead == 0)
            {
                /* Wait for a time */
                SYS_SLEEP(SERIAL_POLLTIME);

                /* Read the data from the serial Port. */
                Result = (boolean)ReadFile(pObject->hPort, pBuf, BytesToRead, pBytesRead, NULL);
            }
#else

            if (*pBytesRead == 0)
            {
                /* Wait for an event to occur for the Port. */
                Result = (boolean)WaitCommEvent(pObject->hPort, &DwCommModemStatus, 0);

                /* Specify a set of events to be monitored for the Port. */
                SetCommMask(pObject->hPort, EV_RXCHAR | EV_DSR);
                
                if (Result == TRUE)
                {
                    if (DwCommModemStatus & EV_RXCHAR) 
                    {
                        /* Read the data from the serial Port. */
                        Result = (boolean)ReadFile(pObject->hPort, pBuf, BytesToRead, pBytesRead, &pObject->OSRead);
                        if (!Result)
                        {
                            if (GetLastError() == ERROR_IO_PENDING)
                            {
                                while((Result = (boolean)GetOverlappedResult(pObject->hPort, &pObject->OSRead, pBytesRead, TRUE)) != TRUE)
                                {
                                    if(GetLastError() == ERROR_IO_INCOMPLETE)
                                        continue;
                                    else
                                        break;
                                }
                            }
                        }
                    }
                }
            }
#endif
        }
    }

    return(Result);
}

/*********************************************************************************
CSerialCommSvr_CheckDTR: Checks the serial connection for the DTR line to go high
    and creates a message to simulate the arrival of a RNDZ connect message.
    This routine blocks until the DTR line goes high or the pipe is closed.
    
Parameters:
    pObject - Pointer to the CSerialCommSvr structure.

Returns: 
    TRUE, DTR line goes high.
    FALSE, if the object is invalid or the handle is invalid

*********************************************************************************/
boolean CSerialCommSvr_CheckDTR(CSerialCommSvr* pObject)
{
    DWORD   DwCommModemStatus;
    boolean Result = FALSE;

    if (pObject)
    {
        if (pObject->hPort != INVALID_HANDLE_VALUE) 
        {
            /* Get the status of the modem lines, so we can check the DSR line */
            GetCommModemStatus(pObject->hPort, &DwCommModemStatus);

            while(!(DwCommModemStatus & MS_DSR_ON) && (pObject->hPort != INVALID_HANDLE_VALUE))
            {       
                /* Specify a set of events to be monitored for the Port. */
                SetCommMask (pObject->hPort, EV_DSR);

                /*  Wait for an event to occur for the Port. */
                Result = (boolean)WaitCommEvent (pObject->hPort, &DwCommModemStatus, 0);

                /* Get the status of the modem lines, so we can check the DSR line */
                GetCommModemStatus(pObject->hPort, &DwCommModemStatus);
            }
        
            if (pObject->hPort != INVALID_HANDLE_VALUE) 
            {
//                uint32 PacketLen=0;

                //WACPParser(pObject->SourceID, (uint8*)START_RENDEZVOUS, sizeof(START_RENDEZVOUS), &PacketLen);
            }

        }
    }
    return Result;
}

/*********************************************************************************
CSerialCommSvr_StartClient: Handles the start of the new client listener 
    thread. The function loops continously reading data from the com port
    and passing it into the Parser to handle valid WACP messages. The thread 
    terminates when the ReadData returns FALSE or the Com Port is closed
    and set to INVALID_HANDLE_VALUE.
    
Parameters:
    pData - Pointer to the CSerialCommSvr structure that contains the addressing
            information for the com port. 

Returns: 
    Always returns TRUE.

*********************************************************************************/
uint32 WACPAPI CSerialCommSvr_StartClient(void* pData)
{
    CSerialCommSvr* pCommSvr = (CSerialCommSvr*)pData;
    char    Buffer[MAX_PACKET_SIZE];
    uint32  MsgSize, PacketLen;
    int     Result = TRUE;
    uint32  BytesProcessed, Index = 0;

    if (pCommSvr)
    {
		WaitForSingleObject(pCommSvr->ExitMutex, INFINITE);

        PurgeComm(pCommSvr->hPort, PURGE_TXCLEAR | PURGE_RXCLEAR);

        /* SETDTR: Sends the DTR (data-terminal-ready) signal. */
        EscapeCommFunction (pCommSvr->hPort, SETDTR);

        //if(pCommSvr->Config.bAutoDiscovery == TRUE)
        {
            CSerialCommSvr_CheckDTR(pCommSvr);
        }

        /* Specify a set of events to be monitored for the Port. */
        SetCommMask (pCommSvr->hPort, EV_RXCHAR | EV_DSR);

        while (pCommSvr->hPort != INVALID_HANDLE_VALUE) 
        {
            /* Receive the next message from client */
            Result = CSerialCommSvr_ReadData(pCommSvr, (char*)&Buffer[Index], MAX_PACKET_SIZE - Index, &MsgSize);
            if ((Result) && (MsgSize > 0))
            {
                /* Pass the data to the WACP Parser */
                BytesProcessed = WACPParser(pCommSvr->SourceID, (uint8*)Buffer, MsgSize + Index, &PacketLen);

                if (BytesProcessed == 0)
                {
                    /* Partial packet, increment the Index and continue to read data */
                    Index += MsgSize;
                }
                else if (BytesProcessed == (uint32)MsgSize + Index)
                {
                    /* Full packet received, reset Index to 0 */
                    Index = 0;
                    memset(Buffer, 0, MAX_PACKET_SIZE);
                }
                else if (BytesProcessed < (uint32)MsgSize + Index)
                {
                    /* Multiple packets, with partial packet found, copy over processed packet 
                     * and set Index to the first unprocessed packet byte
                     */
                    Index = MsgSize + Index - BytesProcessed;
                    memcpy(Buffer, &Buffer[BytesProcessed], Index);
                }
            }
        }/* while (pCommSvr->hPort != INVALID_HANDLE_VALUE)  */
        
        ReleaseMutex(pCommSvr->ExitMutex);
    }
#ifdef DEBUG_PRINT
    SYS_PRINTF("CSerialCommSvr HandleClient: Thread exiting\n");
#endif

    return TRUE;
}/* CSerialCommSvr_StartClient() */




