/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Mem.c          :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "Mem.h"
#include "SysUtil.h"
#include <string.h>


/*================================================================================
ReadInt16Ext Endian-proof function to read a 16 bit integer from a buffer without
updating the pointer
================================================================================*/
uint16 ReadInt16Ext(const uint8* pBuf)
{
#if (MEDIA_ENDIANNES == BIG_ENDIAN)
    return ((((uint16)*(pBuf)) << 8) | ((uint16)*(pBuf+1))); 
#else
    return ((((uint16)*(pBuf+1)) << 8) | ((uint16)*(pBuf)));
#endif
}

/*================================================================================
ReadInt32Ext Endian-proof function to read a 32 bit integer from a buffer without
updating the pointer
================================================================================*/
uint32 ReadInt32Ext(const uint8* pBuf)
{
#if (MEDIA_ENDIANNES == BIG_ENDIAN)
    return ((((uint32)*(pBuf))   << 24) | 
            (((uint32)*(pBuf+1)) << 16) |
            (((uint32)*(pBuf+2)) <<  8) |
              (uint32)*(pBuf+3)); 
#else
    return ((((uint32)*(pBuf+3)) << 24) | 
            (((uint32)*(pBuf+2)) << 16) |
            (((uint32)*(pBuf+1)) <<  8) |
              (uint32)*(pBuf)); 
#endif
}
/*================================================================================
ReadInt8 Endian-proof function to read a 8 bit integer from a buffer.
================================================================================*/
uint8 ReadInt8(uint8** ppBuf)
{
    uint8* pBuf = *ppBuf;      /* Dereference buffer pointer */
    *ppBuf += sizeof(uint8);    /* Increment pointer */

    return ((uint8)*(pBuf)); 
}

/*================================================================================
ReadInt16 Endian-proof function to read a 16 bit integer from a buffer.
================================================================================*/
uint16 ReadInt16(uint8** ppBuf)
{
    uint8* pBuf = *ppBuf;      /* Dereference buffer pointer */
    *ppBuf += sizeof(uint16);    /* Increment pointer */

#if (MEDIA_ENDIANNES == BIG_ENDIAN)
    return ((((uint16)*(pBuf)) << 8) | ((uint16)*(pBuf+1))); 
#else
    return ((((uint16)*(pBuf+1)) << 8) | ((uint16)*(pBuf)));
#endif

}

/*================================================================================
ReadInt32 Endian-proof function to read an 32 bit integer from a buffer.
================================================================================*/
uint32 ReadInt32(uint8** ppBuf)
{
    uint8* pBuf = *ppBuf;      /* Dereference buffer pointer */
    *ppBuf += sizeof(uint32);    /* Increment pointer */

#if (MEDIA_ENDIANNES == BIG_ENDIAN)
    return ((((uint32)*(pBuf))   << 24) | 
            (((uint32)*(pBuf+1)) << 16) |
            (((uint32)*(pBuf+2)) <<  8) |
              (uint32)*(pBuf+3)); 
#else
    return ((((uint32)*(pBuf+3)) << 24) | 
            (((uint32)*(pBuf+2)) << 16) |
            (((uint32)*(pBuf+1)) <<  8) |
              (uint32)*(pBuf)); 
#endif
}

/*================================================================================
Method to get a 4 byte IEEE format floating point value from the buffer. 
================================================================================*/
float ReadFloat(uint8** ppBuf)
{
    union {
        float fval;
        uint8 uval[SIZEOF_FLOAT];
    } ftou;
    uint8   *bptr;
    int8    binc;
    uint8   cnt;

    bptr = &ftou.uval[F32_START_INDEX];
    binc = F32_INC;

    for (cnt = 0; cnt < SIZEOF_FLOAT; cnt++)
    {
        *bptr = ReadInt8(ppBuf);
        bptr += binc;
    }

    return ftou.fval;
}

/*================================================================================
Method to get a 8 byte IEEE format double value from the buffer. 
================================================================================*/
double ReadDouble(uint8** ppBuf)
{
    union {
        double dval;
        uint8 uval[SIZEOF_DOUBLE];
    } ftou;
    uint8*  bptr;
    int8    binc;
    uint8   cnt;

    bptr = &ftou.uval[F64_START_INDEX];
    binc = F64_INC;

    for (cnt = 0; cnt < SIZEOF_DOUBLE; cnt++)
    {
        *bptr = ReadInt8(ppBuf);
        bptr += binc;
    }

    return ftou.dval;
}

/*================================================================================
WriteInt8 Endian-proof function to write a 16 bit integer into a buffer.
================================================================================*/
void WriteInt8(uint8** ppBuf, uint8 Val)
{
    uint8* pBuf = *ppBuf;      /* Dereference buffer pointer */
    *ppBuf += sizeof(uint8);    /* Increment pointer */

    *(pBuf) = (uint8)(Val);
}

/*================================================================================
WriteInt16 Endian-proof function to write a 16 bit integer into a buffer.
================================================================================*/
void WriteInt16(uint8** ppBuf, uint16 Val)
{
    uint8* pBuf = *ppBuf;      /* Dereference buffer pointer */
    *ppBuf += sizeof(uint16);    /* Increment pointer */

#if (MEDIA_ENDIANNES == BIG_ENDIAN)
    *(pBuf) = (uint8)(Val >> 8);
    *(pBuf+1) = (uint8)(Val);
#else
    *(pBuf+1) = (uint8)(Val >> 8);
    *(pBuf) = (uint8)(Val);
#endif
}

/*================================================================================
WriteInt32 Endian-proof function to write a 32 bit integer into a buffer.
================================================================================*/
void WriteInt32(uint8** ppBuf, uint32 Val)
{
    uint8* pBuf = *ppBuf;      /* Dereference buffer pointer */
    *ppBuf += sizeof(uint32);    /* Increment pointer */

#if (MEDIA_ENDIANNES == BIG_ENDIAN)
    *(pBuf)   = (uint8)(Val >> 24);
    *(pBuf+1) = (uint8)(Val >> 16);
    *(pBuf+2) = (uint8)(Val >> 8);
    *(pBuf+3) = (uint8)(Val);
#else
    *(pBuf+3) = (uint8)(Val >> 24);
    *(pBuf+2) = (uint8)(Val >> 16);
    *(pBuf+1) = (uint8)(Val >> 8);
    *(pBuf)   = (uint8)(Val);
#endif
}

/*================================================================================
Method to write a 4 byte IEEE format floating point value to the buffer. 
================================================================================*/
void WriteFloat(uint8** ppBuf, float Val)
{
    union {
        float fval;
        uint8 uval[SIZEOF_FLOAT];
    } ftou;
    uint8   *bptr;
    int8    binc;
    uint8   cnt;

    ftou.fval = Val;
    bptr = &ftou.uval[F32_START_INDEX];
    binc = F32_INC;
    for (cnt = 0; cnt < SIZEOF_FLOAT; cnt++)
    {
        WriteInt8(ppBuf, *bptr);
        bptr += binc;
    }
}

/*================================================================================
Method to write a 8 byte IEEE format double value to the buffer. 
================================================================================*/
void WriteDouble(uint8** ppBuf, double Val)
{
    union {
        double dval;
        uint8 uval[SIZEOF_DOUBLE];
    } ftou;
    uint8*  bptr;
    int8    binc;
    uint8   cnt;

    ftou.dval = Val;
    bptr = &ftou.uval[F64_START_INDEX];
    binc = F64_INC;

    for (cnt = 0; cnt < SIZEOF_DOUBLE; cnt++)
    {
        WriteInt8(ppBuf, *bptr);
        bptr += binc;
    }
}

/*================================================================================
ReadInt18 Endian-proof function to write a 18 bit integer into a buffer.
================================================================================*/
uint32 ReadInt18(const uint8* buffer, uint32 index)
{   
    uint8   bitOffset = (uint8)((index*2) & 0x0F);
    uint16  LSBs = ReadInt16Ext(buffer += (2 * (index + (index/8))));
    uint32  MSBs = ReadInt16Ext(buffer + 2);
    
    LSBs = ((LSBs & (0x0FFFF << bitOffset )) >> bitOffset);
    MSBs = ((MSBs & (0x0FFFF >> (16 - (bitOffset + 2)))) << (16 - bitOffset));

    return (uint32)((MSBs | LSBs) & 0x0003FFFF | (MSBs & 0x20000 ? 0xFFFC0000 : 0x00));    
}

/*================================================================================
WriteInt18 Endian-proof function to read a 18 bit integer from a buffer.
================================================================================*/
void WriteInt18(uint8* buffer, uint32 index, uint32 value)
{
    uint8   bitOffset = (uint8)((index*2) & 0x0F);
    uint16  first;
    uint16  second;

    value &= 0x3FFFF;

    first = (uint16)((value << bitOffset) & (0x0FFFF << bitOffset)) |
                    (ReadInt16Ext(buffer += (2 * (index + (index/8)))) & (0x0FFFF >> (16 - bitOffset)));

    second =(uint16)((value >> (16 - bitOffset)) & (0x3FFFF >> (16 - bitOffset))) |
                    (ReadInt16Ext(buffer + 2) & (0x0FFFF << (bitOffset + 2)));
        
    WriteInt16(&buffer, first);    
    WriteInt16(&buffer, second);

}

/*================================================================================
SetData: Common routine to add a buffer to the dynamic data buffer used by the 
wacp stack object servers. If pSrcBuf is NULL, then clear the pDestBuf.
================================================================================*/
boolean SetData(uint8* *pDestBuf, const uint8* pSrcBuf, uint32 Size)
{
    boolean Result = FALSE;

    if (pSrcBuf)
    {
        if (Size > 0) 
        {
            *pDestBuf = (uint8*)SYS_REALLOC(*pDestBuf, Size);
            if(*pDestBuf)
            {
                memcpy(*pDestBuf, pSrcBuf, Size);
                Result = TRUE;
            }
        }
    }
    else
    {
        SYS_FREE(*pDestBuf);
        *pDestBuf = NULL;
        Result = TRUE;
    }

    return (Result);
}

/*================================================================================
Helper function to enqueue MOIB_TIME struct
================================================================================*/
void Put_MOIBTIME(uint8** ppBuf, MOIB_TIME sDateTime)
{
    WriteInt16(ppBuf, sDateTime.year);
    WriteInt8(ppBuf, sDateTime.month);
    WriteInt8(ppBuf, sDateTime.day);
    WriteInt8(ppBuf, sDateTime.hour);
    WriteInt8(ppBuf, sDateTime.minute);
    WriteInt8(ppBuf, sDateTime.second);
    WriteInt8(ppBuf, sDateTime.format);
}

/*================================================================================
Helper function to dequeue MOIB_TIME struct
================================================================================*/
MOIB_TIME Get_MOIBTIME(uint8** ppBuf)
{
    MOIB_TIME sDateTime;

    sDateTime.year = ReadInt16(ppBuf);
    sDateTime.month = ReadInt8(ppBuf);
    sDateTime.day = ReadInt8(ppBuf);
    sDateTime.hour = ReadInt8(ppBuf);
    sDateTime.minute = ReadInt8(ppBuf);
    sDateTime.second = ReadInt8(ppBuf);
    sDateTime.format = ReadInt8(ppBuf);

    return sDateTime;
}

/*================================================================================
Helper function to copy data into a member
================================================================================*/
void CopyBlock(void* pDest, uint16 DestSize, const void* pSrc, uint16 SrcLength)
{
    /*Clean out the structure field*/
    memset(pDest, 0, DestSize);

    /*Check for Null pointer*/ 
    if(pSrc != NULL)
    {
        /*Check if there is enough room in the structure field*/
        if((sizeof(ansichar)*SrcLength) <= DestSize)
        {
            memmove(pDest, pSrc, (sizeof(ansichar)*SrcLength));
        }
    }
}

/*================================================================================
Helper function to copy a string into a member
================================================================================*/
void CopyString(ansichar * pDest, uint16 DestSize, const ansichar* pSrc, uint16 SrcLength)
{
    /*Clean out the structure field*/
    memset(pDest, 0, DestSize);
   
    /*Check for Null pointer*/
    if(pSrc != NULL)
    {
        /*Check if there is enough room in the structure field*/
        if((sizeof(ansichar)*SrcLength) < DestSize)
        {
            memmove(pDest, pSrc, (sizeof(ansichar)*SrcLength));
        }
    }
}
