/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// SysUtil.h      :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(SYSUTIL_H)
    #define  SYSUTIL_H

#include "Target.h"
#include "StdVarTyp.h"
#include <time.h>               /* For time_t */

#if defined(__cplusplus)
extern "C" {
#endif

/*
 * Abstraction for system utility setup
 */
void SysUtil_Setup(void);

#define SYS_SETUP  SysUtil_Setup


/*
 * Abstractions for C memory allocations functions
 *
 * void *malloc( size_t size );
 * void *calloc( size_t num, size_t size );
 * void free( void *memblock );
 * void *realloc( void *memblock, size_t size );
 */
typedef enum tagMemoryPool {
    GENERAL_POOL,
    OBJECT_POOL,
    BUFFER_POOL
} MEMORY_POOL;

#if defined(MEMORY_DEBUG)
    void *sys_malloc(MEMORY_POOL Pool, size_t Size, char *, int);
    void *sys_calloc(MEMORY_POOL Pool, size_t Num, size_t Size, char *, int);
    void sys_free(MEMORY_POOL Pool, void *pMemBlock, char *, int);
    void *sys_realloc(MEMORY_POOL Pool, void *pMemBlock, size_t Size, char *, int);

    #define DFLT_POOL   GENERAL_POOL
    #define SYS_MALLOC(A)       sys_malloc(DFLT_POOL, (A), __FILE__, __LINE__)
    #define SYS_CALLOC(A,B)     sys_calloc(DFLT_POOL, (A), (B), __FILE__, __LINE__)
    #define SYS_FREE(A)         sys_free(DFLT_POOL, (A), __FILE__, __LINE__)
    #define SYS_REALLOC(A,B)    sys_realloc(DFLT_POOL, (A), (B), __FILE__, __LINE__)

#else
    void *sys_malloc(MEMORY_POOL Pool, size_t Size);
    void *sys_calloc(MEMORY_POOL Pool, size_t Num, size_t Size);
    void sys_free(MEMORY_POOL Pool, void *pMemBlock);
    void *sys_realloc(MEMORY_POOL Pool, void *pMemBlock, size_t Size);

    #define DFLT_POOL   GENERAL_POOL
    #define SYS_MALLOC(A)       sys_malloc(DFLT_POOL, (A))
    #define SYS_CALLOC(A,B)     sys_calloc(DFLT_POOL, (A), (B))
    #define SYS_FREE(A)         sys_free(DFLT_POOL, (A))
    #define SYS_REALLOC(A,B)    sys_realloc(DFLT_POOL, (A), (B))
#endif


/*
 * Abstractions for Windows functions
 *
 * Get the process identification.
 * int _getpid(void);
 */
int sys_getpid(void);

#define DFLT_PID    1
#define SYS_GETPID  sys_getpid


/*
 * Abstraction for OS sleep/delay/pause functions.
 *
 * Suspend current process/thread/task for a specified time.
 */
void sys_sleep(unsigned long Msecs);

#define SYS_SLEEP(A)    sys_sleep(A)


/*
 * Abstraction for C time() function.
 *
 * Get system time.
 * time() returns the time in elapsed seconds. There is no error return.
 * time_t time( time_t *timer );
 */
time_t sys_time(time_t *pTimer);

#define SYS_TIME(A)     sys_time(A)


/*
 * Connection notification function.
 *
 * Notify the system of the connectivity state.
 */
typedef enum tagConnectionState {
    NO_CONNECT,
    LINK_CONNECT,
    SESSION_CONNECT
} CONNECTION_STATE;

void sys_connection_notify(CONNECTION_STATE state);

#define SYS_CONNECTION_NOTIFY(A)   sys_connection_notify(A)


/*
 * Abstractions for C pseudorandom number functions.
 *
 * void srand( unsigned int seed );
 * int rand( void );
 */

/*
 * Set a random starting point.
 */
void sys_srand(unsigned int Seed);

#define SYS_SRAND(A)    sys_srand(A)


/*
 * Generates and returns a pseudorandom number.
 * Note that the macro casts the returned value as a uint8.
 */
int sys_rand(void);

#define SYS_RAND        (uint8)sys_rand

/*
 * Abstraction for printf function
 */
int sys_printf(const char *pFormat, ...);

#define SYS_PRINTF       sys_printf

#if defined(TRACE_SERVER_ON)
    boolean sys_trace(uint16 TraceType, 
                  char* pClassName, ansichar* pFunctionName,
                  uint16 Family, uint8 Genus, uint8 Species, 
                  ansichar* pDescriptionFormat, ...);

    #define SYS_TRACE sys_trace

#endif

/*
 * Abstraction for Init function
 */
boolean sys_init();

#define SYS_INIT      sys_init

/*
 * Abstraction for Destroy function
 */
boolean sys_destroy();

#define SYS_DESTROY      sys_destroy

#if defined(__cplusplus)
}
#endif

#endif  /* #if !defined(SYSUTIL_H) */

/*---------------------------------------------------*/
/* End of File */
