/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// Wacp.c         :                                                  //
'**  '// Language       :   C                                              //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :												   //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "Wacp.h"
#include <string.h>
#include "Mem.h"
#include "MiscUtil.h"
#include "DtSupport.h"

#if defined(WACP_DEBUG)
#include "Sysutil.h"
#endif

extern boolean RecvMessage(const char *pSourceID, uint32 ClassID, uint8 *pBuffer, uint32 BufSize);
extern boolean RecvError(const char *pSourceID, uint8 ErrorType, uint32 ClassID, uint8 *pBuffer, uint32 BufSize);

/*================================================================================
GenerateMessage: Creates a byte stream with the message layer wrappings. 
    Calculated size is done based on the message type:
    
    Standard: ENVELOPE_HEADER_SIZE + TRAP_HEADER_SIZE + BufSize + CRCSIZE; 
    Trap: ENVELOPE_HEADER_SIZE + MESSAGE_HEADER_SIZE + BufSize + CRCSIZE;    

Parameters:
    pOutBuffer:     Pointer to a preallocated buffer used to write the generated
                    message. Note: This buffer must not be the same as pBuf.
    pOutSize:       Pointer to a uint32 OUT paramter that stores the size 
                    of the message generated.
    pBuf:           Pointer to a data stream to be added in generated messaged. 
                    This buffer can be NULL. Note: This buffer must not be the 
                    same as pOutBuffer.
    BufSize         Size of the buffer pointed to by pBuf. This size may be 0.
    MsgClassID      ClassID of the message.
    ErrClassID      ClassID of the Object that caused the error.

Returns: 

    WACP_SUCCESS - If the message was generated.
    WACP_PARAMETER_ERROR - If a invalid parameter was passed.

================================================================================*/
uint32 GeneratePacket(uint8* pOutBuffer, uint32* pOutSize, uint8* pBuf, 
                      uint32 BufSize, uint32 MsgClassID, uint32 ErrClassID)
{
    uint8* pTemp = pOutBuffer; 
    uint32 MsgSize = 0;

    /* Check for invalid parameters */
    if ((pOutBuffer == NULL) || (pOutSize == NULL))
    {
#if defined(WACP_DEBUG)
        SYS_PRINTF("\nGenerateMessage Invalid Parameter error\n");
#endif
        return(WACP_PARAMETER_ERROR);
    }
    
    if ((uint16)(MsgClassID >> 16) == FmTRAP)
    {
        *pOutSize = ENVELOPE_HEADER_SIZE + TRAP_HEADER_SIZE + BufSize + CRCSIZE;    
    }
    else
    {
        *pOutSize = ENVELOPE_HEADER_SIZE + MESSAGE_HEADER_SIZE + BufSize + CRCSIZE;    
    }

    /* Serialize the packet header struct */
    /* Set the PREAMBLE */    
    *pTemp++ = CTL_W;
    *pTemp++ = CTL_A;
    *pTemp++ = CTL_L;

    /* Write the Packet Length */
    WriteInt32(&pTemp, *pOutSize);

    /* Write the Port */
    WriteInt16(&pTemp, RNDZ_PORT_DEFAULT);

    if ((uint16)(MsgClassID >> 16) == FmTRAP)
    {
        /* Set the Size of the entire message. */
        MsgSize = TRAP_HEADER_SIZE + BufSize;

        /* Write the Trap MessageClassID */
        WriteInt32(&pTemp, MsgClassID);

        /* Write the Trap Static Struct size */
        WriteInt16(&pTemp,(uint16)(MSG_SMSIZE + MSG_CLASSID + MSG_PAYLOAD_SIZE + BufSize + CRCSIZE));

        /* Write the Trap Generating Message ID size */
        WriteInt16(&pTemp, sizeof(uint32));

        /* Write the Trap Error Class ID */
        WriteInt32(&pTemp, ErrClassID);
    }
    else
    {
        /* Set the Size of the entire message. */
        MsgSize = MESSAGE_HEADER_SIZE + BufSize;

        /* Write the message ClassID */
        WriteInt32(&pTemp, MsgClassID);

        /* Write the Message size */
        WriteInt32(&pTemp,(MSG_ENCRYPTION + MSG_PAYLOAD_SIZE + BufSize + CRCSIZE));

        /* Write the encryption field */
        *pTemp++ = 0;
    }

    /* Write the Message Payload Size */
    WriteInt32(&pTemp, BufSize);

    /* Copy the Objects/data to stream if there are any to copy */
    if (BufSize>0)
    {
        memmove((char*)pTemp, pBuf, BufSize);
        pTemp += BufSize;
    }
    
    /* Write the Message CRC */
    WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pOutBuffer + ENVELOPE_HEADER_SIZE, MsgSize - CRCSIZE));

    /* Write the Envelope CRC */
    WriteInt16(&pTemp, CalculateCRC(CRC_SEED, pOutBuffer, *pOutSize - CRCSIZE));

    return(WACP_SUCCESS);
}

/*================================================================================
DeserializeEnvelope: Deserialize the envelope header and leave the pointer 
                     pointing to the Message.

Parameters:
    ppBuf:          Pointer to a Pointer to the data stream
    BufSize:        Size of the data stream.
    pOutSize:       Pointer to a uint32 OUT paramter that stores the size 
                    of the remaining bytes in the buffer.

Returns: 

    WACP_SUCCESS - If the message was deserialized.
    WACP_CRC_ERROR - If a CRC error is found.
    WACP_PARAMETER_ERROR - If a invalid parameter was passed.

================================================================================*/
uint32 DeserializeEnvelope(uint8** ppBuf, uint32 BufSize, uint32* pOutSize)
{
    uint32 EnvelopeSize;
    uint16 Port;

    /* Check for valid parameters */
    if ((ppBuf == NULL) || (BufSize == 0) || (pOutSize == NULL))
    {
#if defined(WACP_DEBUG)
        SYS_PRINTF("\nDeserializeEnvelope Invalid Parameter error\n");
#endif
        return(WACP_PARAMETER_ERROR);
    }
        
    if (*ppBuf == NULL)
    {
#if defined(WACP_DEBUG)
        SYS_PRINTF("\nDeserializeEnvelope Invalid Parameter Error *pBuf == NULL\n");
#endif
        return(WACP_PARAMETER_ERROR);
    }

    /*Check the CRC*/
    if( !CalculateCRC(CRC_SEED, *ppBuf, BufSize - CRCSIZE) == ReadInt16Ext(*ppBuf + (BufSize - CRCSIZE)))
    {	
#if defined(WACP_DEBUG)
        SYS_PRINTF("\n\n\nEnvelope Integrity Check Failed!\n");
#endif
        return(WACP_CRC_ERROR);
    }

    /*Check the preamble and advance the pointer*/
    if ((*ppBuf)[0] == CTL_W && (*ppBuf)[1] == CTL_A && (*ppBuf)[2] == CTL_L)
    {		
        *ppBuf += PREAMBLE_SIZE;
    }
    else
    {
#if defined(WACP_DEBUG)
        SYS_PRINTF("\n\n\nEnvelope Preamble Check Failed!\n");
#endif
        return (WACP_PREAMBLE_ERROR);
    }

    /* Read the size of the envelope and adavance the pointer*/
    EnvelopeSize = 	ReadInt32(ppBuf);

    /*Read the application port and advance the pointer */
    Port = ReadInt16(ppBuf);

    /* Check for a valid port */
    if (Port != WACP_PORT_DEFAULT && Port != RNDZ_PORT_DEFAULT)
    {	
#if defined(WACP_DEBUG)
        SYS_PRINTF("Invalid Port\n");
#endif
        return(WACP_PORT_ERROR);
    }

    /*At this Point we are pointing at the start of the Message.
    Calculate the pOutSize which represents the size of the message PDU*/
    *pOutSize = EnvelopeSize-(PREAMBLE_SIZE + ENVELOPE_SIZE + ENVELOPE_PORT + CRCSIZE); 

    return(WACP_SUCCESS);
}

/*================================================================================
MsgHandler: Parses the Message layer wrapping and routes to the appropriate 
    message type routine.

Parameters:
    pSourceID:      SourceID of the incoming message.
    ppBuf:          Pointer to a Pointer to the data stream
    BufSize:        Size of the data stream.

Returns: 

    WACP_SUCCESS - If the message was properly parsed and handled.
    WACP_CRC_ERROR - If a CRC error is found.
    WACP_PARAMETER_ERROR - If a invalid parameter was passed.

================================================================================*/
uint32 MsgHandler(char* pSourceID, uint8** ppBuf, uint32 BufSize)
{	uint32 PayloadSize;
    uint16 MsgType;
    uint32 MsgClassID = 0;
    uint32 ErrClassID = 0;

    /* Check for valid parameters */
    if ((ppBuf == NULL) || (BufSize == 0) || pSourceID == NULL)
    {
#if defined(WACP_DEBUG)
        SYS_PRINTF("\nMsgHandler Invalid Parameter Error\n");
#endif
        return(WACP_PARAMETER_ERROR);
    }

    if (*ppBuf == NULL)
    {
#if defined(WACP_DEBUG)
        SYS_PRINTF("\nMsgHandler Invalid Parameter Error *ppBuf == NULL\n");
#endif
        return(WACP_PARAMETER_ERROR);
    }
        
    /*Check the CRC*/
    if(!CalculateCRC(CRC_SEED, *ppBuf, BufSize - CRCSIZE) == 
        ReadInt16Ext(*ppBuf + (BufSize - CRCSIZE)))
    {	
#if defined(WACP_DEBUG)
        SYS_PRINTF("\nDeserializeMessage CRC Check Failed\n");
#endif
        return(WACP_CRC_ERROR);
    }
    
    /*Get the MsgClassID */
    MsgClassID = ReadInt32(ppBuf);

    MsgType = (uint16)(MsgClassID >> 16);
    if (MsgType == FmTRAP)
    {
        /* Go past the payload size */
        *ppBuf += MSG_SMSIZE;
        
        /* Go past the static struct size */
        *ppBuf += MSG_SMSIZE;
        
        /* Get the ErrClassID */
        ErrClassID = ReadInt32(ppBuf);
        
        /* Get the Payload size */
        PayloadSize = ReadInt32(ppBuf);
        
        /* Notify the application of the incoming message */
        RecvError(pSourceID, (uint8)MsgClassID, ErrClassID, *ppBuf, PayloadSize);
    }
    else
    {
        /* Go past the payload size */
        *ppBuf += MSG_LGSIZE;
        
        /* Go past the encryption byte */
        *ppBuf += MSG_ENCRYPTION;

        /* Get the Payload size */
        PayloadSize = ReadInt32(ppBuf);

        /* Notify the application of the incoming message */
        RecvMessage(pSourceID, MsgClassID, *ppBuf, PayloadSize);
    }

    return(WACP_SUCCESS);
}

/*================================================================================
WACPParser: Parses a data stream for a WACP packets. This will process
                     all WACP packets contained in the data stream.
Parameters:
    pSourceID:      SourceID of the incoming message.
    pBuf:           Pointer to the data stream
    BufSize:        Size of the data stream.
    pPacketLength:  Pointer to a value that will return the packet length
                    for the current WACP packet.

Returns: The number of bytes processed. 
    0, if a partial WACP packet was not found.
    BufSize, if a WACP packet was not found.

    If the WACP header is found *pPacketLength is set to the current packet length.
    Otherwise, the Packetlength is set to 0.
================================================================================*/
uint32 WACPParser(char* pSourceID, uint8* pBuf, uint32 BufSize, 
                  uint32 *pPacketLength)
{
    uint32      BytesProcessed = 0;
    uint32      PacketLen;
    uint8*      pTemp = pBuf;
    uint32      EnvSize = 0;
    uint32      Result = 0;

    /* Initialize the PacketLength pointer to 0 */
    if (pPacketLength != NULL)
    {
        *pPacketLength = 0;
    }

    for (;;)
    {
        /* Only process messages that contain a full message header */
        if (BufSize > PACKET_HEADER)
        {
            /* Verify the Preamble */
            if( (pBuf[0] == CTL_W) && 
                (pBuf[1] == CTL_A) && 
                (pBuf[2] == CTL_L) )
            {
                /* Get the packet length */
                pTemp = (uint8*)&pBuf[PREAMBLE_SIZE];
                PacketLen = (int)ReadInt32Ext(pTemp);

                /* Check to see if the buffer size is greater than or equal to the 
                 * PacketLength and that we don't exceed the max buffer size.
                 */
                if (PacketLen <= BufSize)
                {
                    Result = DeserializeEnvelope(&pBuf, PacketLen, &EnvSize);
                    if (Result == WACP_SUCCESS)
                    {
                        Result = MsgHandler(pSourceID, &pBuf, EnvSize);
#if defined(WACP_DEBUG)
                        if (Result != WACP_SUCCESS)
                        {
                            SYS_PRINTF("DeserializeMessage ret. ERROR %d\n", Result);
                        }
#endif
                    }
#if defined(WACP_DEBUG)
                    else
                    {
                        SYS_PRINTF("DeserializeEnvelope ret. ERROR %d\n", Result);
                    }
#endif
                    /* Increment the pBuf pointer to the next byte after the WACP packet */
                    pBuf += PacketLen;

                    /* Update the size of the remaining data stream */
                    BufSize -= PacketLen;

                    /* Update the number of bytes parsed */
                    BytesProcessed += PacketLen;
                    continue;
                }
                else
                {
                    if (pPacketLength != NULL)
                    {
                        /* Update the number of bytes of the current packet */
                        *pPacketLength = PacketLen;
                    }
#if defined(WACP_DEBUG_EXT)
                    SYS_PRINTF("Partial WACP Packet found: Current Packet length = %d\n", PacketLen);
                    SYS_PRINTF("BufSize = %d, BytesProcessed = %d\n", BufSize, BytesProcessed);
#endif
                }
                break;
            }
            else
            {
                if (BufSize > PREAMBLE_SIZE)
                {
#if defined(WACP_DEBUG)
                    SYS_PRINTF("Found invalid preamble, checking next byte\n");
#endif
                    /* Update the number of bytes parsed */
                    BytesProcessed++;

                    /* Update the number of bytes left to parsed */
                    BufSize--;

                    /* increment the pointer */
                    pBuf++;
                }
                else
                {
#if defined(WACP_DEBUG)
                    SYS_PRINTF("No valid PREAMBLE found\n");
#endif
                    break;
                }
            }

        }/* if (MsgSize > MSG_HEADER) */
        else
        {
#if defined(WACP_DEBUG_EXT)
            SYS_PRINTF("BufSize = %d, BytesProcessed = %d\n", BufSize, BytesProcessed);
#endif
            break;
        }

    }/* for (;;) */


    return BytesProcessed;
}


