/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CEthCommSvr.cpp:                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CEthCommSvr.h"
#include "CCommSvr.h"

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor

*********************************************************************************/
CEthCommSvr::CEthCommSvr(boolean (*RecvMsg)(char* pSemName, char* pSourceID, 
                         uint8* pBuf, uint32 MsgSize, void* pNewConnection),                   
                         boolean (*TerminateClient)(char* pSemName, char* pSourceID, uint16 Port), 
                         char* pAppName)
                  
{
    /* Store the callback function to add a new message */
    pSetRecvMsg = RecvMsg;
    pTerminate = TerminateClient;
    
    /* Save the application name that is created from the Semaphore name.
     * This is used to differentiate between separate dll instances of the stack.
     */
    pSemName = pAppName;

    /* Zero out structure */
    memset(SourceID, 0, sizeof(SourceID));         
    
    /* Initial the socket handles */
    TxSocket = INVALID_SOCKET;
    RxSocket = INVALID_SOCKET;
    
    /* Default to TCP */
    Protocol = IPPROTO_TCP;

    /* Initialize the Port */
    Port = 0;

    /* Initialize the Port used by auto-discovery */
    AutoPort = 0;

    pUDPMutex = NULL;

    /* Set the stack type */
    StackType = ETHERNET;

    /* Initialize the WIN32 dll flag */
    WSAStartFlag = FALSE;
}

CEthCommSvr::~CEthCommSvr()
{
    if (Protocol == IPPROTO_UDP)
    {
        if (pUDPMutex != NULL)
        {
            /* Close the sockets if we can get the UDPMutex lock (no listeners) */
            if (pUDPMutex->Lock(1) == WAIT_SUCCESS)
            {
                pUDPMutex->Release();
                if (TxSocket != INVALID_SOCKET)
                {
                    closesocket(TxSocket);
                }

                if (RxSocket != INVALID_SOCKET)
                {
                    closesocket(RxSocket);
                }
                /* Destroy the Mutexes */
                if (pUDPMutex != NULL)
                {
                    delete(pUDPMutex);
                    pUDPMutex = NULL;
                }
            }
        }
    }
    else
    {
        if (TxSocket != INVALID_SOCKET)
        {
            closesocket(TxSocket);
        }

        if (RxSocket != INVALID_SOCKET)
        {
            closesocket(RxSocket);
        }
    }

    TxSocket = INVALID_SOCKET;
    RxSocket = INVALID_SOCKET;

    /* Wait for the CEthCommSvr threads to complete before completing the destructor */
    ClientExitMutex.Lock(INFINITE);
    ClientExitMutex.Release();


    if (WSAStartFlag == TRUE)
    {
        WSACleanup( );
    }
}

/*================================================================================
CEthCommSvr GetSourceID: Returns the SourceID of the object
================================================================================*/
char* CEthCommSvr::GetSourceID()
{
    return((char*)SourceID);
}
/*================================================================================
CEthCommSvr SetSourceID: Sets the SourceID
================================================================================*/
boolean CEthCommSvr::SetSourceID(char* pSourceID)
{   
    boolean Result = FALSE;
    if (pSourceID != NULL)
    {
        uint16 Size;

        memset(SourceID, 0, sizeof(SourceID));
        Size = (uint16)strlen(pSourceID);
        if (Size > sizeof(SourceID))
            Size = sizeof(SourceID);

        memcpy(SourceID, pSourceID, Size);
        Result = TRUE;
    }

    return (Result);
}
/*================================================================================
CEthCommSvr GetPortNum: Returns the Port used by the object
================================================================================*/
uint16 CEthCommSvr::GetPortNum()
{
    return(Port);
}

/*================================================================================
CEthCommSvr SetPortNum: Sets the Port used by the object
================================================================================*/
boolean CEthCommSvr::SetPortNum(uint16 NewPort)
{
    boolean Result = TRUE;

    Port = NewPort;

    return (Result);
}

/*================================================================================
CEthCommSvr SetAutoPort: Sets the Port used by the object to perform auto discovery.
================================================================================*/
void CEthCommSvr::SetAutoPort(uint16 NewPort)
{
    AutoPort = NewPort;
}

/*================================================================================
CEthCommSvr SetConfig: Updates the addressing information of the object
================================================================================*/
void CEthCommSvr::SetConfig(void* pAddr)
{
    EthAddr* pConfig = (EthAddr*)pAddr;
    if (pConfig != NULL)
    {
        Port = pConfig->Port;
        Protocol = pConfig->Protocol;

        /* In the case of switching from RNDZ to WACP, always close the 
         * socket to force the StartClient thread to terminate
         */
        if (TxSocket != INVALID_SOCKET)
        {
            closesocket(TxSocket);
        }

        TxSocket = pConfig->Socket;
        SetSourceID(pConfig->pSourceID);
        memcpy(&(Addr.ClntAddr), &pConfig->ClntAddr, sizeof(pConfig->ClntAddr));
    }
}

/*================================================================================
CEthCommSvr GetConfig: Returns the configuration information
================================================================================*/
void* CEthCommSvr::GetConfig()
{
    return((void*)(&Addr));
}

/*================================================================================
CEthCommSvr GetStackType: Returns the type of stack that is being used ( i.e. Ethernet, 
                     RS232, Bluetooth, Zigby, etc
================================================================================*/
STACK_TYPE CEthCommSvr::GetStackType()
{
    return(StackType);
}

/*================================================================================
CEthCommSvr SetProtocol: Sets the protocol to either IPPROTO_TCP or IPPROTO_UDP.
================================================================================*/
boolean CEthCommSvr::SetProtocol(uint16 NewProtocol)
{
    boolean Result = FALSE;

    if ((NewProtocol == IPPROTO_TCP) || (NewProtocol == IPPROTO_UDP))
    {
        Protocol = NewProtocol;
        Result = TRUE;
    }

    return (Result);
}

/*================================================================================
CEthCommSvr Connect: Creates a socket that connects to the passed in SourceID and Port.
================================================================================*/
boolean CEthCommSvr::Connect(uint8* pSourceID, int PortNum)
{
    WSAData WSAData;
    boolean Result = FALSE;
    struct sockaddr_in  DestAddr;
    struct linger   linger;         /* amount of time to SO_LINGER */
    linger.l_onoff = 1;             /* Turn ON the SO_LINGER option */
    linger.l_linger = 0;            /* Use default amount of time to shutdown i.e
                                       the default value of TCP_LINGERTIME in tcp_timer.h*/ 

    /*** BEGIN WIN32 API ***/
    if (WSAStartup(MAKEWORD(2,1),&WSAData) == 0)
    {
        WSAStartFlag = TRUE;
    }
#if defined(TRACE_SERVER_ON)
    else
    {
        SYS_PRINTF("StartServer: WSAStartup failed: %d\n", WSAGetLastError());
    }
#endif

    /*** END WIN32 API ***/

    /* Construct the server Address structure */
    memset(&DestAddr, 0, sizeof(DestAddr));             /* Zero out structure */
    DestAddr.sin_family      = AF_INET;                 /* Internet Address family */
    DestAddr.sin_addr.s_addr = inet_addr((char*)pSourceID); /* Server IP Address */
    DestAddr.sin_port        = htons((u_short)PortNum); /* Server Port */

    if (Protocol == IPPROTO_TCP)
    {
        /* Create a reliable, Stream socket using TCP */
        TxSocket = socket(PF_INET, SOCK_STREAM, Protocol);
        if (TxSocket >= 0)
        {
            /* Establish the connection to the server */
            if (connect(TxSocket, (struct sockaddr *) &DestAddr, sizeof(DestAddr)) >= 0)
            {
                /* Specify the SO_LINGER option to perform a  "graceful" close.
                * A graceful close occurs when a connection is shutdown, TCP  will  
                * try  to  make sure that all the unacknowledged data in transmission 
                * channel are acknowledged and the peer is shutdown properly by going 
                * through an elaborate  set  of  state transitions. 
                */
                setsockopt (TxSocket, SOL_SOCKET, SO_LINGER, (char* ) &linger, sizeof (linger));

                Result = TRUE;
            }
#if defined(TRACE_SERVER_ON)
            else
            {
                SYS_PRINTF("ERROR!!! Unable to connect %d\n", WSAGetLastError());
            }
#endif
        }/* if (TxSocket >= 0) */

#if defined(TRACE_SERVER_ON)
        else
        {
            SYS_PRINTF("ERROR!!! Unable to create a TCP socket %d\n", WSAGetLastError());
        }
#endif
    }
    else if (Protocol == IPPROTO_UDP)
    {
        /* Create a reliable, Stream socket using TCP */
        RxSocket = TxSocket = socket(PF_INET, SOCK_DGRAM, Protocol);

        if (TxSocket >= 0)
        {
            Addr.Port = (uint16)PortNum;
            Addr.Protocol = Protocol;
            Addr.Socket = TxSocket;
            Addr.pSourceID = (char*)pSourceID;
            memcpy(&(Addr.ClntAddr), &DestAddr, sizeof(DestAddr));
            Result = TRUE;
        }
#if defined(TRACE_SERVER_ON)
        else
        {
            SYS_PRINTF("ERROR!!! Unable to create UDP socket\n");
        }
#endif
    }

    /* Update the connection data */
    Port = (uint16)PortNum;

    /* Set the destination IP to be the source */
    SetSourceID((char*)pSourceID);

#if defined(DEBUG_PRINT_STATUS)
    if (Result == TRUE)
    {
        SYS_PRINTF("Connection open to %s to Port %d\n", 
                inet_ntoa(DestAddr.sin_addr), PortNum);
    }
    else
    {
        SYS_PRINTF("Unable to connect to %s to Port %d\n",
                inet_ntoa(DestAddr.sin_addr), PortNum);
    }
#endif

    return Result;

}

/*================================================================================
CEthCommSvr SendMsg: Uses the current socket descriptor to send a message.
================================================================================*/
boolean CEthCommSvr::SendMsg( uint8* pBuf, uint32 pBufSize)
{
    boolean RetVal = TRUE;
    int Bytes;

    /* Check to see if the StartClient thread has started
     * If not, then wait until it starts before sending
     */
    if (!ClientExitMutex.Lock(1))
    {
        ClientExitMutex.Release();
        SYS_SLEEP(1);
    }

    /* Send the string to the server */
    if ( TxSocket != INVALID_SOCKET)
    {
        if (Protocol == IPPROTO_TCP)
        {
            Bytes = send(TxSocket, (char*)pBuf, pBufSize, 0);
        }
        else
        {
            Bytes = sendto(TxSocket, (char*)pBuf, pBufSize, 0, (struct sockaddr *) &(Addr.ClntAddr), 
                            sizeof(Addr.ClntAddr));
        }

        if (Bytes != (int)pBufSize)
        {
            RetVal = FALSE;
#if defined(DEBUG_PRINT)
            SYS_PRINTF("\n\nERROR!!! SendMessage: %d Bytes were sent out of %d\n\n", 
                    Bytes, pBufSize);
#endif
        }
#if defined(DEBUG_PRINT)
        else
        {
            SYS_PRINTF("SendMessage: Sent %d Bytes\n", Bytes);
            for (uint32 i=0; i < pBufSize;i++)
            {
                if ( i % 25 == 0)
                    SYS_PRINTF("\n");
                SYS_PRINTF("%.2x ", (uint8)pBuf[i]);
            }
            SYS_PRINTF("\n\n");
        }
#endif  
    }
    else
    {
        RetVal = FALSE;
    }
    
    return RetVal;
}   

/*================================================================================
Static CEthCommSvr StartClient: Handles the start of the new client connection thread.
    The function loops until the full Packet length is received before adding the
    message into the link layer IN_queue. The loop ends when the call to recv() 
    returns a -1 to signify the end of the socket connection .
================================================================================*/
uint32 WACPAPI CEthCommSvr::StartClient(void* pData)
{
    CEthCommSvr* pCommSvr = (CEthCommSvr*)pData;
    int MsgSize, BytesLeft, PacketLen;
    char Buffer[RCVBUFSIZE];

#if defined(DEBUG_PRINT)
    int CommMsgCount = 0;

    SYS_PRINTF("CEthCommSvr StartClient: Thread starting %p\n", pCommSvr);
#endif

    /* Lock this mutex until this thread terminates */
    pCommSvr->ClientExitMutex.Lock(INFINITE);

    if (pCommSvr->Protocol == IPPROTO_TCP)
    {
        /* Loop while there is data available */
        do
        {
            /* Receive the next message from client */
            MsgSize = recv(pCommSvr->TxSocket, Buffer, MSG_HEADER, 0);

            if (MsgSize == MSG_HEADER)
            {
                /* Verify the Preamble */
                if( (Buffer[0] == CTL_W) && 
                    (Buffer[1] == CTL_A) && 
                    ((Buffer[2] == CTL_C) || (Buffer[2] == CTL_L)) )
                {

                    PacketLen = ReadInt32((uint8*)&Buffer[PREAMBLE_SIZE]);
                    if (PacketLen < RCVBUFSIZE)
                    {
#if defined(DEBUG_PRINT)
                        SYS_PRINTF("\n\nCommSvr %p: New Msg size = %d\n", pCommSvr, PacketLen);
#endif

                        /* Calculate the remaining Bytes to receive */
                        BytesLeft = PacketLen - MSG_HEADER;

                        /* Receive the rest of the Packet */
                        MsgSize += recv(pCommSvr->TxSocket, &Buffer[MsgSize], BytesLeft, 0);
                        
                        /* Loop until we get the full Packet */
                        while ((MsgSize > 0) && (MsgSize < (int)PacketLen))
                        {
                            int Bytes;

                            Bytes = recv(pCommSvr->TxSocket, &Buffer[MsgSize], PacketLen - MsgSize, 0);

                            /* If Bytes <= 0 then the connection has been closed */
                            if (Bytes > 0)
                            {
                                MsgSize += Bytes;
                            }
                            else
                            {
                                MsgSize = Bytes;
                            }
#if defined(DEBUG_PRINT)
                            SYS_PRINTF("\n\n1.CommSvr %p: CommMsgCount = %d,MsgSize = %d\n\n", pCommSvr, CommMsgCount, MsgSize);
#endif
                        }

                        /* Check for a valid MsgSize before sending it to the link layer */
                        if ( MsgSize > 0)
                        {
#if defined(DEBUG_PRINT)
                            SYS_PRINTF("\n\nCommSvr %p: CommMsgCount = %d,MsgSize = %d\n\n", pCommSvr, ++CommMsgCount, MsgSize);
                            SYS_PRINTF("\n\nNew Msg:\n");
                            for (int i=0; i < MsgSize;i++)
                            {
                                if ( i % 25 == 0)
                                    SYS_PRINTF("\n");
                                SYS_PRINTF("%.2x ", (uint8)Buffer[i]);
                            }
                            SYS_PRINTF("\n\n");
#endif
                            /* Update the Recv IN_queue with the new message */
                            pCommSvr->pSetRecvMsg(pCommSvr->pSemName, pCommSvr->GetSourceID(), (uint8*)Buffer, MsgSize, NULL);
                        }

                    }/* if (PacketLen < RCVBUFSIZE) */

                }/* if( (Buffer[0] == CTL_W) && .. */
                else
                {
#if defined(TRACE_SERVER_ON)
                    SYS_PRINTF("ERROR!!! Invalid Preamble\n");
#endif
                }
            }

        }while (MsgSize > 0);

        SYS_SLEEP(1);

        /* Check to see if this was a normal termination or break in the communication */
        if (pCommSvr->TxSocket != INVALID_SOCKET)
        {
            pCommSvr->pTerminate(pCommSvr->pSemName, pCommSvr->GetSourceID(), pCommSvr->GetPortNum());
        }

    }/* if (Protocol == IPPROTO_TCP) */
    else if (pCommSvr->Protocol == IPPROTO_UDP)
    {
        char EventBuffer[EVENT_BUF_SIZE];
        uint32  MutexResult;

        /* See if we can get a lock on the UDP port mutex 
         * If the result is a timeout, then we already
         * have a server that is reading data from the 
         * port. This resolves the issue where TCP requires
         * both the server and client versions of the CommSvr
         * to read from individiual sockets, whereas UDP requires
         * only a single reader and multiple writers to a single
         * socket. On client only configurations, we need the 
         * StartClient thread to read from the data. On server
         * configuration, we need the server listeners to read
         * from the port and relay the message to the link layer.
         */
        if (pCommSvr->pUDPMutex == NULL)
        {
            sprintf(EventBuffer,UDP_MUTEX_NAME, SYS_GETPID());
            pCommSvr->pUDPMutex = new CWACPMutex(EventBuffer);
        }

        if (pCommSvr->pUDPMutex != NULL)
        {
            MutexResult = pCommSvr->pUDPMutex->Lock(1);
            if (MutexResult == WAIT_SUCCESS)
            {
                pCommSvr->pUDPMutex->Release();
                for (;;) 
                {
                    if ((MsgSize = recvfrom(pCommSvr->TxSocket, Buffer, RCVBUFSIZE, 0,
                                            NULL, NULL)) > 0)
                    {
                        /* Check for a valid PREAMBLE */
                        if( (Buffer[0] == CTL_W) && 
                            (Buffer[1] == CTL_A) && 
                            ((Buffer[2] == CTL_C) || (Buffer[2] == CTL_L)) )
                        {
#if defined(DEBUG_PRINT)
                                SYS_PRINTF("\n\nCommSvr %p: CommMsgCount = %d,MsgSize = %d\n\n", pCommSvr, ++CommMsgCount, MsgSize);
                                SYS_PRINTF("\n\nNew Msg:\n");
                                for (int i=0; i < MsgSize;i++)
                                {
                                    if ( i % 25 == 0)
                                        SYS_PRINTF("\n");
                                    SYS_PRINTF("%.2x ", (uint8)Buffer[i]);
                                }
                                SYS_PRINTF("\n\n");
#endif
                            /* Call the callback routine to create a new Comm Session */
                            pCommSvr->pSetRecvMsg(pCommSvr->pSemName, pCommSvr->GetSourceID(), (uint8*)Buffer, MsgSize, NULL);
                        }
                    }

                    if (pCommSvr->TxSocket == INVALID_SOCKET)
                        break;

                    if (MsgSize < 0) 
                    {
                        int LastError = WSAGetLastError();

#if defined(DEBUG_PRINT)
                        SYS_PRINTF("StartClient: WSAGetLastError = %d\n", LastError);
#endif
                        /* recvFrom can return an error if the socket has not been binded.
                         * The socket becomes binded in UDP either by explicitly calling bind() (not suggested).
                         * or via a message sent by sendto(). The StartClient thread starts reading 
                         * before the first RNDZ msg is sent so we have to check for this condition
                         */
                        if ((MsgSize == -1) && ((LastError == 0) || (LastError == WSAEINVAL) || (LastError == WSAEINTR)))
                        {
                            SYS_SLEEP(WAIT_COUNT);
                        }
                        else
                            break;
                    }
                }
            }/* if (MutexResult == WAIT_SUCCESS) */
            else
                delete(pCommSvr->pUDPMutex);
        
        }/* if (pCommSvr->pUDPMutex != NULL) */

    }/*else if (Protocol == IPPROTO_UDP) */

    /* Release the exit mutex */
    pCommSvr->ClientExitMutex.Release();

#if defined(DEBUG_PRINT)
    SYS_PRINTF("CEthCommSvr StartClient: Thread exiting %p\n", pCommSvr);
#endif

    return TRUE;
}

/*================================================================================
CEthCommSvr StartServer: Listens for new socket connections.
================================================================================*/
uint32 WACPAPI CEthCommSvr::StartServer(void* pData)
{
    boolean Result = TRUE;
    char*   pSourceID = NULL;
    char    RecvBuffer[RCVBUFSIZE];         
    char    EventBuffer[EVENT_BUF_SIZE];
    char    SourceBuffer[ADDR_BUF_SIZE];
    int     ClntLen;                     
    int     ExitResult;
    int     ReadSocket = 0;
    EthAddr NewClntAddr;
    CEthCommSvr*    pCommSvr = (CEthCommSvr*)pData;
    CWACPEvent*     pExitEvent;
    struct sockaddr_in ServAddr;                /* Server Address */
    struct sockaddr_in ClntAddr;                /* Client Address */
    struct  timeval Timeout ={ETH_TIMEOUT,0};   /* Timeout for select */
    fd_set  Socks;
    WSAData WSAData;
    SessionArgs NewSessionArgs;


#if defined(DEBUG_PRINT)
    SYS_PRINTF("StartServer: Starting\n");
#endif

    /* Exit if we do not have a valid pointer to a CEthCommSvr instance */
    if (!pCommSvr)
        return FALSE;

    /* Clear the EvenBuffer */
    memset(EventBuffer, 0, EVENT_BUF_SIZE);
    
    /* Create the event to check for termination of the server */
    sprintf(EventBuffer, END_SERVER, SYS_GETPID());
    pExitEvent = new CWACPEvent(EventBuffer, TRUE);

    /* Initiates the use of WS2_32.DLL */
    if (WSAStartup(MAKEWORD(2,1),&WSAData) == 0)
    {
        pCommSvr->WSAStartFlag = TRUE;
    }
#if defined(TRACE_SERVER_ON)
    else
    {
        SYS_PRINTF("StartServer: WSAStartup failed: %d\n", WSAGetLastError());
    }
#endif
    /*** END WIN32 API ***/

    /* Construct local Address structure */
    memset(&ServAddr, 0, sizeof(ServAddr));             /* Zero out structure */
    ServAddr.sin_family      = AF_INET;                 /* Internet Address family */
    ServAddr.sin_addr.s_addr = htonl(INADDR_ANY);       /* Any incoming interface */
    ServAddr.sin_port        = htons((u_short)pCommSvr->Port);/* Local Port */

    if (pCommSvr->Protocol == IPPROTO_TCP)
    {
        /* Create socket for incoming connections */
        if ((pCommSvr->RxSocket = socket(PF_INET, SOCK_STREAM, pCommSvr->Protocol)) >= 0)
        {
            /* Bind to the local Address */
            if (bind(pCommSvr->RxSocket, (struct sockaddr *) &ServAddr, sizeof(ServAddr)) >= 0)
            {
                /* Listen for any socket connection...TODO: define MAXPENDING */
                if (listen(pCommSvr->RxSocket, MAX_CONNECTIONS) >= 0)
                {

#if defined(DEBUG_PRINT)
                    /* TxSocket is connected to a client! */
                    SYS_PRINTF("\nStartServer: Waiting for a client to connect\n");
#endif
                    /* Run until we receive a close event */
                    for (;;) 
                    {
                        ReadSocket = 0;
                        while (ReadSocket == 0)
                        {
                            /* Check for raised ExitEvent. If set, then 
                             * end this thread by exiting the for loop
                             */
                            ExitResult = pExitEvent->Wait(1);
                            if (ExitResult == WAIT_SUCCESS)
                                break;

                            Socks.fd_array[0] = pCommSvr->RxSocket;
                            Socks.fd_count = 1;
                            ReadSocket = select(1, &Socks, (fd_set *) 0,(fd_set *) 0, &Timeout);
                        }

                        if (ReadSocket == 0)
                        {
#if defined(DEBUG_PRINT)
                            SYS_PRINTF("StartServer: Found exit event %d\n", WSAGetLastError());
#endif
                            closesocket(pCommSvr->TxSocket);
                            closesocket(pCommSvr->RxSocket);
                            break;
                        }

                        /* Set the size of the in-out parameter */
                        ClntLen = sizeof(ClntAddr);

                        /* Wait for a client to connect */
                        if ((pCommSvr->TxSocket = accept(pCommSvr->RxSocket, (struct sockaddr *) &ClntAddr, (int*)&ClntLen)) != INVALID_SOCKET)
                        {  
                            struct linger   Linger;         /* amount of time to SO_LINGER */
                            Linger.l_onoff = 1;             /* Turn ON the SO_LINGER option */
                            Linger.l_linger = 0;            /* Use default amount of time to shutdown i.e */

                            /* Specify the SO_LINGER option to perform a  "graceful" close */
                            setsockopt (pCommSvr->TxSocket, SOL_SOCKET, SO_LINGER, (char* ) &Linger, sizeof (Linger));

                            /* Set the Application Port number */
                            NewClntAddr.Port = pCommSvr->Port;

                            /* Set the Protocol */
                            NewClntAddr.Protocol = pCommSvr->Protocol;

                            /* Set the Socket file handle */
                            NewClntAddr.Socket = pCommSvr->TxSocket;

                            /* Set the SourceID */
                            pSourceID = inet_ntoa(ClntAddr.sin_addr);
                            memcpy(&(NewClntAddr.ClntAddr), &ClntAddr , sizeof(ClntAddr));
                            sprintf(SourceBuffer, "%s_%d", pSourceID, ClntAddr.sin_port);
                            NewClntAddr.pSourceID = SourceBuffer;

#if defined(TRACE_SERVER_ON)
                            /* TxSocket is connected to a client! */
                            SYS_PRINTF("\nStartServer: Handling client %s on Port %d AppPort %d Socket %d\n\n",
                                    pSourceID, pCommSvr->Port, ClntAddr.sin_port, pCommSvr->TxSocket);
#endif                                  
                            NewSessionArgs.Port = pCommSvr->Port;
                            NewSessionArgs.pSourceID = SourceBuffer;
                            NewSessionArgs.StackType = ETHERNET;
                            NewSessionArgs.bByPass= FALSE;
                            NewSessionArgs.pAddr = &NewClntAddr;

                            /* Call the callback routine to create a new Comm Session */
                            Result = pCommSvr->pSetRecvMsg(pCommSvr->pSemName, SourceBuffer, NULL, 0, (void*)&NewSessionArgs);
                            if (Result == FALSE)
                            {
                                closesocket(pCommSvr->TxSocket);
                            }
                            else if (pCommSvr->Port == pCommSvr->AutoPort)/* Autodiscovery */
                            {
                                uint32  PacketLen;
                                uint8*  pPacketBuf;

                                const uint8 RndzConnect[] = /* Faked WACONNECT message */
                                    {   0x00,0x1D,0x0B,0x01,0x00,0x00,0x00,0x12,
                                        0x00,0x00,0x00,0x00,0x0B,0x52,0x4E,0x44,
                                        0x5A,0x43,0x4F,0x4E,0x4E,0x45,0x43,0x54,
                                        0x57,0xF3 };

                                /* Generate a fake connection request for autodiscovery */
                                pPacketBuf = GeneratePacket(&PacketLen, pCommSvr->Port,0,0, (uint8*)RndzConnect, sizeof(RndzConnect));
                                pCommSvr->pSetRecvMsg(pCommSvr->pSemName, SourceBuffer, (uint8*)pPacketBuf, PacketLen, NULL);
                                
                                /* Free up the memory allocated by GeneratePacket */
                                SYS_FREE(pPacketBuf);
                            }

                        }/* if ((pCommSvr->TxSocket = accept(...) */

                    }/* for(;;) */
                }
#if defined(TRACE_SERVER_ON)
                else
                {
                    SYS_PRINTF("ERROR!!! StartServer: Failed to listen for a new socket: %d\n", WSAGetLastError());
                }
#endif
            }/* if (bind(...) >= 0) */
#if defined(TRACE_SERVER_ON)
            else
            {
                SYS_PRINTF("ERROR!!! StartServer: Failed to bind socket: %d\n", WSAGetLastError());
            }
#endif
        }/* if ((RxSocket = socket(PF_INET, SOCK_STREAM, Protocol)) >= 0) */
#if defined(TRACE_SERVER_ON)
        else
        {
            SYS_PRINTF("ERROR!!! StartServer: Failed to create a new socket: %d\n", WSAGetLastError());
        }
#endif
    }
    else if (pCommSvr->Protocol == IPPROTO_UDP)
    {
        int     MsgSize;

        /* Create the lock for accessing the port for UDP messages */
        if (pCommSvr->pUDPMutex == NULL)
        {
            sprintf(EventBuffer,UDP_MUTEX_NAME, SYS_GETPID());
            pCommSvr->pUDPMutex = new CWACPMutex(EventBuffer);
        }

        if (pCommSvr->pUDPMutex != NULL)
        {
            pCommSvr->pUDPMutex->Lock(INFINITE);

            /* Create socket for incoming connections */
            if ((pCommSvr->RxSocket = socket(PF_INET, SOCK_DGRAM, pCommSvr->Protocol)) >= 0)
            {
                /* Bind to the local Address */
                if (bind(pCommSvr->RxSocket, (struct sockaddr *) &ServAddr, sizeof(ServAddr)) >= 0)
                {

                    NewClntAddr.Port = pCommSvr->Port;
                    NewClntAddr.Protocol = pCommSvr->Protocol;
                    NewClntAddr.Socket = pCommSvr->RxSocket;

                    /* Loop until we get a termination event */
                    for(;;) 
                    {
                        ReadSocket = 0;
                        while (ReadSocket == 0)
                        {
                            /* Check for raised ExitEvent. If set, then 
                            * end this thread by exiting the for loop
                            */
                            ExitResult = pExitEvent->Wait(1);
                            if (ExitResult == WAIT_SUCCESS)
                                break;

                            /* Use select() to check for incoming connections without blocking */
                            Socks.fd_array[0] = pCommSvr->RxSocket;
                            Socks.fd_count = 1;
                            ReadSocket = select(1, &Socks, (fd_set *) 0,(fd_set *) 0, &Timeout);
                        }

                        /* Close up the sockets and exit this thread if the ExitEvent was fired */
                        if (ReadSocket <= 0)
                        {
#if defined(DEBUG_PRINT)
                            SYS_PRINTF("StartServer: Found exit event %d\n", WSAGetLastError());
#endif
                            closesocket(pCommSvr->TxSocket);
                            closesocket(pCommSvr->RxSocket);
                            break;
                        }                    

                        /* Set the size of the in-out parameter */
                        ClntLen = sizeof(ClntAddr);

                        if ((MsgSize = recvfrom(pCommSvr->RxSocket, RecvBuffer, RCVBUFSIZE, 0,
                                        (struct sockaddr *) &ClntAddr, &ClntLen)) > 0)
                        {
                            /* Check for a valid PREAMBLE */
                            if( (RecvBuffer[0] == CTL_W) && 
                                (RecvBuffer[1] == CTL_A) && 
                                ((RecvBuffer[2] == CTL_C) || RecvBuffer[2] == CTL_L) )
                            {
                                /* Update the SourceID and ClntAddr structure */
                                /* Set the SourceID */
                                pSourceID = inet_ntoa(ClntAddr.sin_addr);
                                memcpy(&(NewClntAddr.ClntAddr), &ClntAddr , sizeof(ClntAddr));
                                sprintf(SourceBuffer, "%s_%d", pSourceID, ClntAddr.sin_port);
                                NewClntAddr.pSourceID = SourceBuffer;

                                NewSessionArgs.Port = pCommSvr->Port;
                                NewSessionArgs.pSourceID = pSourceID;
                                NewSessionArgs.StackType = ETHERNET;
                                NewSessionArgs.bByPass= FALSE;
                                NewSessionArgs.pAddr = &NewClntAddr;

                                /* Call the callback routine to create a new Comm Session */
                                Result = pCommSvr->pSetRecvMsg(pCommSvr->pSemName, SourceBuffer, (uint8*)RecvBuffer, MsgSize, (void*)&NewSessionArgs);
                            }
                        }
                    }/* for(;;) */

                }/* if (bind(...)) */
#if defined(TRACE_SERVER_ON)
                else
                {
                    SYS_PRINTF("ERROR!!! StartServer: Failed to bind socket: %d\n", WSAGetLastError());
                }
#endif
            }/* if ((RxSocket = socket(PF_INET, SOCK_DGRAM, Protocol)) >= 0) */
#if defined(TRACE_SERVER_ON)
            else
            {
                SYS_PRINTF("ERROR!!! StartServer: Failed to create a new socket: %d\n", WSAGetLastError());
            }
#endif

            pCommSvr->pUDPMutex->Release();
        }
    }/*else if (Protocol == IPPROTO_UDP) */
    
    delete(pExitEvent);

    /* Delete the class instance spawned in CCommSvr::StartServer() */
    delete(pCommSvr);

#if defined(DEBUG_PRINT)
    SYS_PRINTF("StartServer: Exiting\n");
#endif

    return TRUE;
}

/* End of File */





