/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CFmRNDZObjSvr.cpp:                                                //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CFmRNDZObjSvr.h"

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor
    RENDEZVOUS Start up: All available servers are instantiated here. i.e Command Servers,
    Request Servers, Response Servers, etc.

*********************************************************************************/
CFmRNDZObjSvr::CFmRNDZObjSvr()
{       
    /* Initialize the callback function pointer */
    pMessageHandler = NULL;
    Outbuf = NULL;
}

CFmRNDZObjSvr::~CFmRNDZObjSvr()
{
    SYS_FREE(Outbuf);
    Outbuf = NULL;
}

void CFmRNDZObjSvr::Init(CTrapObjSvr* pTrapObjSvr, CTraceSvr* pTraceSvr,
                         boolean (*SendPacket)(char*, char*, uint8*, uint32), 
                         LinkStruct* pLinkStruct)
{       
    MsgSvr.Init(FmRENDEZVOUS, GnDATA, pLinkStruct);
    pTrapServer = pTrapObjSvr;
    pTraceServer = pTraceSvr;
    pSendMsg = SendPacket;
    pSemName = pLinkStruct->pSemName;

}
/*================================================================================
CFmRNDZObjSvr TxMsg : Helper routine that transmits the contents of a CMsgSvr object
================================================================================*/
boolean CFmRNDZObjSvr::TxMsg(char* pSourceID, CMsgSvr* pMsgSvr, uint8 Genus, uint8 Species,
                         uint8* Payload, uint32 Size)
{
    uint32 OutBufSz;
    boolean Result = FALSE;

    /* Set the Genus */
    pMsgSvr->set_Genus(Genus);

    /* Set the Species */
    pMsgSvr->set_Species(Species);

    /* Set the passed in Payload into the dynamic Data member variable */
    pMsgSvr->SetData(Payload, Size);

    /* Get the Size of the serialized byte Stream */
    OutBufSz = pMsgSvr->size_all();

    /* (Re)Allocate space for the serialized Buffer */
    Outbuf = (uint8*)SYS_REALLOC(Outbuf, OutBufSz);
    
    /* Serialize the message */
    if (Outbuf != NULL)
    {
        /* Serialize the CErrSvr object */
        pMsgSvr->serialize_all(Outbuf);

        /* Send the message */
        Result = pSendMsg(pSemName, pSourceID,Outbuf, OutBufSz);
    }

    return(Result);
}

/*================================================================================
RENDEZVOUS SendMsgHandler: Decodes the Family, Genus, and Species tag and deserializes
     the messages with the appropriate object.
================================================================================*/
boolean CFmRNDZObjSvr::SendMsgHandler(char* pSourceID, uint16 Family, uint8 Genus, uint8 Species,
                                      uint8* pBuffer, uint32 Size, boolean EncryptionOn)
{
    boolean RetVal=FALSE;
    TOUCH(EncryptionOn);

#if defined(TRACE_SERVER_ON_STATUS)
    pTraceServer->tprintf(TRACE_Type_SEND, "CFmRNDZObjSvr", "SendMsgHandler",
                          Family, Genus, Species, "");
#endif


    if(Family == FmRENDEZVOUS)
    {
        RetVal = TxMsg(pSourceID, &MsgSvr, Genus, Species, pBuffer, Size);
    }

    return(RetVal);
}
/*================================================================================
RENDEZVOUS RecvMsgHandler: Decodes the Family, Genus, and Species tag and deserializes
     the messages with the appropriate object.
================================================================================*/
boolean CFmRNDZObjSvr::RecvMsgHandler(char* pSourceID, uint8* pBuffer, void* pWACPApp)
{
    uint32  Result = TRUE;
    uint32  Size;
    uint32  ClassID;
    uint16  Family;
    uint8   Genus;
    uint8   Species;

    /* Extract the Family */
    Family = GetFamily(pBuffer);

    /* Extract the Genus */
    Genus = GetGenus(pBuffer);

    /* Extract the Species */
    Species = GetSpecies(pBuffer);
    
    ClassID = MakeClassID(Family, Genus, Species);

    if (Genus != GnDATA)
    {
        /* Set the MsgSvr object to the incoming Genus tag for deserialization */
        MsgSvr.set_Genus(Genus);

        /* Deserialize the data and check for errors */
        Result = MsgSvr.deserialize_all(pBuffer);
        if (!Result)
        {
#if defined(TRACE_SERVER_ON)
            pTraceServer->tprintf(TRACE_Type_ERROR, "CFmRNDZObjSvr", "RecvMsgHandler",
                                  Family, Genus, Species, 
                                  "MsgSvr deserialization failed");
#endif

            /* Call the trapserver with the error code stored in Result */
            pTrapServer->SendError(pSourceID, ClassID, CERR_CRC_MISMATCH, pWACPApp);
        }
    }

    if(Result)
    {
        Result = pMessageHandler(pSourceID, ClassID, MsgSvr.GetData(&Size), pWACPApp);
    }

    return (Result ? TRUE : FALSE);
}
    
/* Function for setting the callback function for all Messages */
boolean CFmRNDZObjSvr::SetMessageFunc(uint32 (*RxMessage)(char*, uint32, uint8*, void*))
{
    boolean Result = FALSE;
    if(RxMessage != NULL)
    {
        pMessageHandler = RxMessage;
        Result = TRUE;
    }
    return(Result);
}
/* Function for setting the callback function for Cmd, Rqst, and Resp Messages */
boolean CFmRNDZObjSvr::SetCallbacks(va_list vList)
{
    int ResultCount = 0;

    uint32 (*RxGeneric)(char*, uint32, uint8*, void*);
    uint32* pRxGeneric = (uint32*)&RxGeneric;

    /* Register Message Callback */
    *pRxGeneric = va_arg(vList, uint32);
    ResultCount += SetMessageFunc(RxGeneric);
   
    return(ResultCount == 1);
}

/* End of File */


