/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CLinkTable.cpp :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CLinkTable.h"
#include "SysUtil.h"

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor
    CLinkTable Start up: 
*********************************************************************************/
CLinkTable::CLinkTable()
{
    Count = 0;
    memset(Connection, 0, sizeof(ConnArgs*) * MAX_CONNECTIONS);
}


/*********************************************************************************
  Default destructor
    CLinkTable termination: 
*********************************************************************************/
CLinkTable::~CLinkTable()
{
    uint16   i;

    for (i=0; i < MAX_CONNECTIONS; i++)
    {
        Connection[i] = NULL;
    }
}

/*================================================================================
CLinkTable AddEntry: Adds the pointer to a ConnArgs into the static array if a
    free slot is found and the pSourceID does not match an existing pointer.
================================================================================*/
boolean CLinkTable::AddEntry(ConnArgs* pConnArgs)
{
    boolean     Result = FALSE;

    /* Check for a valid SourceID */
    if ((pConnArgs) && (pConnArgs->SourceID))
    {
        uint16      i;
        int32       Index;

        /* Check for an existing entry */
        Index = GetEntryIndex(pConnArgs->SourceID);
        if (Index == -1)
        {
            /* Cycle through the list of ConnArgs pointers for an empty spot */
            for(i=0; i < MAX_CONNECTIONS; i++)
            {
                /* Look for an open spot */
                if (Connection[i] == NULL)
                {
                    /* Store the pointer to the ConnArgs */
                    Connection[i] = pConnArgs;

                    /* Initialize the ConnArgs structure */
                    Connection[i]->bConnected = FALSE;
                    Connection[i]->bRNDZStatus = FALSE;
                    Connection[i]->ValidationState = 0;
                    Connection[i]->pKey = NULL;
                    Connection[i]->pGUIDStd = NULL;

                    /* Increment the count */
                    Count++;

#if defined(TRACE_SERVER_ON_STATUS)
                    SYS_PRINTF("CLinkTable::AddEntry: Added %s, Count = %d\n", pConnArgs->SourceID, Count);
#endif
                    Result = TRUE;
                    break;
                }
            }
#if defined(TRACE_SERVER_ON_STATUS)
            if (!Result)
                SYS_PRINTF("CLinkTable::AddEntry: ERROR MAX_CONNECTIONS %d\n", Count);
#endif
        }
        else
        {
            /* Store the pointer to the ConnArgs if the connection is closed 
             * otherwise return FALSE to indicate an error.
             */
            if (Connection[Index]->bConnected == FALSE)
            {
                Connection[Index] = pConnArgs;

                /* Initialize the ConnArgs structure */
                Connection[Index]->bConnected = FALSE;
                Connection[Index]->bRNDZStatus = FALSE;
                Connection[Index]->ValidationState = 0;
                Connection[Index]->pKey = NULL;
                Connection[Index]->pGUIDStd = NULL;

                Result = TRUE;
            }
#if defined(TRACE_SERVER_ON_STATUS)
            else
                SYS_PRINTF("CLinkTable::AddEntry: Existing Entry already connected for %s\n", pConnArgs->SourceID);
#endif
        }
    }
#if defined(TRACE_SERVER_ON_STATUS)
    else
        SYS_PRINTF("CLinkTable::AddEntry: Invalid SourceID\n");
#endif

    return (Result);
}

/*================================================================================
CLinkTable AddEntry: Searches the CLinkTable instance for a matching SourceID and 
    removes it from the list. It is expected that the application who allocated 
    the memory for the ConnArgs structure will also free it.
================================================================================*/
boolean CLinkTable::RemoveEntry(ConnArgs* pConnArgs)
{
    boolean     Result = FALSE;

    /* Check for a valid SourceID */
    if ((pConnArgs) && (pConnArgs->SourceID))
    {
        int32 Index = GetEntryIndex(pConnArgs->SourceID);
        if (Index > -1)
        {
            /* Make sure we only look at valid instances */
            if (Connection[Index] != NULL)
            {
                SYS_FREE(Connection[Index]->pKey);
                Connection[Index]->pKey = NULL;

                if (Connection[Index]->pGUIDStd)
                {
                    delete(Connection[Index]->pGUIDStd);
                    Connection[Index]->pGUIDStd = NULL;
                }

                Connection[Index] = NULL;
                Count--;
                Result = TRUE;

#if defined(TRACE_SERVER_ON_STATUS)
                SYS_PRINTF("CLinkTable::RemoveEntry: Removed %s, Count = %d\n", pConnArgs->SourceID, Count);
#endif
            }
        }
#if defined(TRACE_SERVER_ON_STATUS)
    
        if (!Result)
            SYS_PRINTF("CLinkTable::RemoveEntry: Unable to find %s for removal\n", pConnArgs->SourceID);
#endif
    }


    return (Result);
}


/********************************************************************************************
CLinkTable GetEntryIndex - Helper routine to search the ConnArgs array for a matching 
    SourceID. This returns the 0 based index into the Connection array if an entry is found.
    It returns -1 if the entry is not found.
********************************************************************************************/
int32 CLinkTable::GetEntryIndex(char* pSourceID)
{
    uint16       i;
    uint16       TestCount = 0;
    int32        RetVal = -1;

    /* Check for a valid SourceID */
    if (pSourceID)
    {
        /* Cycle through the list of ConnArgs pointers for a match */
        for(i=0; i < MAX_CONNECTIONS; i++)
        {
            /* Stop the search once we have tested all existing connections */
            if (TestCount >= Count)
                break;

            /* Make sure we only look at valid instances */
            if (Connection[i] != NULL) 
            {
                TestCount++;

                /* If we find a matching SourceID, return a pointer to it */
                if (strcmp(pSourceID, Connection[i]->SourceID) == 0)
                {
                    RetVal = i;
                    break;
                }
            }
        }
    }

#if defined(TRACE_SERVER_ON_STATUS)
    if (RetVal == -1)
        SYS_PRINTF("CLinkTable::GetEntryIndex: %s not found\n", pSourceID);
#endif
    return (RetVal);
}

/********************************************************************************************
CLinkTable GetEntry - Uses GetEntryIndex to return a pointer to the ConnArgs structure based
    on the SourceID passed in. This return NULL if an entry is not found.
********************************************************************************************/
ConnArgs* CLinkTable::GetEntry(char* pSourceID)
{
    ConnArgs*   pConnArgs = NULL;
    int32       Index = -1;

    /* Check for a valid SourceID */
    if (pSourceID)
    {
        Index = GetEntryIndex(pSourceID);
        if (Index != -1)
        {
            pConnArgs = Connection[Index];
        }
    }

#if defined(TRACE_SERVER_ON_STATUS)
    if (!pConnArgs)
        SYS_PRINTF("CLinkTable::GetEntry: %s not found\n", pSourceID);
#endif
    return (pConnArgs);
}

/********************************************************************************************
CLinkTable::GetSourceList - Returns a filled in array of SourceIDs of all existing entries in the 
    CLinkTable. The array passed in is expected to be a size of MAX_CONNECTION and will always  
    be cleared by this routine.
********************************************************************************************/
void CLinkTable::GetSourceList(char** pSourceList)
{
    uint16   i;
    uint16   TestCount = 0;

    if (pSourceList)
    {
        /* Always clear the passed in array */
        memset(pSourceList, 0, sizeof(char*) * MAX_CONNECTIONS);

        /* Check for any existing entries */
        if (Count)
        {
            /* Cycle through the list of ConnArgs pointers for a match */
            for(i=0; i < MAX_CONNECTIONS; i++)
            {
                /* Stop the search once we have tested all existing connections */
                if (TestCount >= Count)
                    break;

                /* Make sure we only look at valid instances */
                if (Connection[i] != NULL) 
                {
                    /* Save the SourceID */
                    pSourceList[TestCount] = Connection[i]->SourceID;

                    /* Increment the current count */
                    TestCount++;
                }
            }
        }
    }
}

#if defined(TEST_LINKTABLE)
#include "CSerialCommSvr.h"
#include "CEthCommSvr.h"

void Test_CLinkTable(void)
{
    CLinkTable  Link;
    ConnArgs*   Config[11];
    uint16       i;
    boolean     Result;

    for(i=0; i<11; i++)
    {
        Config[i] = (ConnArgs*)SYS_CALLOC(sizeof(ConnArgs), 1);
        Config[i]->pConfig = (EthAddr*)SYS_MALLOC(sizeof(EthAddr));
        ((EthAddr*)(Config[i]->pConfig))->Protocol = IPPROTO_TCP;
        sprintf(Config[i]->SourceID, "%s%d", "127.0.0.", i);
        Config[i]->bSvrFlag = i % 2;
        Config[i]->EncryptionType = WACP_ENCRYPTION_NONE;
        Config[i]->Timeout = DEFAULT_RNDZ_TIMEOUT;
    }

    /* Add all 10 + one extra to test max connections */
    for(i=0; i<11; i++)
    {
        Result = Link.AddEntry(Config[i]);
        SYS_PRINTF("AddEntry ret. %d\n", Result);
    }

    Link.RemoveEntry(Config[1]);
    Link.RemoveEntry(Config[2]);
    Link.RemoveEntry(Config[0]);
    Link.RemoveEntry(Config[0]);
    Link.RemoveEntry(NULL);

    /* Check for existing entries */
    Link.AddEntry(Config[9]);
    Link.AddEntry(Config[8]);

    /* Re-add the removed entries */
    Link.AddEntry(Config[2]);
    Link.AddEntry(Config[0]);
    Link.AddEntry(Config[1]);

    /* Test removing all connection plus an extra one that shouldn't exist*/
    for(i=0; i<11; i++)
    {
        Result = Link.RemoveEntry(Config[i]);
        SYS_PRINTF("RemoveEntry ret. %d\n", Result);
    }

    for(i=0; i<11; i++)
    {
        SYS_FREE(Config[i]);
    }
}
#endif

/* End of File */





