/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                         Copyright (c) 2003-2005 by
'*                              Welch Allyn Inc.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CMsgSvr.cpp    :                                                  //
'**  '// Version        :   1.00                                           //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CMsgSvr.h"

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor
*********************************************************************************/
CMsgSvr::CMsgSvr()
{
    /* Initialize the Dynamic members */
    Data = NULL;
    DataSize = 0;

#if defined(ENCRYPTION_ON)
    /* Initialize the encryption message flag. This is used to indicate whether
     * each message should be encrypted or not 
     */
    bEncryptionOn = FALSE;

    /* Initialize the SourceID */
    pSourceID = NULL;
#endif

}
/*********************************************************************************
  Default destructor
*********************************************************************************/
CMsgSvr::~CMsgSvr()
{
    /* free the data */
    SYS_FREE(Data);

    /* Reset the pointer to NULL */
    Data = NULL;
}

void CMsgSvr::Init(uint16 Family, uint8 Genus, LinkStruct *pLinkStruct) 
{
    /* Save the pointer to the LinkStruct structure to get access to Encryption data */
    pLnkStruct = pLinkStruct;

    classIdentifier = MakeClassID(Family, Genus, CMSG_SpNONE );
}
/*---------------------------------
 Static Members
---------------------------------*/
#if defined(ENCRYPTION_ON)
/*********************************************************************************
 CMsgSvr::SetEncryption: Sets the Encryption flag to either TRUE or FALSE.
 If TRUE, the serialize/deserialize methods will encrypt/decrypt the payload.
*********************************************************************************/
void CMsgSvr::SetEncryption(char *pSource, boolean bEncryptFlag)
{
    pSourceID = pSource;
    bEncryptionOn = bEncryptFlag;
}
#endif

/*---------------------------------
 Dynamic Members
---------------------------------*/
/*********************************************************************************
 Set function for dynamic data member Data
*********************************************************************************/
void CMsgSvr::SetData(uint8* NewData, uint32 Size)
{
    /*Allocate memory for the new Buffer*/
    if (NewData != NULL)
    {
        uint32 PaddedSize = Size;
#if defined(ENCRYPTION_ON)
        ConnArgs *pConnArgs = NULL;

        if (pSourceID)
        {
            pConnArgs = ((CLinkTable*)(pLnkStruct->pCLinkTable))->GetEntry(pSourceID);
            if(pConnArgs)
            {
                if ((bEncryptionOn) && (pConnArgs->EncryptionType))
                {
                    /* When using AES encryption, pad Data so that it lines up on 16 byte boundary */            
                    PaddedSize = Size + (16 - (Size % 16));
                }
            }
        }
#endif
        Data = (uint8*)SYS_REALLOC(Data, PaddedSize);
        if (Data != NULL)
        {
            /* Make a copy of the original data*/
            memcpy(Data, NewData, Size);

#if defined(ENCRYPTION_ON)
            if (pConnArgs)
            {
                if((bEncryptionOn) && (pConnArgs->EncryptionType))
                {
                    /* Clear the padded bytes */
                    memset(&Data[Size], 0, PaddedSize - Size);
                }
            }
#endif
            /* Set the DataSize Var */
            DataSize = PaddedSize;
        }
    }
    else
    {
        SYS_FREE(Data);
        Data = NULL;
    }
}
/*********************************************************************************
 Get function for dynamic data member Data
*********************************************************************************/
uint8* CMsgSvr::GetData(uint32 *Size)
{
    if (Size)
    {
        *Size = DataSize;
    }
    return (uint8*)Data;
}

/*================================================================================
    INHERITED / OVERIDDEN
================================================================================*/
/*********************************************************************************
Helper Function to return TRUE if a serialized object has a uint32 size identifier
*********************************************************************************/
boolean CMsgSvr::is_Extended(uint8* pBuffer)
{   
    TOUCH(pBuffer);

    return TRUE;
};
/*********************************************************************************
Helper Function to return TRUE if a serialized object has a uint32 size identifier
*********************************************************************************/
boolean CMsgSvr::is_Extended(uint32 ClassID)
{
    TOUCH(ClassID);

    return TRUE;
};
/*********************************************************************************
Returns the number of Bytes that would be used by the serialize_payload and
deserialize_payload methods.
*********************************************************************************/
uint32 CMsgSvr::get_payload_size ( void )
{   
    uint32 Size = 0;
    
    /*Encryption seed Key*/
    Size = sizeof(byte);                

    /* Get the Size of each dynamic member */
    if (Data != NULL)
    {
        Size += sizeof(uint32) + DataSize;
    }
    else
    {
        Size += sizeof(uint32);
    }

    Size += sizeof(uint16);             /* CRC Bytes */

    return Size;            
}
/*********************************************************************************
Used to place the data elements of the object into a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CMsgSvr::serialize_payload(uint8QManager * QMgr)
{
    uint32      CRCLen;

    /* Save the start of the byte stream for calculating the CRC */
    byte* TailPtr = (byte*)QMgr->GetTailPtr();
 
#if defined(ENCRYPTION_ON)

    ConnArgs *pConnArgs = NULL;
    if (pSourceID)
    {        
        pConnArgs = ((CLinkTable*)(pLnkStruct->pCLinkTable))->GetEntry(pSourceID);
    }

    if(pConnArgs)
    {
        /* Enqueue the encryption byte if encrytion is enabled otherwise 
        * use WACP_ENCRYPTION_NONE 
        */
        if (bEncryptionOn)
            QMgr->put(pConnArgs->EncryptionType);
        else
            QMgr->put((uint8)WACP_ENCRYPTION_NONE);
    }
    else
#endif
        QMgr->put((uint8)WACP_ENCRYPTION_NONE);

    /* Enqueue variable sized members and there sizes */
    if (Data != NULL)
    {
        QMgr->put(DataSize);

#if defined(ENCRYPTION_ON)
        if(pConnArgs)
        {
            /* Check to see if Encryption is enabled */
            if((bEncryptionOn) && (pConnArgs->EncryptionType))
            {
                /* Encrypt the byte stream after EncryptionType up to (but not including) the CRC*/
                Crypt.AESEncrypt(pConnArgs->pKey,Data,Data,DataSize);     
            }
        }
#endif        
        QMgr->enqueue_array((byte*)Data, DataSize);
    }
    else
    {
        QMgr->put((uint32)(0) );
    }

    /* Enqueue the CRC */
    CRCLen = QMgr->get_num_elements() - sizeof(uint16);
    QMgr->put(uint16(CalculateCRC(TailPtr,CRCLen)));

    return TRUE; 
}

/*********************************************************************************
Used to deserialize the data elements of the object from a circular queue. This
excludes the class identifier.
*********************************************************************************/
boolean CMsgSvr::deserialize_payload(uint8QManager * QMgr, uint32 Sz)
{
    /*These data pieces will need to be dequeued*/
    boolean     Result = TRUE;
    uint32      CRCSize;
    uint16      CalcCRC;
    uint16      StoredCRC;
    byte        EncryptionType;
        
    TOUCH(Sz);

    /* Save the start of the byte stream for calculating the CRC */
    byte* HeadPtr = (byte*)QMgr->GetHeadPtr();

    /* Calculate the CRC on all but the 2 Bytes of the CRC */
    CRCSize = QMgr->get_num_elements() - sizeof(uint16);
    CalcCRC = CalculateCRC(HeadPtr, CRCSize);

    /* Dequeue the encryption byte */
    QMgr->get(&EncryptionType);

    /* Dequeue the Dynamic members */
    QMgr->get(&DataSize);

    if (DataSize > 0)
    {
        Data = (uint8*)SYS_REALLOC(Data, DataSize);
        if (Data != NULL)
        {
            QMgr->dequeue_array((byte*)Data, DataSize);

#if defined(ENCRYPTION_ON)
            /* Check the encryption type */
            if(EncryptionType)
            {
                if (pSourceID)
                {
                    ConnArgs *pConnArgs = ((CLinkTable*)(pLnkStruct->pCLinkTable))->GetEntry(pSourceID);
                    if(pConnArgs)
                    {
                        /* Decrypt the byte stream after EncryptionType up to (but not including) the CRC*/
                        Crypt.AESDecrypt(pConnArgs->pKey,Data,Data,DataSize);
                    }
                }
            }
#endif
        }
    }
    else
    {
        SYS_FREE(Data);
        Data = NULL;
    }
    
    /* Dequeue the CRC */
    QMgr->get(&StoredCRC);
    if (CalcCRC != StoredCRC)
    {
#if defined(DEBUG_PRINT)
        SYS_PRINTF("CRC Mismatch Error: Calculated=0x%d, Stored=0x%d ", CalcCRC, StoredCRC);
#endif

        Result = FALSE;
    }

    return Result; 
}

/* End of File */




