/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CRNDZStub.cpp  :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CRNDZStub.h"
#include "CWACPApp.h"

CRNDZStub::CRNDZStub()
{
    pWACPApp = NULL;
}
CRNDZStub::~CRNDZStub()
{
}

void CRNDZStub::Init(CWACPApp* WACPApp)
{
    pWACPApp = WACPApp;
}   

/*================================================================================
RecvMessage : Handles RNDZ Command, Request, Response, and Status messages.
================================================================================*/
uint32 CRNDZStub::RecvMessage(char* pSourceID, uint32 ClassID, uint8* pBuffer)
{
    uint32      Result = TRUE;
    uint16      Family = GetFamily(ClassID);
    uint8       Genus  = GetGenus(ClassID);
    uint8       Species = GetSpecies(ClassID);

    if (Family == FmRENDEZVOUS)
    {
        switch (Genus)
        {
            case GnCOMMAND:
            {
                RecvCommand(pSourceID, Species, pBuffer);
                break;
            }
            case GnRESPONSE:
            {
                RecvResponse(pSourceID, Species, pBuffer);
                break;
            }
            case GnREQUEST:
            {
                RecvRequest(pSourceID, Species, pBuffer);
                break;
            }
#if defined(TRACE_SERVER_ON)
            default:
            {
                ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_ERROR, "CRNDZStub", "RecvMessage",
                        FmRENDEZVOUS, Genus, Species, "%s: \nReceived Invalid RNDZ Genus.\n", pSourceID);
                break;
            }
#endif
        }
    }/* if (Family == FmRENDEZVOUS) */

    return (Result);
}

/*================================================================================
RecvCommand : Handles RNDZ command messages.
================================================================================*/
uint32 CRNDZStub::RecvCommand(char* pSourceID, uint8 CommandID, uint8* pBuffer)
{
    uint32      Result = FALSE;
    ConnArgs*   pConnArgs = NULL;

    TOUCH(pBuffer);

    pConnArgs = pWACPApp->pCLinkTable->GetEntry(pSourceID);
    if (!pConnArgs)
        return 0;

    switch (CommandID)
    {   
        /* RNDZ_CCMD_SpTERMINATE is sent from the host when the RNDZ Authentication 
         * process is aborted.
         */
        case(RNDZ_CCMD_SpTERMINATE):
        {
#if defined(TRACE_SERVER_ON)
            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_RECEIVE, "CRNDZStub", "RecvCommand",
                        FmRENDEZVOUS, GnCOMMAND, CommandID, "%s: \nReceived Command for RNDZ_CCMD_SpTERMINATE.\n", pSourceID);
#endif
            /* Stop this DataThread and send an event to the RNDZ class to terminate */
            TERMINATE_RNDZ(pSourceID, FALSE);

            /* Reset the Validation state for failed RNDZ authentications */
            pConnArgs->ValidationState = RNDZ_VALIDATION_NONE;
            break;
        }
        /* RNDZ_CCMD_SpSEND_HOST_READY is sent from the host indicating a successful
         * RNDZ authentication. This signals the client to terminate the RNDZ client
         * and to start up the WACP session.
         */
        case(RNDZ_CCMD_SpSEND_HOST_READY):
        {
#if defined(TRACE_SERVER_ON)
            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_RECEIVE, "CRNDZStub", "RecvCommand", FmRENDEZVOUS, GnCOMMAND, 
                    CommandID, "%s: \nReceived Command for RNDZ_CCMD_SpSEND_HOST_READY.\n", pSourceID);
#endif
            /* Store the GUID from the Host */
            if (pBuffer != NULL)
            {
                if (pConnArgs->pGUIDStd == NULL)
                {
                    /* Allocate room for the GUID object if it doesn't exists */
                    /* This is free'd up in CInterchange::Disconnect() */
                    pConnArgs->pGUIDStd = new CRNDZDStd();
                }
                
                if (pConnArgs->pGUIDStd)
                {
                    Result = ((CRNDZDStd*)(pConnArgs->pGUIDStd))->deserialize_all(pBuffer);
                }

#if defined(TRACE_SERVER_ON_STATUS)
                if (Result)
                {
                    uint32 XMLSize = ((CRNDZDStd*)(pConnArgs->pGUIDStd))->serialize_to_XMLbuf(NULL);
                    char* pTemp = (char*)SYS_MALLOC(XMLSize);
                    if (pTemp != NULL)
                    {
                        ((CRNDZDStd*)(pConnArgs->pGUIDStd))->serialize_to_XMLbuf(pTemp);
                        ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_RECEIVE, "CRNDZStub", "RecvCommand", FmRENDEZVOUS, GnCOMMAND, 
                            CommandID, "%s: \nReceived GUID\n\n%s", pSourceID, pTemp);
                        SYS_FREE(pTemp);
                    }
                }
#endif
            }

            /* Properly terminate the RNDZ Session */
            TERMINATE_RNDZ(pSourceID, TRUE);
            break;
        }
        /* RNDZ_CCMD_SpSTART_HOST_PROC is sent from the client once RNDZ initialization is
         * completed. This includes negotiating the encryption type and key if encryption is
         * enabled. 
         */
        case(RNDZ_CCMD_SpSTART_HOST_PROC):
        {
#if defined(TRACE_SERVER_ON)
            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_RECEIVE, "CRNDZStub", "RecvCommand", FmRENDEZVOUS, GnCOMMAND, 
                    CommandID, "%s: \nReceived Command for RNDZ_CCMD_SpSTART_HOST_PROC.\n", pSourceID);
#endif
            if ((pConnArgs->ValidationState == RNDZ_VALIDATION_START_RNDZ_RECVD) ||
                (pConnArgs->ValidationState == RNDZ_VALIDATION_KEY_EXCHANGE))
            {
                uint8* pObjBuffer = NULL;
                uint32 ObjSize = 0;
                
                pConnArgs->ValidationState = RNDZ_VALIDATION_COMPLETE;

#if defined(TRACE_SERVER_ON)
                ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_SEND, "CRNDZStub", "RecvCommand", FmRENDEZVOUS, GnCOMMAND, 
                        CommandID, "%s: \nSending Command RNDZ_CCMD_SpSEND_HOST_READY.\n", pSourceID);
#endif

                /* Store the GUID from the Client */
                if (pBuffer != NULL)
                {
                    if (pConnArgs->pGUIDStd == NULL)
                    {
                        /* Allocate room for the GUID object if it doesn't exists */
                        /* This is free'd up in CInterchange::Disconnect() */
                        pConnArgs->pGUIDStd = new CRNDZDStd();
                    }

                    if (pConnArgs->pGUIDStd)
                    {
                        Result = ((CRNDZDStd*)(pConnArgs->pGUIDStd))->deserialize_all(pBuffer);
                    }

#if defined(TRACE_SERVER_ON_STATUS)
                    if (Result)
                    {
                        uint32 XMLSize = ((CRNDZDStd*)(pConnArgs->pGUIDStd))->serialize_to_XMLbuf(NULL);
                        char* pTemp = (char*)SYS_MALLOC(XMLSize);
                        if (pTemp != NULL)
                        {
                            ((CRNDZDStd*)(pConnArgs->pGUIDStd))->serialize_to_XMLbuf(pTemp);
                            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_RECEIVE, "CRNDZStub", "RecvCommand", FmRENDEZVOUS, GnCOMMAND, 
                                CommandID, "%s: \nReceived GUID\n\n%s", pSourceID, pTemp);
                            SYS_FREE(pTemp);
                        }
                    }
                    else
                    {
                            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_ERROR, "CRNDZStub", "RecvCommand", FmRENDEZVOUS, GnCOMMAND, 
                                CommandID, "%s: \nUnable to deserialize GUID\n\n", pSourceID);
                    }

#endif
                }

                /* Send out the Host's GUID */
                pObjBuffer = (uint8*)SYS_MALLOC(ObjSize = GUIDDStd.size_all());
                if (pObjBuffer)
                {
                    GUIDDStd.serialize_all(pObjBuffer);
                }

                /* Send the START HOST PROC command with the serialized GUID object stream */
                Result = SEND_RNDZ_COMMAND(pSourceID, RNDZ_CCMD_SpSEND_HOST_READY, pObjBuffer, ObjSize);

                /* Free up the memory */
                SYS_FREE(pObjBuffer);
                pObjBuffer = NULL;
            }

            if(pConnArgs->ValidationState == RNDZ_VALIDATION_COMPLETE)
            {
                /* Properly terminate the RNDZ Session */
                TERMINATE_RNDZ(pSourceID, TRUE);
            }
            else{
                /* Properly terminate the RNDZ Session */
                TERMINATE_RNDZ(pSourceID, FALSE);

                /* Reset the Validation state for failed RNDZ authentications */
                pConnArgs->ValidationState = RNDZ_VALIDATION_NONE;
            }
            break;
        }
        default:
        {
#if defined(TRACE_SERVER_ON)
            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_SEND, "CRNDZStub", "RecvCommand", FmRENDEZVOUS, GnCOMMAND, 
                    CommandID, "%s: \nReceived unknown command\n", pSourceID);
#endif
            Result = FALSE;
        }
    }

    return Result;
}

/*================================================================================
RecvRequest : Handles RNDZ Request messages.
================================================================================*/
uint32 CRNDZStub::RecvRequest(char* pSourceID, uint8 RequestID, uint8* pBuffer)
{
    uint32  Result = TRUE;
    ConnArgs* pConnArgs = NULL;
    
    pConnArgs = pWACPApp->pCLinkTable->GetEntry(pSourceID);
    if (!pConnArgs)
        return 0;

    switch (RequestID)
    {   
        /* RNDZ_CRST_SpSTART_RNDZ is a request from the client for starting the 
         * RNDZ authentication sequence. This is faked if DTR is supported for
         * serial connections, otherwise, a message is sent from the client with
         * the appropriate validation string "RNDZCONNECT".
         */
        case(RNDZ_CRST_SpSTART_RNDZ):
        {
#if defined(TRACE_SERVER_ON)
            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_RECEIVE, "CRNDZStub", "RecvRequest", FmRENDEZVOUS, GnREQUEST, 
                    RequestID, "%s: \nReceived Request for RNDZ_CRST_SpSTART_RNDZ\n", pSourceID);
#endif
            /* If RNDZ is not disabled, then check to see if we are in a valid state */
            if (pConnArgs->bRNDZDisabled == FALSE)
            {
                /* If ValidationState is not RNDZ_VALIDATION_NONE, then drop any attempt
                 * at another RNDZ request.
                 */
                if (pConnArgs->ValidationState != RNDZ_VALIDATION_NONE)
                {
#if defined(TRACE_SERVER_ON)
                    ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_ERROR, "CRNDZStub", "RecvRequest", FmRENDEZVOUS, GnREQUEST, 
                        RequestID, "%s: \nRNDZ connection already in progress.\n", pSourceID);
#endif
                    break;
                }
            }

            if(strstr((const char*)pBuffer,"RNDZCONNECT"))
            {
                /* Update the ValidationState */
                pConnArgs->ValidationState = RNDZ_VALIDATION_START_RNDZ_RECVD;

#if defined(TRACE_SERVER_ON)
                ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_STATUS, "CRNDZStub", "RecvRequest", FmRENDEZVOUS, GnREQUEST, 
                    RequestID, "%s: \nRendezvous request validated.\n", pSourceID);
#endif
                /* Entry point for client(host) devices.
                 * Set the connection object server option to CHECK mode to 
                 * allow the RNDZ process to timeout if the authentication 
                 * takes a long period of time. 
                 */
                pWACPApp->SetConnOptions(pSourceID, CONN_CFG_SpKEEPALIVECHECK, FALSE, 
                                         DEFAULT_KEEPALIVE_INTERVAL, DEFAULT_TIMEOUT_INTERVAL);

#if defined(TRACE_SERVER_ON)
                ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_SEND, "CRNDZStub", "RecvRequest", FmRENDEZVOUS, GnRESPONSE, 
                        RNDZ_CRSP_SpRNDZ_ACCEPT, "%s: \nSending Response RNDZ_CRSP_SpRNDZ_ACCEPT.\n", pSourceID);
#endif
                /* We received a start request, return an acceptance response */
                Result = SEND_RNDZ_RESPONSE(pSourceID, RNDZ_CRSP_SpRNDZ_ACCEPT, NULL,0);

            }
            else
            {
#if defined(TRACE_SERVER_ON)
                ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_SEND, "CRNDZStub", "RecvRequest", FmRENDEZVOUS, GnCOMMAND, 
                    RNDZ_CCMD_SpTERMINATE, "%s: \nInvalid Connect Request. Sending Command RNDZ_CCMD_SpTERMINATE.\n", pSourceID);
#endif
                /* We received a start request, but the header message was wrong.
                   Send back a terminate command to close the session. */
                Result = SEND_RNDZ_COMMAND(pSourceID, RNDZ_CCMD_SpTERMINATE, NULL, 0);
            }
            break;
        }
#if defined(ENCRYPTION_ON)
        /* RNDZ_CRST_SpGET_ENCRYPT_KEY is a request from the client to retreive the
         * host-generated encryption key for the session.
         */
        case(RNDZ_CRST_SpGET_ENCRYPT_KEY):
        {
#if defined(TRACE_SERVER_ON)
            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_RECEIVE, "CRNDZStub", "RecvRequest", FmRENDEZVOUS, GnREQUEST, 
                    RequestID, "%s: \nReceived Request for RNDZ_CRST_SpGET_ENCRYPT_KEY.\n", pSourceID);
#endif
            /* Create an encryption object */
            CEncryption Crypt;

            /* Allocate room for the new session key */
            pConnArgs->pKey = (uint8*)SYS_MALLOC(AES_ENCRYPTION_KEY_SIZE);
            
            if(pConnArgs->pKey != NULL)
            {
                /* Generate a key for this session */
                Crypt.GenerateAESKey(pConnArgs->pKey);

                /* Set the encryption type */
                pConnArgs->EncryptionType = WACP_ENCRYPTION_AES;
            }

#if defined(TRACE_SERVER_ON)
                ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_SEND, "CRNDZStub", "RecvRequest", FmRENDEZVOUS, GnRESPONSE, 
                    RNDZ_CRSP_SpSEND_ENCRYPT_KEY, "%s: \nSending Response RNDZ_CRSP_SpSEND_ENCRYPT_KEY.\n", pSourceID);
#endif
            /* TODO: Encrypt the session key with a master key before sending it out */ 
            Result = SEND_RNDZ_RESPONSE(pSourceID, RNDZ_CRSP_SpSEND_ENCRYPT_KEY, pConnArgs->pKey, AES_ENCRYPTION_KEY_SIZE);

            break;
        }
#endif /* #if defined(ENCRYPTION_ON) */

        default:
        {
            /* No need to flag an error since the presentation layer 
             * will handle this error
             */
            Result = FALSE;
        }
    }

    return Result;
}
/*================================================================================
RecvResponse :  ResponseID matches the Species of the Response, and pBuffer is
                a generic byte Buffer holding data defined by the developer
================================================================================*/
uint32 CRNDZStub::RecvResponse(char* pSourceID, uint8 ResponseID, uint8* pBuffer)
{
    uint32      Result = TRUE;
    ConnArgs*   pConnArgs = NULL;
    uint8*      pObjBuffer = NULL;
    uint32      ObjSize = 0;

    pConnArgs = pWACPApp->pCLinkTable->GetEntry(pSourceID);
    if (!pConnArgs)
        return 0;

    TOUCH(pBuffer);

    /* Create the serialized byte stream for the GUID object */
    pObjBuffer = (uint8*)SYS_MALLOC(ObjSize = GUIDDStd.size_all());
    if (pObjBuffer)
    {
        GUIDDStd.serialize_all(pObjBuffer);
    }

    switch (ResponseID)
    {   
        /* RNDZ_CRSP_SpRNDZ_ACCEPT is a response from the host to acknowledging
         * the RNDZ_CRST_SpSTART_RNDZ request.
         */
        case(RNDZ_CRSP_SpRNDZ_ACCEPT):
        {
#if defined(TRACE_SERVER_ON)
            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_RECEIVE, "CRNDZStub", "RecvResponse", FmRENDEZVOUS, GnRESPONSE, 
                    ResponseID, "%s: \nReceived Response RNDZ_CRSP_SpRNDZ_ACCEPT.\n", pSourceID);
#endif
            /* Entry point for client devices.
             * Set the connection object server option to CHECK mode to 
             * allow the RNDZ process to timeout if the authentication 
             * takes a long period of time. 
             */
            pWACPApp->SetConnOptions(pSourceID, CONN_CFG_SpKEEPALIVECHECK, FALSE, 
                                     DEFAULT_KEEPALIVE_INTERVAL, DEFAULT_TIMEOUT_INTERVAL);

#if defined(ENCRYPTION_ON)
            /* If we are supporting encryption then send a request to get the encryption key */
            if (pConnArgs->EncryptionType)
            {
#if defined(TRACE_SERVER_ON)
            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_SEND, "CRNDZStub", "RecvResponse", FmRENDEZVOUS, GnREQUEST, 
                    RNDZ_CRST_SpGET_ENCRYPT_KEY, "%s: \nSending Request RNDZ_CRST_SpGET_ENCRYPT_KEY.\n", pSourceID);
#endif
                Result = SEND_RNDZ_REQUEST(pSourceID, RNDZ_CRST_SpGET_ENCRYPT_KEY);
            }
            else 
#endif /* #if defined(ENCRYPTION_ON) */
            {
#if defined(TRACE_SERVER_ON)
            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_SEND, "CRNDZStub", "RecvResponse", FmRENDEZVOUS, GnCOMMAND, 
                    RNDZ_CCMD_SpSTART_HOST_PROC, "%s: \nSending Command RNDZ_CCMD_SpSTART_HOST_PROC.\n", pSourceID);
#endif
                /* If encryption is not enabled, then start the host process */
                Result = SEND_RNDZ_COMMAND(pSourceID, RNDZ_CCMD_SpSTART_HOST_PROC, pObjBuffer, ObjSize);
            }
            break;
        }
#if defined(ENCRYPTION_ON)
        /* RNDZ_CRSP_SpSEND_ENCRYPT_KEY is a response from the server that contains the
         * session encryption key. 
         */
        case(RNDZ_CRSP_SpSEND_ENCRYPT_KEY):
        {
#if defined(TRACE_SERVER_ON)
            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_RECEIVE, "CRNDZStub", "RecvResponse", FmRENDEZVOUS, GnRESPONSE, 
                    ResponseID, "%s: \nReceived Response RNDZ_CRSP_SpSEND_ENCRYPT_KEY.\n", pSourceID);
#endif
            /* Store the session key */           
            pConnArgs->pKey = (uint8*)SYS_MALLOC(AES_ENCRYPTION_KEY_SIZE);

            if ((pConnArgs->pKey != NULL) && (pBuffer))
            {
                /* Store this session key */
                memcpy(pConnArgs->pKey, pBuffer, AES_ENCRYPTION_KEY_SIZE);
        
                /* Set the encryption type */
                pConnArgs->EncryptionType = WACP_ENCRYPTION_AES;               
            }
            
#if defined(TRACE_SERVER_ON)
            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_SEND, "CRNDZStub", "RecvResponse", FmRENDEZVOUS, GnCOMMAND, 
                    RNDZ_CCMD_SpSTART_HOST_PROC, "%s: \nSending Command RNDZ_CCMD_SpSTART_HOST_PROC.\n", pSourceID);
#endif
            /* Start the host process */
            Result = SEND_RNDZ_COMMAND(pSourceID, RNDZ_CCMD_SpSTART_HOST_PROC, pObjBuffer, ObjSize);

            break;
        }
#endif /* #if defined(ENCRYPTION_ON) */

        /* Unknown Response */
        default:
            Result = 0;
    }

    /* Free up the memory allocation */
    SYS_FREE(pObjBuffer);
    pObjBuffer = NULL;

    return Result;

}

/*================================================================================
RecvConn : Handles connection status message based on the ConnType.
================================================================================*/
uint32 CRNDZStub::RecvConnection(char* pSourceID, uint8 ConnType, uint8* pBuf)
{
    uint32 Result  = 0;

    TOUCH(pBuf);

#if defined(TRACE_SERVER_ON)
    ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_RECEIVE, "CRNDZStub", "RecvConnection", FmCONNECTION, GnCOMMAND, 
            ConnType, "%s: \nReceived Connection ConnType = %d.\n", pSourceID, ConnType);
#endif
    switch (ConnType)
    {
        case CONN_CMD_SpSHUTDOWN:
        {   
            /* Stop this DataThread and send an event to the RNDZ class to terminate */
            TERMINATE_RNDZ(pSourceID, FALSE);
            break;
        }
        default:
        {
            break;
        }
    }

    return Result;
}

/* End of File */



