/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CSerialCommSvr.cpp:                                               //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CSerialCommSvr.h"
#include "CCommSvr.h"

const uint8 RndzConnect[] = /* Faked WACONNECT message */
    {   0x00,0x1D,0x0B,0x01,0x00,0x00,0x00,0x12,
        0x00,0x00,0x00,0x00,0x0B,0x52,0x4E,0x44,
        0x5A,0x43,0x4F,0x4E,0x4E,0x45,0x43,0x54,
        0x57,0xF3 };

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
*********************************************************************************/
CSerialCommSvr::CSerialCommSvr(boolean (*RecvMsg)(char* pSemName, char* pSourceID, 
                         uint8* pBuf, uint32 MsgSize, void* pNewConnection),                   
                         boolean (*TerminateClient)(char* pSemName, char* pSourceID, uint16 Port), 
                         char* pAppName)
                  
{
    /* Store the callback function to add a new message */
    pSetRecvMsg = RecvMsg;
    pTerminate = TerminateClient;

    /* Save the application name that is created from the Semaphore name.
     * This is used to differentiate between separate dll instances of the stack.
     */
    pSemName = pAppName;

    /* Zero out structure */
    memset(&SourceID, 0, sizeof(SourceID));         
    
    /* Set the stack type */
    StackType = SERIAL;
    
    /* Initial the member vars */
    hPort = INVALID_HANDLE_VALUE;
    Port = 0;

#if defined(OVERLAPPED_SERIAL)
    OSWrite.Offset = 0 ;
    OSWrite.OffsetHigh = 0 ;
    OSRead.Offset = 0 ;
    OSRead.OffsetHigh = 0 ;

    OSRead.hEvent = CreateEvent( NULL,   /* no security */
                                TRUE,    /* explicit reset req */
                                FALSE,   /* initial event reset */
                                NULL ) ; /* no Name */

    /* Create the Write Event */
    OSWrite.hEvent = CreateEvent( NULL,  /* no security */
                                TRUE,    /* explicit reset req */
                                FALSE,   /* initial event reset */
                                NULL ) ; /* no Name */
#endif
}

CSerialCommSvr::~CSerialCommSvr()
{
    LockMutex.Lock(INFINITE);

    if (hPort != INVALID_HANDLE_VALUE)
    {
        /* Call SetCommMask to cancel any current threads waiting on WaitCommEvent() */
        SetCommMask (hPort, EV_DSR);

        /* Close the communication Port. */
        CloseHandle(hPort);

        hPort = INVALID_HANDLE_VALUE;

        /* Wait until the StartClient Thread exits */
        ExitMutex.Lock(INFINITE);
        ExitMutex.Release();
    }

    LockMutex.Release();

#if defined(OVERLAPPED_SERIAL)
    CloseHandle(OSWrite.hEvent);
    CloseHandle(OSRead.hEvent);
#endif
}

/*================================================================================
CSerialCommSvr GetSourceID: Returns the SourceID of the object
================================================================================*/
char* CSerialCommSvr::GetSourceID()
{
    return((char*)SourceID);
}
/*================================================================================
CSerialCommSvr SetSourceID: Sets the SourceID
================================================================================*/
boolean CSerialCommSvr::SetSourceID(char* pSourceID)
{   
    boolean Result = FALSE;
    if ((pSourceID != NULL) && (pSourceID != SourceID))
    {
        uint16 Size;

        memset(SourceID, 0, sizeof(SourceID));
        Size = (uint16)strlen(pSourceID);
        if (Size > sizeof(SourceID))
            Size = sizeof(SourceID);

        memcpy(SourceID, pSourceID, Size);
        Result = TRUE;
    }

    return (Result);
}
/*================================================================================
CSerialCommSvr GetPortNum: Returns the Port used by the object
================================================================================*/
uint16 CSerialCommSvr::GetPortNum()
{
    return(Port);
}

/*================================================================================
CSerialCommSvr SetPortNum: Sets the Port used by the object
================================================================================*/
boolean CSerialCommSvr::SetPortNum(uint16 NewPort)
{
    boolean Result = TRUE;

    Port = NewPort;

    return (Result);
}

/*================================================================================
CSerialCommSvr SetConfig: Updates the addressing information of the object
================================================================================*/
void CSerialCommSvr::SetConfig(void* pAddr)
{
    memcpy(&Addr, (SerialAddr* )pAddr, sizeof(SerialAddr));
    SetSourceID(Addr.pSourceID);
    SetBaudRate(Addr.BaudRate);
    hPort = Addr.hPort;
}

/*================================================================================
CSerialCommSvr GetConfig: Returns the configuration information
================================================================================*/
void* CSerialCommSvr::GetConfig()
{
    return((void*)(&Addr));
}

/*================================================================================
CSerialCommSvr GetStackType: Returns the type of stack that is being used ( i.e. Ethernet, 
                     RS232, Bluetooth, Zigby, etc
================================================================================*/
STACK_TYPE CSerialCommSvr::GetStackType()
{
    return(StackType);
}

/*================================================================================
CSerialCommSvr GetBaudRate: Returns the serial baud rate.
================================================================================*/
uint32 CSerialCommSvr::GetBaudRate()
{
    return(BaudRate);
}

/*================================================================================
CCommSvr SetBaudRate: Sets the serial baud rate.
================================================================================*/
boolean CSerialCommSvr::SetBaudRate(uint32 NewBaudRate)
{
    BaudRate = NewBaudRate;
    return (TRUE);
}

/*================================================================================
CCommSvr SetByPass: Sets the bByPass boolean flag used to skip arrival mechanism
    for serial servers.
================================================================================*/
boolean CSerialCommSvr::SetByPass(boolean bPass)
{
    bByPass = bPass;
    return (TRUE);
}

/*================================================================================
CSerialCommSvr Connect: Opens a serial com port based on the pSourceID
================================================================================*/
boolean CSerialCommSvr::Connect(uint8* pSourceID, int PortNum)
{
    boolean Result = TRUE;
    
    LockMutex.Lock(INFINITE);
    Port = (uint16)PortNum;

    /* Port Control Settings Structure */
    DCB PortDCB;                  

    /* Comm Port timeout structure */
    COMMTIMEOUTS CommTimeouts;  

    /* Only open the port if it is closed */
    if (hPort == INVALID_HANDLE_VALUE)
    {
        char TempBuf[ADDR_BUF_SIZE];

        /* Clear the temporary buffer */
        memset(TempBuf, 0, ADDR_BUF_SIZE);

        /* Append "\\.\" to allow port number greater than 9 */
        sprintf(TempBuf, "\\\\.\\%s", pSourceID);

        /* Open the serial Port. */
        hPort = CreateFile ((LPCSTR)TempBuf,    /* Pointer to the Name of the Port */
                        GENERIC_READ | GENERIC_WRITE,
                                                /* Access (read-write) mode */
                        0,                      /* Share mode */
                        NULL,                   /* Pointer to the security attribute */
                        OPEN_EXISTING,          /* How to open the serial Port */
#if defined(OVERLAPPED_SERIAL)
                        FILE_ATTRIBUTE_NORMAL |
                        FILE_FLAG_OVERLAPPED,   /* Port attributes */
#else
                        FILE_ATTRIBUTE_NORMAL,
#endif                        
                        NULL);                  /* Handle to Port with attribute to copy */

        /* Check for a valid port handle */
        if (hPort != INVALID_HANDLE_VALUE)
        {
            PortDCB.DCBlength = sizeof (DCB);     

            /* Get the default Port setting information. */
            GetCommState (hPort, &PortDCB);

            /* Change the DCB structure settings. */
            PortDCB.BaudRate = BaudRate;            /* Current baud */
            PortDCB.fBinary = TRUE;                 /* Binary mode; no EOF check */
            PortDCB.fParity = TRUE;                 /* Enable parity checking */
            PortDCB.fOutxCtsFlow = FALSE;           /* CTS Output flow control */
            PortDCB.fOutxDsrFlow = FALSE;           /* DSR Output flow control */
            PortDCB.fDtrControl = DTR_CONTROL_DISABLE; 
                                                    /* DTR flow control type */
            PortDCB.fDsrSensitivity = FALSE;        /* DSR sensitivity */
            PortDCB.fTXContinueOnXoff = FALSE;      /* XOFF continues Tx */
            PortDCB.fOutX = FALSE;                  /* No XON/XOFF out flow control */
            PortDCB.fInX = FALSE;                   /* No XON/XOFF in flow control */
            PortDCB.fErrorChar = FALSE;             /* Disable error replacement */
            PortDCB.fNull = FALSE;                  /* Disable null stripping */
            PortDCB.fRtsControl = RTS_CONTROL_DISABLE; 
                                                    /* RTS flow control */
            PortDCB.fAbortOnError = FALSE;           /* Do not abort reads/writes on 
                                                        error */
            PortDCB.ByteSize = 8;                    /* Number of bits/byte, 4-8 */
            PortDCB.Parity = NOPARITY;               /* 0-4=no,odd,even,mark,space */
            PortDCB.StopBits = ONESTOPBIT;           /* 0,1,2 = 1, 1.5, 2 */

            /* Configure the Port according to the specifications of the DCB 
            * structure. 
            */
            if (!SetCommState (hPort, &PortDCB))
            {
                Result = FALSE; 
#if defined(TRACE_SERVER_ON)
                SYS_PRINTF("Error %d!!! Unable to set the com Port settings\n", GetLastError ());
#endif
            }

            /* Retrieve the time-out parameters for all read and write operations
            * on the Port. 
            */
            GetCommTimeouts (hPort, &CommTimeouts);

            /* Change the COMMTIMEOUTS structure settings. */
            CommTimeouts.ReadIntervalTimeout = MAXDWORD;  
            CommTimeouts.ReadTotalTimeoutMultiplier = 0;  
            CommTimeouts.ReadTotalTimeoutConstant = 0;    
            CommTimeouts.WriteTotalTimeoutMultiplier = 10;  
            CommTimeouts.WriteTotalTimeoutConstant = 1000;    

            /* Set the time-out parameters for all read and write operations
            * on the Port. 
            */
            if (!SetCommTimeouts (hPort, &CommTimeouts))
            {
                Result = FALSE; 
#if defined(TRACE_SERVER_ON)
                SYS_PRINTF("Error %d!!! Unable to set comm timeouts\n", GetLastError ());
#endif
            }

            /* Set the destination IP to be the source */
            SetSourceID((char*)pSourceID);
        }
        else
        {
#if defined(DEBUG_PRINT_STATUS)
            uint32 Error = GetLastError();
            {
                char TempBuf[64];
                FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, 
                              0, 
                              Error,
                              MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                              (LPTSTR) &TempBuf,
                              64,
                              NULL);
                SYS_PRINTF("CCommSvr ERROR: Unable to open port %s\n", TempBuf);
            }
#endif
            Result = FALSE;
        }
    }

    LockMutex.Release();

    return Result;
}

/*================================================================================
CSerialCommSvr SendMsg: Sends the passed in buffer through the com port.
================================================================================*/
boolean CSerialCommSvr::SendMsg(uint8* pBuf, uint32 pBufSize)
{
    boolean RetVal = TRUE;
    DWORD DwNumBytesWritten;

#if !defined(OVERLAPPED_SERIAL)
    if (!WriteFile (hPort,              /* Port handle */
                    pBuf,               /* Pointer to the data to write */
                    pBufSize,           /* Number of Bytes to write */
                    &DwNumBytesWritten, /* Pointer to the number of Bytes 
                                           written */
                    NULL))              /* Must be NULL for Windows CE */
    {
        RetVal = FALSE;
    }
#else
    if (!WriteFile (hPort,              /* Port handle */
                    pBuf,               /* Pointer to the data to write */
                    pBufSize,           /* Number of Bytes to write */
                    &DwNumBytesWritten, /* Pointer to the number of Bytes 
                                           written */
                    &OSWrite))          /* Must be NULL for Windows CE */
    {
        if (GetLastError() != ERROR_IO_PENDING) 
        { 
            /* WriteFile failed, but it isn't delayed. Report error. */
            RetVal = FALSE;
        }
        else
        {
            /* Write is pending. */
            DWORD DwRes = WaitForSingleObject(OSWrite.hEvent, INFINITE);
            switch(DwRes)
            {
                /* Overlapped event has been signaled. */
                case WAIT_SUCCESS:
                    if (!GetOverlappedResult( hPort, &OSWrite, &DwNumBytesWritten, FALSE ))
                        RetVal = FALSE;
                    else 
                    {
                        if( DwNumBytesWritten != pBufSize ) 
                        {
                           /* The write operation timed out. I now need to 
                            * decide if I want to abort or retry. If I retry, 
                            * I need to send only the Bytes that weren't sent. 
                            * If I want to abort, I would just set fRes to 
                            * FALSE and return.
                            */
                            RetVal = FALSE;
                        }
                        else
                            /* Write operation completed successfully. */
                            RetVal = TRUE;
                    }
                    break;
                default:
                    /* An error has occurred in WaitForSingleObject. This usually 
                     * indicates a problem with the overlapped event handle.
                     */

                    RetVal = FALSE;
                    break;
            }
        }
    }
#endif

#if defined(DEBUG_PRINT)
    SYS_PRINTF("SendMessage: Sent %d of %d Bytes\n", DwNumBytesWritten, pBufSize);
    for (uint32 i=0; i < DwNumBytesWritten;i++)
    {
        if ( i % 25 == 0)
            SYS_PRINTF("\n");
        SYS_PRINTF("%.2x ", (uint8)pBuf[i]);
    }
    SYS_PRINTF("\n\n");
#endif  

    return RetVal;
}   

/*================================================================================
CSerialCommSvr ReadData: Reads BytesToRead number of bytes of data from the serial 
    pipe. If overlapped IO is supported, then we will block until data is ready, 
    otherwise, we'll poll incrementally until the data is received or the pipe is
    closed.
================================================================================*/
boolean CSerialCommSvr::ReadData(char* pBuf, uint32 BytesToRead, uint32* pBytesRead)
{
    boolean Result = FALSE;
    DWORD DwCommModemStatus;

    if (hPort != INVALID_HANDLE_VALUE) 
    {
        /* Read the data from the serial Port. */
#if !defined(OVERLAPPED_SERIAL)
        Result = (boolean)ReadFile (hPort, pBuf, BytesToRead, pBytesRead, NULL);
#else
        Result = (boolean)ReadFile (hPort, pBuf, BytesToRead, pBytesRead, &OSRead);
        if (!Result)
        {
            if (GetLastError() == ERROR_IO_PENDING)
            {
                while((Result = (boolean)GetOverlappedResult(hPort, &OSRead, pBytesRead, TRUE)) != TRUE)
                {
                    if(GetLastError() == ERROR_IO_INCOMPLETE)
                        continue;
                    else
                        break;
                }
            }
        }
#endif
        /* If we support DTR, then check for a disconnected state */
        if (Addr.bAutoDiscovery == TRUE)
        {
            /* Get the latest modem status */
            if (GetCommModemStatus(hPort, &DwCommModemStatus))
            {
                /* Check to see if we are in a low DTR state */ 
                if(!(DwCommModemStatus & MS_DSR_ON))
                {
                    /* Wait CONNECTION_TIMEOUT milleseconds to see if the connection comes back */
                    SYS_SLEEP(CONNECTION_TIMEOUT);
                    if (GetCommModemStatus(hPort, &DwCommModemStatus))
                    {
                        /* If we are still low, then notify exchange to terminate */
                        if(!(DwCommModemStatus & MS_DSR_ON))
                        {
                            pTerminate(pSemName, GetSourceID(), GetPortNum());
                        }
                    }
                }
            }
        }

#if !defined(OVERLAPPED_SERIAL)
        if (*pBytesRead == 0)
        {
            /* Wait for a time */
            SYS_SLEEP(SERIAL_POLLTIME);

            /* Read the data from the serial Port. */
            Result = (boolean)ReadFile (hPort, pBuf, BytesToRead, pBytesRead, NULL);
        }
#else
        if (*pBytesRead == 0)
        {
            /* Wait for an event to occur for the Port. */
            Result = (boolean)WaitCommEvent (hPort, &DwCommModemStatus, 0);

            /* Specify a set of events to be monitored for the Port. */
            SetCommMask (hPort, EV_RXCHAR | EV_DSR);
            
            if (Result == TRUE)
            {
                if (DwCommModemStatus & EV_RXCHAR) 
                {
                    /* Read the data from the serial Port. */
                    Result = (boolean)ReadFile (hPort, pBuf, BytesToRead, pBytesRead, &OSRead);
                    if (!Result)
                    {
                        if (GetLastError() == ERROR_IO_PENDING)
                        {
                            while((Result = (boolean)GetOverlappedResult( hPort, &OSRead, pBytesRead, TRUE )) != TRUE)
                            {
                                if(GetLastError() == ERROR_IO_INCOMPLETE)
                                    continue;
                                else
                                    break;
                            }
                        }
                    }
                }
            }
        }
#endif

#if defined(DEBUG_PRINT)
        SYS_PRINTF("\n\nReadData Msg:\n");
        for (uint32 i=0; i < *pBytesRead;i++)
        {
            if ( i % 25 == 0)
                SYS_PRINTF("\n");
            SYS_PRINTF("%.2x ", (uint8)pBuf[i]);
        }
        SYS_PRINTF("\n\n");
#endif
    }

    return(Result);
}

/*================================================================================
CSerialCommSvr CheckDTR(): Checks the serial connection for the DTR line to go high
    and creates a message to simulate the arrival of a RNDZ connect message.
    This routine blocks until the DTR line goes high or the pipe is closed.
================================================================================*/
boolean CSerialCommSvr::CheckDTR()
{
    DWORD   DwCommModemStatus;
    boolean Result = FALSE;
    uint32  PacketLen;
    uint8*  pPacketBuf;

    if (hPort != INVALID_HANDLE_VALUE) 
    {
        /* Get the status of the modem lines, so we can check the DSR line */
        GetCommModemStatus(hPort, &DwCommModemStatus);

        while(!(DwCommModemStatus & MS_DSR_ON) &&
        (hPort != INVALID_HANDLE_VALUE))
        {
            /* Specify a set of events to be monitored for the Port. */
            SetCommMask (hPort, EV_DSR);

            /*  Wait for an event to occur for the Port. */
            WaitCommEvent (hPort, &DwCommModemStatus, 0);

            /* Get the status of the modem lines, so we can check the DSR line */
            GetCommModemStatus(hPort, &DwCommModemStatus);
        }

        if (hPort != INVALID_HANDLE_VALUE)
        {
            pPacketBuf = GeneratePacket(&PacketLen, Port,0,0, (uint8*)RndzConnect, sizeof(RndzConnect));
            pSetRecvMsg(pSemName, SourceID, pPacketBuf, PacketLen, NULL);
            
            /* Free up the memory allocated by GeneratePacket */
            SYS_FREE(pPacketBuf);
            
            Result = TRUE;
        }
    }

    return Result;
}

/*================================================================================
Static CSerialCommSvr StartClient: Handles the start of the new client listener 
    thread. The function loops until the full Packet length is received before 
    passing the mesage through the callback supplied in the constructor. The 
    thread terminates when the ReadData returns FALSE or the Com Port is closed
    and set to INVALID_HANDLE_VALUE.
================================================================================*/
uint32 WACPAPI CSerialCommSvr::StartClient(void* pData)
{
    CSerialCommSvr* pCommSvr = (CSerialCommSvr*)pData;
    char    Buffer[RCVBUFSIZE];
    uint32  MsgSize, BytesLeft, PacketLen;
    int     Result = TRUE;
    uint8   Index = 0;

    if (pCommSvr->ExitMutex.Lock(1) == WAIT_SUCCESS)
    {
        PurgeComm(pCommSvr->hPort, PURGE_TXCLEAR | PURGE_RXCLEAR);

#if defined(OVERLAPPED_SERIAL)
        /* SETDTR: Sends the DTR (data-terminal-ready) signal */
        EscapeCommFunction (pCommSvr->hPort, SETDTR);
#endif

        if(pCommSvr->Addr.bAutoDiscovery == TRUE)
        {
            pCommSvr->CheckDTR();
        }

        /* Set the event flag to trigger when there is data to be read */
        SetCommMask(pCommSvr->hPort, EV_RXCHAR | EV_DSR);

        while (pCommSvr->hPort != INVALID_HANDLE_VALUE) 
        {
            if(!Result)
                SYS_SLEEP(WAIT_COUNT);

            /* Loop until we get a valid preamble or an error */
            Index = 0;
            memset(Buffer, 0, RCVBUFSIZE);
            while(Index != PREAMBLE_SIZE)
            {
                Result = pCommSvr->ReadData((char*)&Buffer[Index], 1, &MsgSize);
                if ((Result) && (MsgSize > 0))
                {
                    switch(Index)
                    {
                        case 0:
                            if (Buffer[Index] == CTL_W)
                                Index++;
                            break;
                        case 1:
                            if (Buffer[Index] == CTL_A)
                                Index++;
                            else
                                Index = 0;
                            break;
                        case 2:
                            if ((Buffer[Index] == CTL_C) || (Buffer[Index] == CTL_L))
                                Index++;
                            else
                                Index = 0;
                            break;
                    }
                }
                else
                {
                    if(!Result)
                    break;
                }
            }
            if (!Result)
                break;

            /* Read the remaining bytes of the message header */
            Result = pCommSvr->ReadData((char*)&Buffer[PREAMBLE_SIZE], MSG_HEADER - PREAMBLE_SIZE, &MsgSize);
            if (Result)
            {
                BytesLeft = MSG_HEADER - PREAMBLE_SIZE - MsgSize;
                while (BytesLeft > 0)
                {
                    Result = pCommSvr->ReadData((char*)&Buffer[MSG_HEADER - BytesLeft], BytesLeft, &MsgSize);
                    if (!Result)
                        break;
                    BytesLeft -= MsgSize;
                    SYS_SLEEP(1);
                }

                /* Check that we received a valid number of bytes for the message header */
                if (BytesLeft == 0)
                {
                    /* Get the packet length */
                    PacketLen = ReadInt32((uint8*)&Buffer[PREAMBLE_SIZE]);
                    if (PacketLen < RCVBUFSIZE)
                    {
                        /* Calculate the remaining Bytes to receive */
                        BytesLeft = PacketLen - MSG_HEADER;

                        /* Receive the rest of the Packet */
                        pCommSvr->ReadData((char*)&Buffer[MSG_HEADER], BytesLeft, &MsgSize);
                        
                        BytesLeft -= MsgSize;

                        /* Loop until we get the full Packet */
                        while (BytesLeft > 0)
                        {
                            uint32 Bytes;

                            Result = pCommSvr->ReadData((char*)&Buffer[PacketLen - BytesLeft], BytesLeft, &Bytes);
                            if (!Result)
                                break;
                            BytesLeft -= Bytes;
                            SYS_SLEEP(1);
                        }

                        /* Check for a valid MsgSize before sending it to the link layer */
                        if ( BytesLeft == 0)
                        {
#if defined(DEBUG_PRINT)
                            SYS_PRINTF("\n\nNew Msg:\n");
                            for (uint32 i=0; i < PacketLen;i++)
                            {
                                if ( i % 25 == 0)
                                    SYS_PRINTF("\n");
                                SYS_PRINTF("%.2x ", (uint8)Buffer[i]);
                            }
                            SYS_PRINTF("\n\n");
#endif
                            /* Update the Recv IN_queue with the new message */
                            pCommSvr->pSetRecvMsg(pCommSvr->pSemName, pCommSvr->GetSourceID(), (uint8*)Buffer, PacketLen, NULL);
                        }

                    }/* if (PacketLen < RCVBUFSIZE) */

                }/* if (BytesLeft == 0) */

            }/* if (Result) */

        }/* while (pCommSvr->hPort != INVALID_HANDLE_VALUE) */

    }/* if (pCommSvr->ExitMutex.Lock(1) == WAIT_SUCCESS) */

#if defined(DEBUG_PRINT)
    SYS_PRINTF("CCommSvr HandleClient: Thread exiting\n");
#endif
    pCommSvr->ExitMutex.Release();
    
    return TRUE;
}

/*================================================================================
CSerialCommSvr StartServer: Listens for serial connections
================================================================================*/
uint32 WACPAPI CSerialCommSvr::StartServer(void* pData)
{
    CWACPEvent*     pExitEvent;
    char            Buffer[EVENT_BUF_SIZE];
    int             ExitResult;
    CSerialCommSvr* pSerialCommSvr = (CSerialCommSvr*)pData;
    DWORD           DwCommModemStatus;
    uint32          PacketLen;
    uint8*          pPacketBuf;
    SerialAddr      NewClntAddr;
    SessionArgs     NewSessionArgs;
    boolean         Result; 

    memset(Buffer, 0, EVENT_BUF_SIZE);
    
    /* Raise an event to signify the end of the client session */
    sprintf(Buffer, END_SERVER, SYS_GETPID());
    pExitEvent = new CWACPEvent(Buffer, TRUE);

    /* Check for ByPass flag to allow Serial Server mode to act as a client */
    if (pSerialCommSvr->bByPass == FALSE)
    {
        /* For backward compatibility */
        for(;;)
        {
            /* Reset the Com Status and Port Handle */
            DwCommModemStatus = 0;
            pSerialCommSvr->hPort = INVALID_HANDLE_VALUE;

            /* Open the com port */
            pSerialCommSvr->Connect((uint8*)pSerialCommSvr->SourceID, pSerialCommSvr->GetPortNum());
            
            /* Clear any remaining data in the INPUT/OUTPUT buffers */
            PurgeComm(pSerialCommSvr->hPort, PURGE_TXCLEAR | PURGE_RXCLEAR);

    #if defined(TRACE_SERVER_ON_STATUS)
            SYS_PRINTF("Listening for new connections on %s\n", pSerialCommSvr->SourceID);
    #endif
            do
            {
                /* Get the status of the modem lines, so we can check the DSR line */
                GetCommModemStatus(pSerialCommSvr->hPort, &DwCommModemStatus);

                SYS_SLEEP(WAIT_COUNT * 10);
            }while(!(DwCommModemStatus & MS_DSR_ON) && (pSerialCommSvr->hPort != INVALID_HANDLE_VALUE));

            if (pSerialCommSvr->hPort != INVALID_HANDLE_VALUE)
            {
                /* Setup the configuration for the CSerialCommSvr instance */
                NewClntAddr.BaudRate = pSerialCommSvr->GetBaudRate();
                NewClntAddr.bAutoDiscovery = FALSE;
                NewClntAddr.Port = pSerialCommSvr->Port;
                NewClntAddr.pSourceID = pSerialCommSvr->SourceID;
                NewClntAddr.hPort = pSerialCommSvr->hPort;
                
                /* Setup default parameters for new connection setup */
                NewSessionArgs.Port = pSerialCommSvr->Port;
                NewSessionArgs.pSourceID = pSerialCommSvr->SourceID;
                NewSessionArgs.StackType = SERIAL;
                NewSessionArgs.bByPass= FALSE;
                NewSessionArgs.pAddr = (void*)&NewClntAddr; 

                pPacketBuf = GeneratePacket(&PacketLen, pSerialCommSvr->Port,0,0, (uint8*)RndzConnect, sizeof(RndzConnect));
                pSerialCommSvr->pSetRecvMsg(pSerialCommSvr->pSemName, pSerialCommSvr->SourceID, pPacketBuf, PacketLen, (void*)&NewSessionArgs);
                
                /* Free up the memory allocated by GeneratePacket */
                SYS_FREE(pPacketBuf);
            }

            while (pSerialCommSvr->hPort != INVALID_HANDLE_VALUE)
            {
                /* Check for a raised ExitEvent.*/
                ExitResult = pExitEvent->Wait(1);
                if (ExitResult == WAIT_SUCCESS)
                {
                    /* Clean up the ExitEvent */
                    delete(pExitEvent);

                    /* Reset this handle since it will be closed by the application instance */
                    pSerialCommSvr->hPort = INVALID_HANDLE_VALUE;

                    /* Delete the class instance spawned in CCommSvr::StartServer() */
                    delete(pSerialCommSvr);

                    return FALSE;
                }

                /* Get the status of the modem lines, so we can check the DSR line */
                Result = (boolean)GetCommModemStatus(pSerialCommSvr->hPort, &DwCommModemStatus);

                /* If GetCommModemStatus fails, then the connection is broken */
                if(!Result)
                {
    #if defined(TRACE_SERVER_ON_STATUS)
                    SYS_PRINTF("Detected break in connection\n");
    #endif
                    break;
                }            

                SYS_SLEEP(SERIAL_SERVER_WAIT);
            }

            SYS_SLEEP(SERIAL_SERVER_WAIT);

            /* Check for a raised ExitEvent.*/
            ExitResult = pExitEvent->Wait(1);
            if (ExitResult == WAIT_SUCCESS)
            {
                /* Clean up the ExitEvent */
                delete(pExitEvent);

                /* Reset this handle since it was already closed by the application instance */
                pSerialCommSvr->hPort = INVALID_HANDLE_VALUE;

                /* Delete the class instance spawned in CCommSvr::StartServer() */
                delete(pSerialCommSvr);

                return TRUE;
            }
        } /* for(;;) */
    }/* if (pSerialCommSvr->Addr.bByPass == FALSE) */
    else
    {
        /* Setup the configuration for the CSerialCommSvr instance */
        NewClntAddr.BaudRate = pSerialCommSvr->GetBaudRate();
        NewClntAddr.bAutoDiscovery = FALSE;
        NewClntAddr.Port = pSerialCommSvr->Port;
        NewClntAddr.pSourceID = pSerialCommSvr->SourceID;
        NewClntAddr.hPort = pSerialCommSvr->hPort;
        
        /* Setup default parameters for new connection setup */
        NewSessionArgs.Port = pSerialCommSvr->Port;
        NewSessionArgs.pSourceID = pSerialCommSvr->SourceID;
        NewSessionArgs.StackType = SERIAL;
        NewSessionArgs.bByPass= TRUE;
        NewSessionArgs.pAddr = (void*)&NewClntAddr; 

        pSerialCommSvr->pSetRecvMsg(pSerialCommSvr->pSemName, pSerialCommSvr->SourceID, NULL, 0, (void*)&NewSessionArgs);

        /* Check for a raised ExitEvent.*/
        ExitResult = pExitEvent->Wait(INFINITE);
        if (ExitResult == WAIT_SUCCESS)
        {
            /* Clean up the ExitEvent */
            delete(pExitEvent);

            /* Reset this handle since it was already closed by the application instance */
            pSerialCommSvr->hPort = INVALID_HANDLE_VALUE;

            /* Delete the class instance spawned in CCommSvr::StartServer() */
            delete(pSerialCommSvr);
        }
    }
 
    return TRUE;
}

/* End of File */





