/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CTraceSvr.cpp  :                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CTraceSvr.h"

const char* CTraceSvr::TypTraceTbl[TRACE_Type_MAX]={
    "ERROR",
    "STATUS",
    "RECEIVE",
    "SEND"
};

/*================================================================================
CTraceSvr:
================================================================================*/
CTraceSvr::CTraceSvr()
{
    Stream = NULL;
    mFileName = NULL;

    PrintOptions = 0;
    TimeBase = SYS_TIME(NULL);
    
}
/*================================================================================
CTraceSvr:
================================================================================*/
CTraceSvr::~CTraceSvr()
{
    CloseFile();
}


/*================================================================================
static vprintfPassThrough : Small helper function to create an abstract layer for
                     the vprintf and vfprintf function differences
================================================================================*/
static int __cdecl vprintfPassThrough(FILE* Stream, const char* format, va_list vList)
{
    if(Stream == NULL)  
        return vprintf(format, vList);
    else 
        return 0;
}

/*================================================================================
CTraceSvr::OpenFile : Opens an .xml file to add log entries. The file is completely
                      overwritten when opened. This function takes 3 optional 
                      arguments; the file Name of the .xml file, the unique ID
                      to distinguish the log from others, and the file Name of
                      the associated .xsl file that will be used to format the
                      .xml log for viewing. The XSLReference argument should match
                      that sent to the GenereateXSL() function, if used.
================================================================================*/
boolean CTraceSvr::OpenFile(const ansichar* Filename, 
                            const ansichar* uniqueID,
                            const ansichar* XSLReference)
{
    boolean Result = FALSE;
    uint16  NameLength = 0;

    /* If we already have a file open, exit immediately and return FALSE */
    if(Stream != NULL) return Result;

    if(Filename != NULL)
    {
        /* Get the length of the passed in Filename */
        NameLength = (uint16)strlen(Filename) + (uint16)strlen(uniqueID);

        /* Allocate memory for the String to be put in mFileName and copy */
        mFileName = (ansichar*)SYS_REALLOC(mFileName, NameLength + 2);
        sprintf(mFileName, "%s_%s", uniqueID, Filename);
        StripFileName(mFileName);
        if((Stream  = fopen( mFileName, "w+" )) == NULL )
        {   
            //TODO: Do something if the file can't be opened
            
            /* we can't open the file, so release mFileName from memory, and set it
            to NULL so we can check it later.  */
            SYS_FREE(mFileName);
            mFileName = NULL;
        }
        else
        {
            /* File was opened sucessfully, so write header information */
            fprintf(Stream, "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n");
            fprintf(Stream, "<?xml-stylesheet type=\"text/xsl\" href=\"%s\"?>\n", XSLReference);
            fprintf(Stream, "<TRACE_LOG>\n");

            /* Add first tag element for the Unique ID */
            fprintf(Stream, "\t<UNIQUE_ID>%s</UNIQUE_ID>\n", uniqueID);
            Result = TRUE;
        }
    }

    return Result;
}
/*================================================================================
CTraceSvr:
================================================================================*/
boolean CTraceSvr::CloseFile()
{
    boolean Result = FALSE;
    
    /* Check if we have a file open */
    if(Stream)
    {
        /* Add the closing tag to the file */
        fprintf(Stream, "</TRACE_LOG>\n");

        if(!fclose( Stream ))
        {
            /* Free the memory allocated for the Filename, and set the pointers
               to NULL so they can be reused */
            SYS_FREE(mFileName);
            mFileName = NULL;
            //Stream = NULL;
            Result = TRUE;
        }
    }
    return Result;
}
/*================================================================================
CTraceSvr::EnableLogging : Turns on logging for either printing to file, printing
                           to screen or both.
================================================================================*/
boolean CTraceSvr::EnableLogging(boolean PrintToFile, boolean PrintToScreen)
{
    PrintToFile ? PrintOptions |= PRINT_TO_FILE_MASK : PrintOptions &= ~PRINT_TO_FILE_MASK;
    PrintToScreen ? PrintOptions |= PRINT_TO_SCREEN_MASK : PrintOptions &= ~PRINT_TO_SCREEN_MASK;

    return TRUE;
}
/*================================================================================
CTraceSvr::DisableLogging
================================================================================*/
boolean CTraceSvr::DisableLogging()
{
    PrintOptions &= (~PRINT_TO_FILE_MASK & ~PRINT_TO_SCREEN_MASK);

    return TRUE;
}

/*================================================================================
CTraceSvr::EnableLogging : Helper function that strips off custom String format 
                           characters, handles them as we see fit, then passes
                           the rest of the String off to the normal printf. 
================================================================================*/
uint32 CTraceSvr::printfStripOff(FILE* Stream, const ansichar* String, va_list vList)
{
    ansichar* StringIter = (ansichar*)String;
    uint32  UniValue = 0;
    uint32     BytesWritten = 0;

    /* For our printf functionality, we will default to one of the vprintf
    functions. We use these vprintf functions because most of out work 
    is with va_lists. Even when we don't have a va_list, we can fake one
    up to use. This lets us simplify the printf discrimination down to
    vfprintf and vprintf, where the only difference is the FILE* argument.
    By default, we will use a function pointer to the standard vfprintf
    function for our uses. We check to see if Stream is NULL immediately,
    and if so, then we will use our help vprintfPassThrough instead. */
    int (__cdecl*my_vprintf)(FILE*, const char*, va_list) = vfprintf;

    if(Stream == NULL)
    {
        my_vprintf = vprintfPassThrough;
    }

    /* Start scanning the String, and keep going until we hit the end */
    while(*StringIter != '\0')
    {
        if(*StringIter == '%')
        {
            /* If we hit a formatting character, increment our format Count.
            This is so when we hit our custom format, we know how much of
            the va_list to pass off and how much to keep for processing.*/
            UniValue++;

            if(*(StringIter+1) == 'A')
            {
                /* We hit our custom format type, so cut off the original
                String and pass it off to print with the va_list up to 
                this point.
                NOTE: This only works on non-constant char arrays, since
                we need to overwrite the values. This would not be 
                necessary if there was a vnprintf statement or something
                where you could specify the length of the String to print.
                This hard cutoff seems to be the easiest and least 
                intensive way to do it... */
                *StringIter = '\0';
                BytesWritten += my_vprintf(Stream, String, vList);

                /* replace the format character for later processing */
                *StringIter = '%';

                /* Step the va_list back to include the 2 arguments
                for our array processing */
                vList += (UniValue - 1)*sizeof(va_list);

                /* print out a special character to signify the start 
                of byte-array data that we can look for in our XSL 
                processing later*/
                BytesWritten += my_vprintf(Stream, "\xAE", NULL);

                /* Grab the Size of the array from our argument list and
                start looping to print out the value of each element. */
                for (uint32 i=0; i < ((uint16)((char**)vList)[1]);i++)
                {
                    /* Format the byte data to print out in blocks of 8 */
                    if ( i % 8 == 0)
                    {
                        BytesWritten += my_vprintf(Stream,"\n\t\t", NULL);
                    }

                    /* Printing with %X formats the Hexadecimal value as a 
                    uint32 Result. In order to display the appropriate
                    byte values as 2 digits instead of 8, we must mask off 
                    the more significant bits and cast as a unint32 */
                    UniValue = ((uint32)(((char**)vList)[0][i]) & 0x00FF);
                    BytesWritten += my_vprintf(Stream, "0x%.2X ", (va_list)&UniValue);
                }
                /* print out a special character to signify the end 
                of byte-array data*/
                BytesWritten += my_vprintf(Stream, "\n\xAF\n", NULL);

                /* Adjust the va_list to skip over the first elements that were
                already used for the earlier vprintf */
                vList += 2 * sizeof(va_list);

                /* Once we reach this point, we have already handled the elements
                in front of our custom format character in the String. Now we
                call this function recursively to handle the rest of the String
                after our custom format character. */
                BytesWritten += printfStripOff(Stream, StringIter + 2, vList);

                return BytesWritten;
            }
        }
        /* Next character in the String... */
        StringIter++;
    }
    /* If we reach this point, then we have gone through an entire String without
    reaching our custom format character. So just print the String as normal. */
    BytesWritten += my_vprintf(Stream, String, vList); 

    return BytesWritten;

}

/*================================================================================
CTraceSvr::tprintf : Provides an easy to use function for printing log entries.
                     Takes 4 mandatory arguments, and allows a variable number
                     of arguments past that. The first is an enumerated value to
                     specify the type of log entry (ERROR, STATUS, etc.). The 
                     second and third are text descriptions of the class and 
                     function that the entry originated from respectively. The
                     fourth argument is a format String that is used in the same
                     ways as a normal printf(...) statement, followed by a various
                     number of arguments matching the format String.
================================================================================*/
uint32 CTraceSvr::tprintf(TRACE_TYPES TraceType, 
                          ansichar* ClassName, ansichar* FunctionName,
                          uint16 Family, uint8 Genus, uint8 Species, 
                          ansichar* DescriptionFormat, ...)
{
    va_list(vList);
    int BytesWritten = 0;
    time_t  CurrentTime = SYS_TIME(NULL);

    /* If any of the strings are invalid, exit immediately */
    if(/*(ClassName == NULL)||(FunctionName == NULL)||*/(DescriptionFormat == NULL)) 
        return BytesWritten;

    /* Check to see if we want to print to file, and if the file is open */
    if((PrintOptions & PRINT_TO_FILE_MASK) && Stream)
    {
        /* Add the opening tag for this log entry */
        BytesWritten += fprintf(Stream, "\t<ENTRY>\n");

        BytesWritten += fprintf(Stream, "\t\t<TIME>%.2d:%.2d:%.2d</TIME>\n",
                        /* Hour */      ((CurrentTime - TimeBase)/3600)%24,
                        /* Minute */    ((CurrentTime - TimeBase)/60)%60,
                        /* Second */    (CurrentTime - TimeBase)%60);

        /* Add a tag for the log entry type */ 
        BytesWritten += fprintf(Stream, "\t\t<TYPE>%s</TYPE>\n", TypTraceTbl[TraceType]);
        
        ///* Log the Name of the class this entry came from */ 
        BytesWritten += fprintf(Stream, "\t\t<CLASS_NAME>%s</CLASS_NAME>\n", 
            (ClassName == NULL) ? "--" : ClassName);
        ///* Log the Name of the function this entry came from */  
        BytesWritten += fprintf(Stream, "\t\t<FUNCTION_NAME>%s</FUNCTION_NAME>\n", 
            (FunctionName == NULL) ? "--" : FunctionName);

        BytesWritten += fprintf(Stream, "\t\t<FAMILY>%d</FAMILY>\n", Family);
        BytesWritten += fprintf(Stream, "\t\t<GENUS>%d</GENUS>\n", Genus);
        BytesWritten += fprintf(Stream, "\t\t<SPECIES>%d</SPECIES>\n", Species);

        /* Add the opening tag for this description text */
        BytesWritten += fprintf(Stream, "\t\t<DESCRIPTION>");

        /* Get the variable length argument list for this function and pass it
           off to the vfprintf function to print our format String and parameters
           to the file */

        va_start(vList, DescriptionFormat);
        BytesWritten += printfStripOff(Stream, DescriptionFormat, vList);
        va_end(vList);

        /* Close the description tag */
        BytesWritten += fprintf(Stream, "</DESCRIPTION>\n");

        /* Add the closing tag for this log entry */
        BytesWritten += fprintf(Stream, "\t</ENTRY>\n");
        
    }
    
    if(PrintOptions & PRINT_TO_SCREEN_MASK)
    {
        /* Print out an information header to screen that tells the trace type,
           class Name, and function Name where this entry came from */
        BytesWritten = SYS_PRINTF("\n%s %s::%s F:%d G:%d S:%d-> ", 
                                TypTraceTbl[TraceType],
                                ClassName, FunctionName,
                                Family, Genus, Species );
        if(BytesWritten > 0)
        {
            /* Get the variable length argument list for this function and pass it
               off to the vprintf function to print our format String and parameters
               to the screen */
            va_start(vList, DescriptionFormat);
            BytesWritten += printfStripOff(NULL, DescriptionFormat, vList);
            BytesWritten += SYS_PRINTF("\n");
            va_end(vList);
        }   
    }       
            
    return BytesWritten;
}
/*================================================================================
CTraceSvr::GenerateXSLFile : Writes an .xsl file with all the formatting specs
                             relating to the .xml log file. Takes an optional
                             argument for the file Name of the .xsl file to be
                             generated. This XSLReference argument should match
                             the one passed to OpenFile, if used.
================================================================================*/
boolean CTraceSvr::GenerateXSLFile(const ansichar* XSLReference)
{
    boolean Result = FALSE;
    FILE*   TempStream = NULL;

    if( (TempStream = fopen(XSLReference, "w+")) != NULL)
    {
        fprintf(TempStream, TRACE_TRANSFORM_XSL);
        if(fclose(TempStream) != 0)
        {
            Result = TRUE;
        }
    }
    return Result;
}

/*================================================================================
CTraceSvr::StripFileName : Strips the passed in Filename of all non alpha numeric
                           characters and replaces them with a " ". This is needed 
                           since the call to open a new filename does not allow
                           certain characters.
================================================================================*/
void CTraceSvr::StripFileName(ansichar* FileName)
{
   char* pBuf = FileName;
   char Delimiter[] = "/\\:*?\"<>|";
   int  pos;
   int  max = (int)strlen(FileName);

   pos = (int)strcspn( pBuf, Delimiter );
   while (pos != max)
   {
        pBuf[pos] = ' ';
        pos = (int)strcspn( pBuf, Delimiter );
   }
}

/* End of File */





