/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CUSBCommSvr.cpp:                                                  //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CUSBCommSvr.h"
#include "CCommSvr.h"
#include "CWACPEvent.h"
#include "Mem.h"
#include "Target.h"

/* WAUSBDLL requires 0 to be passed back on success for callback functions */
#if !defined(SUCCESS)
    #define SUCCESS 0
#endif

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
  Default constructor

*********************************************************************************/
CUSBCommSvr::CUSBCommSvr(boolean (*RecvMsg)(char* pSemName, char* pSourceID, 
                         uint8* pBuf, uint32 MsgSize, void* pNewConnection),                   
                         boolean (*TerminateClient)(char* pSemName, char* pSourceID, uint16 Port), 
                         char* pAppName)
                  
{
    /* Store the callback function to add a new message */
    pSetRecvMsg = RecvMsg;
    pTerminate = TerminateClient;
    
    /* Save the application name that is created from the Semaphore name.
     * This is used to differentiate between separate dll instances of the stack.
     */
    pSemName = pAppName;

    /* Zero out structure */
    memset(SourceID, 0, sizeof(SourceID));         
    
    /* Set the stack type */
    StackType = USB;

    /* Initialize the lock mutexs for destroying the CommSvr */
    pClientExitMutex = new CWACPMutex(NULL);

    pDeviceMutex = new CWACPMutex(NULL);
    pDisconnectMutex = new CWACPMutex(NULL);
    pThreadEvent = new CWACPEvent(NULL, TRUE);
    
    /* Initialize the semaphore for handling data requests */
    pReadSem = new CWACPSemaphore(NULL, 0, MAX_SEM_COUNT);

    /* Creation of this class instance means the USB connection has been established.
     * Set the active state to TRUE
     */
    bActive = TRUE;

    INEndPointNum = 0;
    OUTEndPoint = 0;

    /* Initialize the lock mutexs for accessing the cache */
    pCacheMutex = new CWACPMutex(NULL);

    /* Create the queue managers for the IN/OUT and WACP queues */
    pCacheQMgr = new uint8QManager(Cache, USB_CACHE_SIZE);

    bConnected = FALSE;
}

CUSBCommSvr::~CUSBCommSvr()
{   
    if (pDisconnectMutex)
    {
        pDisconnectMutex->Lock(INFINITE);
    }

    /* Reset the bActive flag to kill the StartClient Thread */
    bActive = FALSE;

    /* Wait until the StartClient Thread has started. */
    if (pThreadEvent != NULL)
    {
        pThreadEvent->Wait(INFINITE);
        pThreadEvent->Reset();
        delete(pThreadEvent);
        pThreadEvent = NULL;
    }

    if (pReadSem != NULL)
    {
        if (bConnected)
        {
            WA_USB_Disconnect(SourceID); 

            if (pDeviceMutex)
            {
                pDeviceMutex->Lock(INFINITE);
                pDeviceMutex->Release();
            }

            SYS_FREE(Addr.pDevice);
            Addr.pDevice = NULL;
        }

        /* Signal the reader thread to stop execution */
        pReadSem->Raise();
    }

    if (pDisconnectMutex != NULL)
    {
        pDisconnectMutex->Release();
    }

    if (pClientExitMutex != NULL)
    {
        /* Wait for the CUSBCommSvr threads to complete before completing the destructor */
        pClientExitMutex->Lock(INFINITE);
        pClientExitMutex->Release();

        /* Destroy the Mutexes */
        delete(pClientExitMutex);
        pClientExitMutex = NULL;
    }

    if (pDeviceMutex)
    {
        delete(pDeviceMutex);
        pDeviceMutex = NULL;
    }


    if (pDisconnectMutex != NULL)
    {
        /* Destroy the Mutexes */
        delete(pDisconnectMutex);
        pDisconnectMutex = NULL;
    }

    if (pReadSem != NULL)
    {
        /* Destroy the Semaphore */
        delete(pReadSem);
        pReadSem = NULL;
    }
    
    if (pCacheMutex != NULL)
    {
        delete(pCacheMutex);
        pCacheMutex = NULL;
    }

    if (pCacheQMgr != NULL)
    {
        delete(pCacheQMgr);
        pCacheQMgr = NULL;
    }
}

/*================================================================================
CUSBCommSvr GetSourceID: Returns the SourceID of the object
================================================================================*/
char* CUSBCommSvr::GetSourceID()
{
    return((char*)SourceID);
}
/*================================================================================
CUSBCommSvr SetSourceID: Sets the SourceID
================================================================================*/
boolean CUSBCommSvr::SetSourceID(char* pSourceID)
{   
    boolean Result = FALSE;
    if (pSourceID != NULL)
    {
        uint16 Size;

        memset(SourceID, 0, sizeof(SourceID));
        Size = (uint16)strlen(pSourceID);
        if (Size > sizeof(SourceID))
            Size = sizeof(SourceID);

        memcpy(SourceID, pSourceID, Size);
        Result = TRUE;
    }

    return (Result);
}
/*================================================================================
CUSBCommSvr GetPortNum: Returns the Port used by the object
================================================================================*/
uint16 CUSBCommSvr::GetPortNum()
{
    return(Port);
}

/*================================================================================
CUSBCommSvr SetPortNum: Sets the Port used by the object
================================================================================*/
boolean CUSBCommSvr::SetPortNum(uint16 NewPort)
{
    boolean Result = TRUE;

    Port = NewPort;

    return (Result);
}

/*================================================================================
CUSBCommSvr SetConfig: Updates the addressing information of the object
================================================================================*/
void CUSBCommSvr::SetConfig(void* pAddr)
{
    USBAddr* pConfig = (USBAddr*)pAddr;
    int32 i;

    if (pConfig != NULL)
    {
        Addr.pDevice = pConfig->pDevice;

        /* Count the number of endpoints */
        for (i = Addr.pDevice->NumEndPoints - 1; i >= 0 ;i--)
        {
            if (Addr.pDevice->EndPoints[i].Direction == ENDPOINT_IN)
            {
                INEndPointNum++;
            }
        }
 
        /* Look for the first OUT endpoint */
        for (i = 0; i < Addr.pDevice->NumEndPoints;i++)
        {
            if (Addr.pDevice->EndPoints[i].Direction == ENDPOINT_OUT)
            {
                OUTEndPoint = Addr.pDevice->EndPoints[i].EndPointID;
                break;
            }
        }
    }
}

/*================================================================================
CUSBCommSvr GetConfig: Returns the configuration information
================================================================================*/
void* CUSBCommSvr::GetConfig()
{
    return NULL;
}

/*================================================================================
CUSBCommSvr GetStackType: Returns the type of stack that is being used as defined in
    the enumeration STACK_TYPE in Common.h
================================================================================*/
STACK_TYPE CUSBCommSvr::GetStackType()
{
    return(StackType);
}

/*================================================================================
CUSBCommSvr Connect: Starts the USB connection by opening the send/receive streams
    on the USB stack.
================================================================================*/
boolean CUSBCommSvr::Connect(uint8* pSourceID, int PortNum)
{
    boolean Result = FALSE;
    int32  RetVal;
    WAUSBCONFIG ConfigData;

    /* Set the SourceID */
    SetSourceID((char*)pSourceID);

    /* Set the Port Number */
    Port = (uint16)PortNum;

    RetVal = WA_USB_Connect((char*)pSourceID, (RECVDATAPROC)this->RecvData, this);
    if (RetVal == 0)
    {
        bConnected = TRUE;

        ConfigData.ReceiveTimeOut = USB_RECEIVE_TIMEOUT;

        WA_USB_Configure((char*)pSourceID, &ConfigData);
        
        Result = TRUE;
    }
    else
    {
        pThreadEvent->Raise();
    }

    return Result;
}

/*================================================================================
CUSBCommSvr SendMsg: Sends the passed in buffer.
================================================================================*/
boolean CUSBCommSvr::SendMsg(uint8* pBuf, uint32 pBufSize)
{
    uint32 Result = 0;

    Result = WA_USB_Send(SourceID, OUTEndPoint, pBuf, pBufSize);

#if defined(DEBUG_PRINT)
    if (Result)
    {
        for (uint32 i=0; i < pBufSize;i++)
        {
            if ( i % 25 == 0)
                SYS_PRINTF("\n");
            SYS_PRINTF("%.2x ", (uint8)pBuf[i]);
        }
        SYS_PRINTF("\n\n");
    }
    else
    {
        SYS_PRINTF("\n\nERROR!!! Unable to send USB message\n");
    }
#endif  

    return (Result ? TRUE : FALSE);
}   

/* Return FALSE if any of the passed in parameters are NULL or the cache is empty */
boolean CUSBCommSvr::ReadData(char* pBuf, uint32 BytesToRead)
{
    boolean Result = FALSE;
    uint32  DataSize = 0;
    if (pCacheQMgr && pBuf)
    {
        DataSize = pCacheQMgr->get_num_used();

#if defined(DEBUG_PRINT)
        SYS_PRINTF("CUSBCommSvr::ReadData DataSize=%d, BytesToRead=%d\n", DataSize,BytesToRead);
#endif
        if (DataSize < BytesToRead)
        {
            if (pReadSem->Wait(INFINITE) == WAIT_SUCCESS)
            {
                DataSize = pCacheQMgr->get_num_used();
            }
        }

        if (DataSize >= BytesToRead)
        {
            pCacheMutex->Lock(INFINITE);
            Result = pCacheQMgr->dequeue_array((uint8*)pBuf, BytesToRead);
            pCacheMutex->Release();
#if defined(DEBUG_PRINT)
            SYS_PRINTF("CUSBCommSvr::ReadData Read %d bytes of %d, Result = %d\n", BytesToRead, DataSize, Result);
#endif
        }
        else
        {
#if defined(DEBUG_PRINT)
            SYS_PRINTF("CUSBCommSvr::TimeOut ReadData Not enough data to read %d bytes of %d\n", BytesToRead, DataSize);
#endif
        }
    }
    return Result;
}

/*================================================================================
Static CUSBCommSvr RecvData: Callback function for receiving data.
================================================================================*/
uint32 WAUSB_CALLBACK CUSBCommSvr::RecvData(uint32 Status, char* pDevName, int32 EndPointID, 
                           void* pRecvData, int32 RecvLength, void* pUserRecvData)
{
    CUSBCommSvr* pCommSvr = (CUSBCommSvr*)pUserRecvData;
    int     Result = FALSE;

#if defined(DEBUG_PRINT)
    SYS_PRINTF("\nCUSBCommSvr RecvData: pCommSvr %p, EndpointID %d, status = 0x%X, pDevName = %s\n", 
                pCommSvr, EndPointID, Status, pDevName);
#endif

    TOUCH(pDevName);
    TOUCH(EndPointID);

    if((pCommSvr->bActive == TRUE) && ((Status == ERROR_SUCCESS) || (Status == ERROR_SEM_TIMEOUT)))
    {
        if ((Status == ERROR_SUCCESS) && 
            ((RecvLength < RCVBUFSIZE) && RecvLength))
        {      
            pCommSvr->pCacheMutex->Lock(INFINITE);

            /* Add the message to the CacheQMgr*/
            Result = pCommSvr->pCacheQMgr->enqueue_array((uint8*)pRecvData, RecvLength);

            pCommSvr->pCacheMutex->Release();

#if defined(DEBUG_PRINT)
            SYS_PRINTF("\nCUSBCommSvr RecvData: EndPointID %d, Adding %d bytes to the cache\n", 
                        EndPointID, RecvLength);
#endif
            if (Result)
            {
                /* Release the Read semaphore to request the next packet */
                pCommSvr->pReadSem->Raise();
            }
            else
            {
#if defined(DEBUG_PRINT_STATUS)
                SYS_PRINTF("\nCUSBCommSvr RecvData: EndPointID %d, Cache Write failed for %d, Free bytes %d\n", 
                    EndPointID, RecvLength, pCommSvr->pCacheQMgr->get_num_free());
#endif
            }

        }/* if ((RecvLength < RCVBUFSIZE) && RecvLength) */

        if(pCommSvr->bActive == TRUE)
        {
            /* Queue up another request */
            Result = WA_USB_RecvAsync(pCommSvr->SourceID, EndPointID, (uint8*)pRecvData, RCVBUFSIZE);
        }

#if defined(DEBUG_PRINT)
        SYS_PRINTF("CUSBCommSvr %p EndPointID %d, WA_USB_RecvAsync ret. Result = %d\n", 
            pCommSvr, EndPointID, Result);
#endif

    }/* if(pCommSvr->bActive == TRUE) */

    /* Return 0 for SUCCESS */
    return (SUCCESS);
}


/*================================================================================
Static CUSBCommSvr DeviceRemoval: Callback function for device removals.
    Always returns SUCCESS(0).
================================================================================*/
uint32 WAUSB_CALLBACK CUSBCommSvr::DeviceRemoval(char* pDevName, void* pUserArrivalData)
{
    CUSBCommSvr* pUSBCommSvr = (CUSBCommSvr*)pUserArrivalData;

    if (pDevName)
    {
#if defined(TRACE_SERVER_ON)
        SYS_PRINTF("CUSBCommSvr DeviceRemoval: pDevName = %s\n", pDevName);
#endif

        if (pUSBCommSvr)
        {
            pUSBCommSvr->pTerminate(pUSBCommSvr->pSemName, pDevName, 0);
        }
    }

    /* Return 0 for SUCCESS */
    return SUCCESS;
}

/*================================================================================
Static CUSBCommSvr DeviceArrival: Callback function for accepting new device 
    arrivals. Always returns SUCCESS.
================================================================================*/
uint32 WAUSB_CALLBACK CUSBCommSvr::DeviceArrival(PDEVICE_DESC pDeviceDesc,void* pUserArrivalData)
{
    CUSBCommSvr*    pUSBCommSvr = (CUSBCommSvr*)pUserArrivalData;
    uint32          PacketLen;
    uint8*          pPacketBuf;
    USBAddr         NewClntAddr;
    SessionArgs     NewSessionArgs;
    PDEVICE_DESC    pDevice = NULL;
        
    if (pDeviceDesc && pUSBCommSvr)
    {
        if (pUSBCommSvr->bActive == TRUE)
        {
            const uint8 RndzConnect[] = /* Faked WACONNECT message */
            {   0x00,0x1D,0x0B,0x01,0x00,0x00,0x00,0x12,
                0x00,0x00,0x00,0x00,0x0B,0x52,0x4E,0x44,
                0x5A,0x43,0x4F,0x4E,0x4E,0x45,0x43,0x54,
                0x57,0xF3 
            };

#if defined(TRACE_SERVER_ON_STATUS)
            SYS_PRINTF("CUSBCommSvr DeviceArrival: pDevName = %s\n", pDeviceDesc->szDevName);
            SYS_PRINTF("CUSBCommSvr::DeviceArrival Vendor ID = %d:\n", pDeviceDesc->VendorID);
            SYS_PRINTF("CUSBCommSvr::DeviceArrival ProductID ID = %d:\n", pDeviceDesc->ProductID);
            SYS_PRINTF("CUSBCommSvr::DeviceArrival ProductVersion = %d:\n", pDeviceDesc->ProductVersion);
            SYS_PRINTF("CUSBCommSvr::DeviceArrival DeviceSpeed = %d:\n", pDeviceDesc->DeviceSpeed);
            SYS_PRINTF("CUSBCommSvr::DeviceArrival NumEndPoints = %d:\n", pDeviceDesc->NumEndPoints);
            SYS_PRINTF("CUSBCommSvr::DeviceArrival EndPointType = %d:\n", pDeviceDesc->EndPoints[0].EndPointType);
            SYS_PRINTF("CUSBCommSvr::DeviceArrival EndPointID = %d:\n", pDeviceDesc->EndPoints[0].EndPointID);
            SYS_PRINTF("CUSBCommSvr::DeviceArrival Direction = %d:\n", pDeviceDesc->EndPoints[0].Direction);
#endif

            /* Setup default parameters for new connection setup */
            NewSessionArgs.Port = pUSBCommSvr->Port;
            NewSessionArgs.StackType = USB;

            /* Copy the device structure.. freed in USBCommSvr destrutor */
            pDevice = (PDEVICE_DESC)SYS_MALLOC(sizeof(DEVICE_DESC) 
                + (sizeof(ENDPOINT_DESC) * (pDeviceDesc->NumEndPoints - 1)));
            if (pDevice)
            {
                memcpy(pDevice, pDeviceDesc, sizeof(DEVICE_DESC));
                for (uint16 i=0; i < pDeviceDesc->NumEndPoints; i++)
                {
                    pDevice->EndPoints[i].EndPointID = pDeviceDesc->EndPoints[i].EndPointID;
                    pDevice->EndPoints[i].EndPointType = pDeviceDesc->EndPoints[i].EndPointType;
                    pDevice->EndPoints[i].Direction = pDeviceDesc->EndPoints[i].Direction;
                }

                NewClntAddr.pDevice = pDevice;
                NewSessionArgs.pAddr = (void*)&NewClntAddr; 
                NewSessionArgs.bByPass= FALSE;
                NewSessionArgs.pSourceID = pDeviceDesc->szDevName;

                pUSBCommSvr->SetSourceID(pDeviceDesc->szDevName);
                pPacketBuf = GeneratePacket(&PacketLen, pUSBCommSvr->Port,0,0, (uint8*)RndzConnect, sizeof(RndzConnect));
                pUSBCommSvr->pSetRecvMsg(pUSBCommSvr->pSemName, pUSBCommSvr->GetSourceID(), pPacketBuf, PacketLen, (void*)&NewSessionArgs);
                
                /* Free up the memory allocated by GeneratePacket */
                SYS_FREE(pPacketBuf);
            }
        }
    }/* if (pDeviceDesc && pUSBCommSvr) */

    /* Return 0 for SUCCESS */
    return SUCCESS;
}

/*================================================================================
Static CUSBCommSvr DeviceNotify: Callback function for accepting new device arrivals.
    Always returns SUCCESS(0).
================================================================================*/
uint32 WAUSB_CALLBACK CUSBCommSvr::DeviceNotify(PDEVICE_DESC pDeviceDesc, void* pUserArrivalData)
{
    TOUCH(pDeviceDesc);
    TOUCH(pUserArrivalData);

#if defined(TRACE_SERVER_ON)
    if (pDeviceDesc)
    {
        SYS_PRINTF("CUSBCommSvr DeviceNotify: pDevName = %s\n", pDeviceDesc->szDevName);
        SYS_PRINTF("CUSBCommSvr::DeviceNotify Vendor ID = %d:\n", pDeviceDesc->VendorID);
        SYS_PRINTF("CUSBCommSvr::DeviceNotify ProductID ID = %d:\n", pDeviceDesc->ProductID);
        SYS_PRINTF("CUSBCommSvr::DeviceNotify ProductVersion = %d:\n", pDeviceDesc->ProductVersion);
        SYS_PRINTF("CUSBCommSvr::DeviceNotify DeviceSpeed = %d:\n", pDeviceDesc->DeviceSpeed);
        SYS_PRINTF("CUSBCommSvr::DeviceNotify NumEndPoints = %d:\n", pDeviceDesc->NumEndPoints);
        SYS_PRINTF("CUSBCommSvr::DeviceNotify EndPointType = %d:\n", pDeviceDesc->EndPoints[0].EndPointType);
        SYS_PRINTF("CUSBCommSvr::DeviceNotify EndPointID = %d:\n", pDeviceDesc->EndPoints[0].EndPointID);
        SYS_PRINTF("CUSBCommSvr::DeviceNotify Direction = %d:\n", pDeviceDesc->EndPoints[0].Direction);
    }
#endif

    /* Return 0 for SUCCESS */
    return SUCCESS;
}

/*================================================================================
Static CUSBCommSvr StartClient: Handles the start of the new client connection thread.
    The function loops continuously waiting for messages from the stack driver. 
    Once a message is received, the message is sent via the callback function 
    to be processed by the WACP stack. The loop continues until the bActive flag is
    set to FALSE;
================================================================================*/
uint32 WACPAPI CUSBCommSvr::StartClient(void* pData)
{
    CUSBCommSvr* pCommSvr = (CUSBCommSvr*)pData;
    char    DataBuf[RCVBUFSIZE];
    uint32  BytesLeft;
    uint32  PacketLen; 

#if defined(DEBUG_PRINT)
    SYS_PRINTF("CUSBCommSvr StartClient: Thread starting %p %s\n", pCommSvr, pCommSvr->GetSourceID());
#endif
    if (pCommSvr)
    {
        if (pCommSvr->bActive == TRUE)
        {
            uint32 Result = TRUE;
            uint32 Index = 0;
            uint32 i,j;
            uint32 BufCount = 0;
            boolean bError = FALSE;
            AsyncRequest** pRequestList;

            /* Lock this mutex until this thread terminates */
            pCommSvr->pClientExitMutex->Lock(INFINITE);
            pCommSvr->pDeviceMutex->Lock(INFINITE);

            /* Create separate data buffers for each endpoint read request.
            * More requests will facilitate data streaming, but at the cost
            * of more data buffers being allocated. TODO: Consider making this a 
            * configurable parameter via target.h
            */
            pRequestList = (AsyncRequest**)SYS_MALLOC(pCommSvr->INEndPointNum * sizeof(AsyncRequest*) * USB_MAX_REQUEST_QUEUE);
            if (pRequestList && pCommSvr->Addr.pDevice)
            {
                /* Loop through all IN endpoints */
                for (i = 0; i < (uint32)pCommSvr->Addr.pDevice->NumEndPoints; i++)
                {
                    /* Check for IN endpoints and any errors along the way */
                    if (pCommSvr->Addr.pDevice->EndPoints[i].Direction == ENDPOINT_IN)
                    {
                        /* create a separate buffer for each endpoint request */
                        for (j = 0; j < USB_MAX_REQUEST_QUEUE; j++)
                        {
                            pRequestList[BufCount] = (AsyncRequest*)SYS_MALLOC(sizeof(AsyncRequest));
                            if (pRequestList[BufCount])
                            {
                                /* Store the EndPoint ID */
                                pRequestList[BufCount]->EndpointID = pCommSvr->Addr.pDevice->EndPoints[i].EndPointID;

                                /* Create a data buffer for this request */
                                pRequestList[BufCount]->pDataBuf = (uint8*)SYS_MALLOC(RCVBUFSIZE);
                                if (pRequestList[BufCount])
                                {
                                    BufCount++;
                                }
                                else
                                {
                                    /* A single out of memory error is enough to terminate the thread.
                                    * Set Result to TRUE and exit the thread. 
                                    */
                                    bError = TRUE;
                                }
                            }
                            else
                            {
                                /* A single out of memory error is enough to terminate the thread.
                                * Set Result to TRUE and exit the thread. 
                                */
                                bError = TRUE;
                            }
                        }/* for (j = 0; j < MAX_USB_REQUEST_QUEUE; j++) */

                    }/* if ((pCommSvr->Addr.pDevice->EndPoints[i].Direction == ENDPOINT_IN) && (Result)) */

                    /* If bError is TRUE then we found a memory allocation error */
                    if (bError)
                        break;

                }/* for (i = 0; i < pCommSvr->Addr.pDevice->NumEndPoints; i++) */

            }/* if (pRequestList) */

            pCommSvr->pDeviceMutex->Release();

            if (bError == FALSE)
            {
                for (i = 0; i < BufCount && (pCommSvr->bActive == TRUE); i++)
                {
                    Result = WA_USB_RecvAsync(pCommSvr->SourceID, pRequestList[i]->EndpointID, 
                                            pRequestList[i]->pDataBuf, RCVBUFSIZE);
                    if (Result != ERROR_IO_PENDING)
                    {
#if defined(DEBUG_PRINT)
                        SYS_PRINTF("WA_USB_RecvAsync Result = %d\n", Result);
#endif
                        break;
                    }
                }

                /* Allow the Destructor to be called */
                pCommSvr->pThreadEvent->Raise();

                /* Wait on the Recv callback function to signal when to request more data reads */
                pCommSvr->pReadSem->Wait(INFINITE);
                while(pCommSvr->bActive == TRUE)
                {
#if defined(DEBUG_PRINT)
                    SYS_PRINTF("CUSBCommSvr StartClient: Waiting for data 1\n");
#endif
                    /* Loop until we get a valid preamble or an error */
                    Index = 0;
                    memset(DataBuf, 0, sizeof(DataBuf));
                    while(Index != PREAMBLE_SIZE)
                    {
                        Result = pCommSvr->ReadData((char*)&DataBuf[Index], 1);
                        if (Result)
                        {
                            switch(Index)
                            {
                                case 0:
                                    if (DataBuf[Index] == CTL_W)
                                        Index++;
                                    break;
                                case 1:
                                    if (DataBuf[Index] == CTL_A)
                                        Index++;
                                    else
                                        Index = 0;
                                    break;
                                case 2:
                                    if ((DataBuf[Index] == CTL_C) || (DataBuf[Index] == CTL_L))
                                        Index++;
                                    else
                                        Index = 0;
                                    break;
                            }
                        }
                        else
                        {
                            break;
                        }
                    }
                    if (Result)
                    {
                        Result = pCommSvr->ReadData((char*)&DataBuf[PREAMBLE_SIZE], MSG_HEADER - PREAMBLE_SIZE);
                        if (Result)
                        {
                            PacketLen = ReadInt32((uint8*)&DataBuf[PREAMBLE_SIZE]);
#if defined(DEBUG_PRINT)
                            SYS_PRINTF("CUSBCommSvr StartClient: PacketLen = %d,  cache size %d\n", PacketLen, pCommSvr->pCacheQMgr->get_num_used());
#endif
                            if (PacketLen <= RCVBUFSIZE)
                            {
                                /* Calculate the remaining Bytes to receive */
                                BytesLeft = PacketLen - MSG_HEADER;

                                /* Loop until we get the full Packet */
                                while ((pCommSvr->bActive == TRUE) && (BytesLeft > 0))
                                {
                                    /* Receive the rest of the Packet */
                                    Result = pCommSvr->ReadData((char*)&DataBuf[MSG_HEADER], BytesLeft);
                                    if (Result)
                                    {
                                        break;
                                    }
                                    else
                                    {
#if defined(DEBUG_PRINT)
                                        SYS_PRINTF("CUSBCommSvr StartClient: Waiting for data 2\n");
#endif
                                        /* Wait on the Recv callback function to signal when to request more data reads */
                                        pCommSvr->pReadSem->Wait(INFINITE);
                                    }
                                }

#if defined(DEBUG_PRINT)
                                SYS_PRINTF("\n\nNew Msg:\n");
                                for (uint32 i=0; i < PacketLen;i++)
                                {
                                    if ( i % 25 == 0)
                                        SYS_PRINTF("\n");
                                    SYS_PRINTF("%.2x ", (uint8)DataBuf[i]);
                                }
                                SYS_PRINTF("\n\n");
                                SYS_PRINTF("New Msg PacketLen = %d, cache size %d\n", PacketLen, pCommSvr->pCacheQMgr->get_num_used());
#endif
                                /* Update the Recv IN_queue with the new message */
                                pCommSvr->pSetRecvMsg(pCommSvr->pSemName, pCommSvr->GetSourceID(), (uint8*)DataBuf, PacketLen, NULL);

                            }/* if (PacketLen < RCVBUFSIZE) */
#if defined(DEBUG_PRINT_STATUS)
                            else
                            {
                                SYS_PRINTF("Invalid PacketLen %d\n", PacketLen);
                                for (uint32 i=0; i < MSG_HEADER; i++)
                                {
                                    if ( i % 25 == 0)
                                        SYS_PRINTF("\n");
                                    SYS_PRINTF("%.2x ", (uint8)DataBuf[i]);
                                }
                            }
#endif
                        }/* if (Result) */
                    }/* if (Result) */
                }/* while(pCommSvr->bActive == TRUE) */
            }/* if if (bError == FALSE) */
            else
            {
                /* Allow the Destructor to be called */
                pCommSvr->pThreadEvent->Raise();

                pCommSvr->pDeviceMutex->Release();
            }

            pCommSvr->pDisconnectMutex->Lock(INFINITE);
            if (bError == FALSE) 
            {
                /* Free up the data buffers and the AsyncRequest structures */
                for (i = 0; i < BufCount; i++)
                {
                    SYS_FREE(pRequestList[i]->pDataBuf);
                    SYS_FREE(pRequestList[i]);
                }

                SYS_FREE(pRequestList);
            }
            pCommSvr->pDisconnectMutex->Release();

            /* Release the exit mutex */
            pCommSvr->pClientExitMutex->Release();
        }
        else
        {
            /* Release the exit mutex */
            pCommSvr->pThreadEvent->Raise();
        }
    }

#if defined(DEBUG_PRINT)
    SYS_PRINTF("CUSBCommSvr StartClient: Thread exiting %p\n", pCommSvr);
#endif

    return TRUE;
}

/*================================================================================
CUSBCommSvr StartServer: Listens for USB connections
================================================================================*/
uint32 WACPAPI CUSBCommSvr::StartServer(void* pData)
{
    CUSBCommSvr*    pUSBCommSvr = (CUSBCommSvr*)pData; 
    CWACPEvent*     pExitEvent;
    char            Buffer[EVENT_BUF_SIZE];
    uint32           Result;

    pUSBCommSvr->pThreadEvent->Raise();

    /* Clear the buffer */
    memset(Buffer, 0, EVENT_BUF_SIZE);
    
    /* Raise an event to signify the end of the client session */
    sprintf(Buffer, END_SERVER, SYS_GETPID());
    pExitEvent = new CWACPEvent(Buffer, TRUE);

    Result = WA_USB_OpenSession((DEVICEARRIVALPROC)pUSBCommSvr->DeviceArrival, pUSBCommSvr, 
                                (DEVICEREMOVALPROC)pUSBCommSvr->DeviceRemoval, pUSBCommSvr,
                                (DEVICEFREEPROC)pUSBCommSvr->DeviceNotify, pUSBCommSvr); 

    /* Check for a raised ExitEvent.*/
    pExitEvent->Wait(INFINITE);

    /* Clean up the ExitEvent */
    delete(pExitEvent);

    /* Close the USB Session. */
    WA_USB_CloseSession();

    /* Delete the class instance spawned in CCommSvr::StartServer() */
    delete(pUSBCommSvr);

    return TRUE;
}

/* End of File */





