/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CWACPApp.cpp                                                      //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CWACPApp.h"
#include "CMOMPbaseclass.h"

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
    Default constructor
    CWACPApp Start up: All available servers are instantiated here. i.e Command Servers,
    Request Servers, Response Servers, etc.

*********************************************************************************/
CWACPApp::CWACPApp(boolean (*SendPacket)(char*, char*, uint8*, uint32),
                   uint32 (*RecvPacket)(char*, char**, uint8**),
                   boolean (*MsgFunc)(char*, uint16, void*, uint16),
                   CLinkTable* pNewCLinkTable,
                   LinkStruct* pLinkStruct)
{
    /* Set the pointer to the link table and the CLinkTable instance */
    pLnkStruct = pLinkStruct;
    pCLinkTable = pNewCLinkTable;

    bTerminated = FALSE;
    bAppInit = FALSE;

    pCallBackObj = (CMOMPBaseClass*)pLnkStruct->pCallBackObj;
    pCMOMPBaseClass = NULL;

    /* Set the Link Layer callback function for receiving a packet */
    pRecvPacket = RecvPacket;

    /* Initialize the RNDZStub */
    RNDZStub.Init(this);

    /* Initialize the Object Servers */
    TrapObjSvr.Init(SendPacket, &TraceSvr, pLnkStruct);
    FmRouter.Init(&TraceSvr, &TrapObjSvr, this);
    ConObjSvr.Init(SendPacket, MsgFunc, &TraceSvr, pLnkStruct);
    RendezvousSvr.Init(&TrapObjSvr, &TraceSvr, SendPacket, pLnkStruct);

    /* This message server is used for all Family types registered by the application */
    DataObjSvr.Init(&TrapObjSvr, &TraceSvr, SendPacket, pLnkStruct);

#if defined(TRACE_SERVER_ON)
    //================================================================
    /* Open the Trace file here so we can start writing to it */
    TraceSvr.OpenFile(TRACE_WACP_LOG, pLnkStruct->pSemName);
    TraceSvr.GenerateXSLFile();
    TraceSvr.EnableLogging(TRUE, TRUE);
    //================================================================
#endif

    /* Start up the Data thread which process incoming messages */
    ThreadStatus = TRUE;
    StartDataThread();

    /* Register the RNDZ Object Server callbacks */
    RegisterServer(FmRENDEZVOUS, 
                   &RendezvousSvr,
                   RxRNDZMessageWrapper);

    /* Register the TRAP Object Server callbacks */
    RegisterServer(FmTRAP, 
                   &TrapObjSvr, 
                   RxErrorWrapper);

    /* Register the CONNECTION Object Server callbacks */
    RegisterServer(FmCONNECTION, 
                   &ConObjSvr, 
                   RxConnWrapper, 
                   this);

    /* Register the generic callbacks for all Family data processing */
    RegisterServer(FmMAX, 
                   &DataObjSvr,
                   RxMessageWrapper);

    /* Create the start and exit events */
    pExitEvent = new CWACPEvent(NULL, TRUE);
    pStartEvent = new CWACPEvent(NULL, TRUE);
}

CWACPApp::~CWACPApp()
{
    CWACPSemaphore  DataSemaphore(pLnkStruct->pSemName, 0, MAX_SEM_COUNT);

    /* Terminate the threads by setting ThreadStatus to FALSE */
    ThreadStatus = FALSE;

    /* Raise a fake DataSemaphore to allow the thread to unblock and check the thread status */
    DataSemaphore.Raise();

    /* Wait until the DataThread checks the ThreadStatus and terminates */
    DataThreadMutex.Lock(INFINITE);
    DataThreadMutex.Release();

    /* Only call AppEnd if AppInit has been previously called */
    if (bAppInit == TRUE)
    {
        /* Callback into the calling object notifying the application is coming to a close */
        pCallBackObj->AppEndInt(this);
    }

    /* Clean up the event object */
    if(pExitEvent != NULL)
    {
        delete(pExitEvent);
        pExitEvent = NULL;
    }

    /* Clean up the Start Event object */
    if(pStartEvent != NULL)
    {
        delete(pStartEvent);
        pStartEvent = NULL;
    }
}

/*================================================================================
Static Callback function wrapper for handling RNDZ messages
================================================================================*/
uint32 CWACPApp::RxRNDZMessageWrapper(char* pSourceID, uint32 ClassID, uint8* pBuf, void* pWACPApp)
{
    uint32  Result = TRUE;

    /* Call back into the RNDZStub class to handle a RNDZ Status message */
    if (pWACPApp)
    {
        Result = ((CWACPApp*)pWACPApp)->RNDZStub.RecvMessage(pSourceID, ClassID, pBuf);
    }
    return (Result);
}

/*================================================================================
Static Callback function wrapper for handling Command, Request, Response, 
Status, Event, and Data Object messages
================================================================================*/
uint32 CWACPApp::RxMessageWrapper(char* pSourceID, uint32 ClassID, uint8* pBuf, void* pWACPApp)
{
    uint32 Result = 0;
    ConnArgs* pConnArgs;

    /* Check for valid linktable entry */
    pConnArgs = ((CWACPApp*)pWACPApp)->pCLinkTable->GetEntry(pSourceID);
    if (pConnArgs)
    {
        /* Check to see if we passed RNDZ authentication or if it's disabled */
        if ((pConnArgs->bRNDZStatus) || (pConnArgs->bRNDZDisabled == TRUE))
        {
            /* Call back into the current MOMPBaseclass if it exists to notify the
             * application stub class that a data message has arrived
             */
            if (((CWACPApp*)pWACPApp)->pCMOMPBaseClass)
            {
                Result = ((CWACPApp*)pWACPApp)->pCMOMPBaseClass->RecvMessage(pSourceID, ClassID, pBuf);
            }
            else
            {
                /* Otherwise callback into the original mompbaseclass passed into the stack */
                Result = ((CWACPApp*)pWACPApp)->pCallBackObj->RecvMessage(pSourceID, ClassID, pBuf);
            }
        }
#if defined(TRACE_SERVER_ON)
        else
        {
            ((CWACPApp*)pWACPApp)->TraceSvr.tprintf(TRACE_Type_ERROR, "CWACPApp", "RxMessageWrapper",
                                    0, 0, 0, "Unauthenticated Message: RNDZ Authentication incomplete");
        }
#endif
    }

    return (Result);
}
/*================================================================================
Static Callback function wrapper for handling Error messages
================================================================================*/
uint32 CWACPApp::RxErrorWrapper(char* pSourceID, uint8 ErrorType, uint32 ClassID, 
                                uint8* pBuf, void* pWACPApp)
{
    uint32 Result = 0;

    /* Call back into the current MOMPBaseclass if it exists to notify the
     * application stub class that an error message has arrived
     */
    if (((CWACPApp*)pWACPApp)->pCMOMPBaseClass)
    {
        Result = ((CWACPApp*)pWACPApp)->pCMOMPBaseClass->RecvError(pSourceID, ErrorType, ClassID, pBuf);
    }
    else
    {
        /* Otherwise callback into the original mompbaseclass passed into the stack */
        Result = ((CWACPApp*)pWACPApp)->pCallBackObj->RecvError(pSourceID, ErrorType, ClassID, pBuf);
    }

    return (Result);
}

/*================================================================================
Static Callback function wrapper for handling Connection messages
================================================================================*/
uint32 CWACPApp::RxConnWrapper(char* pSourceID, uint8 ConnType, uint8* pBuf, void* pWACPApp)
{
    uint32 Result = 0;
    ConnArgs* pConnArgs;

    /* Call back into the current MOMPBaseclass if it exists to notify the
     * application stub class that a connection status message has arrived
     */
    if (((CWACPApp*)pWACPApp)->pCMOMPBaseClass)
    {
        Result = ((CWACPApp*)pWACPApp)->pCMOMPBaseClass->RecvConnection(pSourceID, ConnType, pBuf);
    }
    else
    {
        /* Otherwise callback into the original mompbaseclass passed into the stack */
        Result = ((CWACPApp*)pWACPApp)->pCallBackObj->RecvConnection(pSourceID, ConnType, pBuf);
    }

    /* Grab the connection information to check for an incomplete RNDZ authentication */
    pConnArgs = ((CWACPApp*)pWACPApp)->pCLinkTable->GetEntry(pSourceID);
    if (pConnArgs)
    {
        /* Terminate RNDZ if the RNDZ authentication has not passed */
        if(pConnArgs->bRNDZStatus == FALSE)
            ((CWACPApp*)pWACPApp)->RNDZTerminate(pSourceID, FALSE);
    }

    return (Result);
}
/*================================================================================
Terminate : 
================================================================================*/
void CWACPApp::RNDZTerminate(char* pSourceID, boolean bStatus)
{
    ConnArgs* pConnArgs;

    pConnArgs = pCLinkTable->GetEntry(pSourceID);
    if (pConnArgs)
    {
        pConnArgs->bRNDZStatus = bStatus;

        /* Close the RNDZ Connection */
        ConObjSvr.UpdateStatus(pConnArgs->SourceID, bStatus);

        /* Return the ConObjSvr option to the default for the WACP application */
        SetConnOptions(pConnArgs->SourceID, DEFAULT_CONN_SETTING, FALSE, 
                       DEFAULT_KEEPALIVE_INTERVAL, DEFAULT_TIMEOUT_INTERVAL);

        if (bStatus)
        {
            /* Force the TxOptions to be synced after a connect. This is to ensure that
            * after a UDP connection has been made, some data is sent to the server
            * to let it know a client is available.
            */
            SetConnOptions(pConnArgs->SourceID, DEFAULT_CONN_SETTING, TRUE, 
                           DEFAULT_KEEPALIVE_INTERVAL, DEFAULT_TIMEOUT_INTERVAL);
        }

        /* Notify the Server the connection status */
        if (pConnArgs->bSvrFlag == TRUE)
        {
            if (bStatus)
                RxConnWrapper(pSourceID, CONN_CEVNT_SpCONNECTED, NULL, this);
        }

        /* Send an event to the WACP App class to start */
        pStartEvent->Raise();
    }
}

/*================================================================================
Terminate : Terminates a WACP connection
================================================================================*/
void CWACPApp::Terminate()
{
    /* Set bTerminate to TRUE to force AppStart to stop checking for connections */
    bTerminated = TRUE;

    /* Send an event to the WACP App class to start in case we are waiting for RNDZ
     * to finish
     */
    pStartEvent->Raise();

    /* Send an event to the WACP App class to terminate */
    pExitEvent->Raise();
}

/*================================================================================
RendezvousConnect : Sends a command to initiate a Rendezvous session
================================================================================*/
boolean CWACPApp::RNDZConnect(char* pSourceID)
{
    boolean Result =  FALSE;
    ConnArgs* pConnArgs = NULL;

    if (pSourceID)
    {
        pConnArgs = pCLinkTable->GetEntry(pSourceID);
        if (pConnArgs)
        {
            const char* RNDZConnect="RNDZCONNECT";
            uint32 Size = (uint32)strlen(RNDZConnect); 

            /* Don't connect if already connected */
            if (!pConnArgs->bRNDZStatus)
            {
                if (pConnArgs->bSvrFlag == FALSE)
                {
                    /* Start the timeout check */
                    SetConnOptions(pSourceID, CONN_CFG_SpKEEPALIVECHECK, FALSE, 
                                DEFAULT_KEEPALIVE_INTERVAL, DEFAULT_TIMEOUT_INTERVAL);
#if defined(TRACE_SERVER_ON)
                    TraceSvr.tprintf(TRACE_Type_SEND, "CWACPApp", "RNDZConnect", FmRENDEZVOUS, GnREQUEST, RNDZ_CRST_SpSTART_RNDZ, 
                        "%s: \nSending Request RNDZ_CRST_SpSTART_RNDZ", pConnArgs->SourceID);
#endif
                    SendMsg(pSourceID, FmRENDEZVOUS, GnREQUEST, RNDZ_CRST_SpSTART_RNDZ, (uint8*)RNDZConnect, Size, FALSE);
                }

                /* Wait indefinitely for the event to fire */
                pStartEvent->Wait(pConnArgs->Timeout);
                pStartEvent->Reset();

                Result = pConnArgs->bRNDZStatus;

#if defined(TRACE_SERVER_ON)
                TraceSvr.tprintf(TRACE_Type_STATUS, "CWACPApp", "RNDZConnect", 0, 0, 0, 
                    "%s: \nRNDZ authentication %s", pSourceID, (pConnArgs->bRNDZStatus ? "Passed" : "Failed"));
#endif
            }/* if (!pConnArgs->bConnected) */
            else
            {
#if defined(TRACE_SERVER_ON)
                TraceSvr.tprintf(TRACE_Type_STATUS, "CWACPApp", "RNDZConnect", 0, 0, 0, 
                    "%s: \nAlready Connected", pConnArgs->SourceID);
#endif
                Result = TRUE;
                pStartEvent->Reset();
            }

        }/*if (pConnArgs)*/

    }/* if (pSourceID) */

    return (Result);
}

boolean CWACPApp::Open(ConnArgs** pConnArgs, uint8 NumConnections)
{
    boolean Result = TRUE;
    uint8 i;
    ConnArgs* pTempArgs;

    /* Check for existing connections that are open */
    for (i=0; i < NumConnections; i++)
    {
        pTempArgs = pCLinkTable->GetEntry(pConnArgs[i]->SourceID);
        if (pTempArgs)
        {
            if (pTempArgs->bConnected)
            {
                Result = FALSE;
                break;
            }
        }
    }

    /* Open the channels only if none of the new channels are open */
    if (Result)
    {
        for (i=0; i < NumConnections; i++)
        {
            /* Update Application CLinkTable. */
            Result = pCLinkTable->AddEntry(pConnArgs[i]);
            ConObjSvr.AddConnection(pConnArgs[i]->SourceID);
        }

        if (Result)
        {
            /* Call Interchange to open the connections */
            Result = ConObjSvr.Connect(pConnArgs, NumConnections);
        }
    }
    return(Result);
}

boolean CWACPApp::Connect(char* pSourceID)
{
    boolean Result = FALSE;

    Result = RNDZConnect(pSourceID);

    return (Result);
}

boolean CWACPApp::Close(char* pSourceID)
{
    return ConObjSvr.Close(pSourceID);
}

boolean CWACPApp::SendMsg(char* pSourceID, uint16 Family, uint8 Genus, uint8 Species, uint8* pBuf, uint32 BufSize, boolean bEncryptionOn)
{
    return FmRouter.SendMsg(pSourceID, Family, Genus, Species, pBuf, BufSize, bEncryptionOn);
}

boolean CWACPApp::SendTrap(char* pSourceID, uint32 MsgClassID, uint32 ErrClassID, uint8 ErrorSpecies, uint16 ErrorCode,
                           uint8* pData, uint16 DataSize)
{
    boolean Result = FALSE;
    uint8*  pBuf = NULL;
    uint16  BufSize = 0;

    CErrEStd ErrObj;
    ErrObj.set_Species(ErrorSpecies);

    /* Set the ErrorCode */
    ErrObj.SetErrorCode(ErrorCode);

    /* Set the ClassID */
    ErrObj.SetErrClassID(ErrClassID);

    /* Set the ClassID */
    if (pData)
    {
        ErrObj.SetData(pData, DataSize);
    }

    /* Get the Size of the error object */
    BufSize = (uint16)ErrObj.size_all();

    /* Allocate the proper Buffer Size */
    pBuf = (uint8*) SYS_MALLOC(BufSize);
    if (pBuf)
    {
        /* Serialize the error object */
        ErrObj.serialize_all(pBuf);
        Result = TrapObjSvr.TxError(pSourceID, MsgClassID, ErrorSpecies, pBuf, BufSize);
    }

    SYS_FREE(pBuf);
    pBuf = NULL;

    return (Result);
}

boolean CWACPApp::SendMsgProcTrap(char* pSourceID, uint32 ClassID, uint16 ErrorCode)
{
    boolean Result = FALSE;

    Result = SendTrap(pSourceID, ClassID, ClassID, CERR_SpERRMSGPROC, ErrorCode, 
             NULL, 0);

    return (Result);
}

boolean CWACPApp::SetConnOptions(char* pSourceID, CONN_CFG_SPECIES Options, 
                                 boolean bSyncFlag, time_t Interval, time_t TimeOut)
{
    boolean Result = FALSE;
    if (ConObjSvr.SetTxOptions(pSourceID, (uint8)Options, bSyncFlag))
    {
        if (ConObjSvr.SetKeepAliveInterval(pSourceID, Interval))
        {
            Result = ConObjSvr.SetTimeoutInterval(pSourceID, TimeOut);
        }
    }
    return (Result);
}

boolean CWACPApp::GetConnOptions(char* pSourceID, CONN_CFG_SPECIES* pOptions, time_t* pInterval, time_t* pTimeOut)
{
    boolean Result = FALSE;

    if ((pSourceID) && (pOptions) && (pInterval) && (pTimeOut))
    {
        *pOptions = (CONN_CFG_SPECIES)ConObjSvr.GetTxOptions(pSourceID);
        *pInterval = ConObjSvr.GetKeepAliveInterval(pSourceID);
        *pTimeOut = ConObjSvr.GetTimeoutInterval(pSourceID);
        Result = TRUE;
    }

    return (Result);
}

boolean CWACPApp::AppStart()
{
    boolean Result = TRUE;
    char* pServerSourceID = NULL;
    ConnArgs* pConnArgs;
    boolean bByPass = FALSE;

    if (pLnkStruct->pSemName)
    {
        /* Grab the connection information */
        pConnArgs = pCLinkTable->GetEntry(pLnkStruct->pSemName);
        if (pConnArgs)
        {
            bByPass = pConnArgs->bByPass;
            if (bByPass)
            {
                pServerSourceID = pLnkStruct->pSemName;
#if defined(TRACE_SERVER_ON_STATUS)
                TraceSvr.tprintf(TRACE_Type_STATUS, "CWACPApp", "AppStart", 0, 0, 0, 
                    "pSourceID %s: \nRNDZ ByPass found\n", pServerSourceID);
#endif
            }
        }
    }

    /* If this is a server baseline, then set the SourceID of the connection 
     * and pass it to the application. Clients do not have a SourceID since
     * it is set by the client application. Also wait until RNDZ completes
     * before calling AppInitInt if this is a server.
     */
    if ((pLnkStruct->bServer) && (bByPass == FALSE))
    {
        pServerSourceID = pLnkStruct->pSemName;
        Result = RNDZConnect(pServerSourceID);

#if defined(TRACE_SERVER_ON)
        TraceSvr.tprintf(TRACE_Type_STATUS, "CWACPApp", "AppStart", 0, 0, 0, 
            "pSourceID %s: \nCWACPApp %p Started\n", pServerSourceID, this);
#endif
    }


    if (Result)
    {
        bAppInit = TRUE;
        /* APP_INIT retrieves a void*  pointer to the calling process object 
        * and sets it to pCMOMPBaseClass
        */
        pCMOMPBaseClass = pCallBackObj->AppInitInt(this, pLnkStruct->bServer, pServerSourceID);

        /* Check for a valid object ptr */
        if (pCMOMPBaseClass)
            Result = TRUE;

        /* Wait indefinitely for the event to fire */
        pExitEvent->Wait(INFINITE);

        /* Reset the event to enable listening for it again */
        pExitEvent->Reset();
    }

#if defined(TRACE_SERVER_ON)
        TraceSvr.tprintf(TRACE_Type_STATUS, "CWACPApp", "AppStart", 0, 0, 0, 
                         "pSourceID %s: \nCWACPApp %p terminating \n", pServerSourceID, this);
#endif

    return(Result);
}




/*================================================================================
STATIC CWACPApp DataThread: Uses a semaphore to process data ready events from
    the exchange layer.
================================================================================*/
uint32 WACPAPI CWACPApp::DataThread(void* pData)
{
    /* Recreate the CWACPApp class from the pointer sent in pData. */
    CWACPApp* pWACPApp = (CWACPApp*)pData;

    CWACPSemaphore  DataSemaphore(pWACPApp->pLnkStruct->pSemName, 0, MAX_SEM_COUNT);
    pWACPApp->DataThreadMutex.Lock(INFINITE);

    /* For Server baselines: Add the new connection to the ConnObjSvr */
    if (pWACPApp->pLnkStruct->bServer == TRUE)
        pWACPApp->ConObjSvr.AddConnection(pWACPApp->pLnkStruct->pSemName);

    /* While the presentation layer exists */
    while(pWACPApp->GetThreadStatus() == (uint16)TRUE)
    {
        DataSemaphore.Wait(INFINITE);

        /* Process the message */
        pWACPApp->ProcessMsg();
    }

#if defined(TRACE_SERVER_ON)
    pWACPApp->TraceSvr.tprintf(TRACE_Type_STATUS, "CWACPApp", "DataThread",
                                  0, 0, 0, "Terminating CWACPApp DataThread");
#endif

    /* For Server baselines: Remove the connection from the ConnObjSvr */
    if (pWACPApp->pLnkStruct->bServer == TRUE)
        pWACPApp->ConObjSvr.RemoveConnection(pWACPApp->pLnkStruct->pSemName);

    pWACPApp->DataThreadMutex.Release();

    return TRUE;
}

/*================================================================================
CWACPApp  StartDataThread
================================================================================*/
void CWACPApp::StartDataThread()
{

    DThread.StartThread(this->DataThread, this, pLnkStruct->ThreadPriority);
    SYS_SLEEP(WAIT_COUNT);
}

/*================================================================================
CWACPApp  Thread Status
================================================================================*/
boolean CWACPApp::GetThreadStatus()
{
    return ThreadStatus;
}

/*================================================================================
CWACPApp ProcessMsg: Retrieves the message from the Link Layer and routes it to
the FmRouter.
================================================================================*/
boolean CWACPApp::ProcessMsg()
{
    uint32 RetVal = 0;
    uint8* pBuf;
    char* pSourceID = NULL;

    /* pRecvPacket will return the number of bytes received */
    RetVal = pRecvPacket(pLnkStruct->pSemName, &pSourceID, &pBuf);

    if(RetVal)
    {
        /* Turn off the keepalive messages when we receive a message */
        ConObjSvr.StopKeepAlive(pSourceID);

        FmRouter.RecvMsg(pSourceID, pBuf, RetVal);

        /* Free the memory allocated by CInterchange for the call into pRecvPacket */
        SYS_FREE(pBuf);
    }

    return RetVal ? TRUE: FALSE;
}

/*================================================================================
CWACPApp RegisterCallbacks: Passes the function call through to the FmRouter
================================================================================*/
boolean CWACPApp::RegisterServer(FAMILY Family, CObjSvr* pObjSvr, ...)
{
    boolean Result = FALSE;
    va_list vList = NULL;

    /* Add the Object server to the FmRouter table */
    FmRouter.registerServer(Family, pObjSvr);

    va_start(vList, pObjSvr);

    /* Register the callback function pointers */
    Result = pObjSvr->SetCallbacks(vList);
        
    va_end(vList);

    return Result;
}


/*================================================================================
CWACPApp GetGUID: Returns the CRNDZDStd object pointer saved for the connection 
corresponding with the passed in pSourceID.
================================================================================*/
CRNDZDStd* CWACPApp::GetGUID(char* pSourceID)
{
    CRNDZDStd* pGUIDStd = NULL;
    ConnArgs* pConnArgs;

    if (pSourceID)
    {
        /* Grab the connection information */
        pConnArgs = pCLinkTable->GetEntry(pSourceID);
        if (pConnArgs)
        {
            pGUIDStd = (CRNDZDStd*)pConnArgs->pGUIDStd;
        }
    }
    return (pGUIDStd);
}

/*================================================================================
CWACPApp GetStackType: Returns the stack type for the connection corresponding 
    with the passed in pSourceID.
================================================================================*/
STACK_TYPE CWACPApp::GetStackType(char* pSourceID)
{
    STACK_TYPE StackType = STACK_UNDEFINED;
    ConnArgs* pConnArgs;

    if (pSourceID)
    {
        /* Grab the connection information */
        pConnArgs = pCLinkTable->GetEntry(pSourceID);
        if (pConnArgs)
        {
            StackType = pConnArgs->StackType;
        }
    }
    return (StackType);
}
/* End of File */


