/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'*                            All rights reserved
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CInterchange.cpp:                                                 //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#include "CInterchange.h"

CInterchange* CInterchange::pThis = NULL;

/* Faked Termiante message */
const uint8 WacpTerminate[] = 
{   
    0x00,0x1E,0x01,0x00,0x00,
    0x00,0x00,0x07,0x00,0x00,
    0x00,0x00,0x00,0x61,0xAD
};

/* Faked CRC Error message */
const uint8 CRCError[] = 
{   
    0x00, 0x1c, 0x03, 0x04, /* CErrSvr ClassID */
    0x00, 0x35,             /* CErrSvr Payload Size */
    0x00, 0x04,             /* CErrSvr Static Struct Size */
    0x00, 0x00, 0x00, 0x00, /* CErrSvr Error ClassID */ 
    0x00, 0x00, 0x00, 0x29, /* CErrSvr Data Size */
          
        0x00, 0x72, 0x03, 0x01, /* CErrEStd ClassID */
        0x00, 0x23,             /* CErrEStd Payload Size */
        0x00, 0x64,             /* CErrEStd Version */
        0xc0,                   /* CErrEStd DataByte */
        0x00, 0x1a,             /* CErrEStd Static Size */
        0x00, 0x00, 0x00, 0x00, /* CErrEStd Error ClassID */
        0x00, 0x05,             /* CErrEStd Errorcode */
        0x43, 0x52, 0x43, 0x20, 0x4d, 0x69, 0x73, 0x6d, 0x61, 0x74, 
        0x63, 0x68, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
        0x00, 0x00,             /* CErrEStd Error Description */
        0x2b, 0x06,             /* CErrEStd CRC*/

    0x46, 0x29              /* CErrSvr CRC*/
};

/*================================================================================
    CONSTRUCTORS / DESTRUCTORS
================================================================================*/
/*********************************************************************************
CInterchange Start up: 
*********************************************************************************/

CInterchange::CInterchange()
{
}

CInterchange::~CInterchange()
{
    /* Close all Mutex and/or file handles */
    Close();

    /* Delete the CommSvr Object */
    if (pCommSvr != NULL)
    {
        delete(pCommSvr);
        pCommSvr = NULL;
    }
    /* Delete the pSyncEvent Object */
    if (pSyncEvent != NULL)
    {
        delete(pSyncEvent);
        pSyncEvent = NULL;
    }
    /* Delete the pListener CommSvr Object */
    if (pListener != NULL)
    {
        delete(pListener);
        pListener = NULL;
    }
}

void CInterchange::Init(uint16 RNDZPortNum, uint16 WACPPortNum, int32 Priority, void* pCallBackObj)
{   
    /* Set the Port values for RNDZ and WACP */
    RNDZPort = RNDZPortNum;
    WACPPort = WACPPortNum;

    /* Set up pseudoThis to point to 'this' for callback functionality */
    pThis = this;

    /* Initialize the WACP and RNDZ Thread Counts */
    WACPCount = 0;

    /* Create the event for synchronizing the connections */
    pSyncEvent = new CWACPEvent(NULL, FALSE);

    /* Create the semaphores for signaling data in the IN/OUT queue */
    pSendSemaphore = new CWACPSemaphore(NULL, 0, MAX_SEND_QUEUE_ITEMS);
    pRecvSemaphore = new CWACPSemaphore(NULL, 0, MAX_RECV_QUEUE_ITEMS);

    /* Open the named handle and signal the Semaphore */
    memset(AppSemName, 0, sizeof(AppSemName));
    sprintf(AppSemName, WACP_SEM_NAME, SYS_GETPID(), this);
    pDataSemaphore = new CWACPSemaphore(AppSemName, 0, MAX_SEM_COUNT);

    /* Initialize the comm objects */
    pCommSvr = new CCommSvr(this->RecvMsgWrapper, this->TerminateClientWrapper, this->AppSemName);
    pListener = NULL;

    /* Set this to TRUE before starting the Recv Threads */
    ThreadStatus = TRUE;

    /* Store stack settings */
    Args.ThreadPriority = Priority;
    Args.pCallBackObj = pCallBackObj;
    Args.pSourceID = NULL;

    /* Set the server state to TRUE */
    Args.bServer = TRUE;

    /* Start the Sending Thread */
    SendThread.StartThread(this->SendingThread, this, Args.ThreadPriority);
    
    /* Start the Receiving Thread */
    RecvThread.StartThread(this->ReceivingThread, this, Args.ThreadPriority);

#if defined(TRACE_SERVER_ON) || (TRACE_SERVER_ON_STATUS)
    /* Open the Trace file here so we can start writing to it */
    TraceServer.OpenFile(TRACE_EXCG_LOG, "Exchange");
    TraceServer.GenerateXSLFile();
    TraceServer.EnableLogging(TRUE, TRUE);
    ErrCount = 0;
#endif

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
    MsgInCount = 0;
    MsgOutCount = 0;
#endif

}

/*================================================================================
STATIC CInterchange Send Thread: Checks for messages in the OUT_list to send.
================================================================================*/
uint32 WACPAPI CInterchange::SendingThread(void* pData)
{
    /* Recreate the Exchange class from the pointer sent in pData. */
    CInterchange* pExchange = (CInterchange*)pData;

    /* Lock the Mutex until the thread is terminated */
    pExchange->SendThreadMutex.Lock(INFINITE);

    /* Wait until we get a notification of data in the Send queue */
    pExchange->pSendSemaphore->Wait(INFINITE);
    
    while(pExchange->GetThreadStatus() == (uint16)TRUE)
    {
        /* Process messages in the OUT_list */
        pExchange->EvaluateSend();  
        
        /* Wait until we get a notification of data in the Send queue */
        pExchange->pSendSemaphore->Wait(INFINITE);
    } 

    /* Release the mutex to allow CInterchange to terminate */
    pExchange->SendThreadMutex.Release();

    return TRUE;
}

/*================================================================================
STATIC CInterchange Receive Thread : Processes messages in the IN queue.
================================================================================*/
uint32 WACPAPI CInterchange::ReceivingThread(void* pData)
{
    /* Recreate the Exchange class from the pointer sent in pData. */
    CInterchange* pExchange = (CInterchange*)pData;

    /* Lock the Mutex until the thread is terminated */
    pExchange->RecvThreadMutex.Lock(INFINITE);

    /* Wait until we get a notification of data in the Recv queue */
    pExchange->pRecvSemaphore->Wait(INFINITE);

    while(pExchange->GetThreadStatus() == (uint16)TRUE)
    {
        /* Check for new messages */
        pExchange->EvaluateReceive();        

        /* Wait until we get a notification of data in the Recv queue */
        pExchange->pRecvSemaphore->Wait(INFINITE);
    }

    /* Release the mutex to allow CInterchange to terminate */
    pExchange->RecvThreadMutex.Release();
 
    return TRUE;
}

/*================================================================================
STATIC CInterchange CreateWACPLayer Creates the WACP Presentation Layer
================================================================================*/
uint32 WACPAPI CInterchange::CreateWACPLayer(void* pData)
{     
    ThreadArgs*   pThreadArgs;
    CInterchange* pExchange;
    char*         pSemName = NULL;
    uint16        Size;
    LinkStruct    LnkStruct;
    ConnArgs*     pConnArgs;

    /* Recreate the ThreadArgs structure from the pointer sent in pData. */
    pThreadArgs = (ThreadArgs*)pData;

    /* Recreate the Exchange class from the pointer sent in pThreadArgs. */
    pExchange = (CInterchange*)(pThreadArgs->pExchange);

    /* Set the semaphore name used to retrieve data to the SourceID */
    Size = (uint16)strlen(pThreadArgs->pSourceID) + 1;
    
    /* Update the bServer flag  */
    LnkStruct.bServer = pThreadArgs->bServer;

    /* Allocate memory for the SourceID */
    pSemName = (char*)SYS_MALLOC(Size);

    /* Return if we are out of memory */
    if (!pSemName)
        return FALSE; 

    memcpy(pSemName, pThreadArgs->pSourceID, Size);

    if (LnkStruct.bServer)
    {
        /* The Semaphore name is the sourceID for server baselines */
        LnkStruct.pSemName = pSemName;
    }
    else
    {
        /* The AppSemName is used for server baselines acting as a client */
        LnkStruct.pSemName = pExchange->AppSemName;
    }

    /* Set the ThreadPriority */
    LnkStruct.ThreadPriority = pExchange->Args.ThreadPriority;

    /* Set the DLL reference */
    LnkStruct.pCallBackObj = pExchange->Args.pCallBackObj;

    /* Set the CLinkTable instance */
    LnkStruct.pCLinkTable = (void*)&(pExchange->LinkTable);

    /* Create the application layer */  
    CWACPApp* pApp = new CWACPApp(pExchange->SendPacketWrapper,
                                  pExchange->RecvWACPPacketWrapper,
                                  pExchange->MsgFuncWrapper,
                                  &(pExchange->LinkTable),
                                  &LnkStruct);

    /* Raise the sync event to notify the link layer that we
     * are ready to receive messages
     */
    pExchange->pSyncEvent->Raise();

    if (pExchange->GetThreadStatus() == TRUE)
    {
        /* Start up the App */
        pApp->AppStart();
    }

    /* Delete the WACP Application Layer. This deletes 
     * the WACP Presentation Layer as well.
     */
    delete(pApp);

    /* Check for stack created connections to clean up.. this test fails
     * if the server baseline is used as a client.
     */
    pConnArgs = pExchange->LinkTable.GetEntry(pSemName);
    if (pConnArgs)
    {
        /* Delete the list entry, which also deletes the CommSvr object */
        pExchange->DeleteListEntry(pConnArgs->SourceID);

        /* Delete the CLinkTable entry */
        pExchange->LinkTable.RemoveEntry(pConnArgs);

        /* Disconnect the CommSvr */
        pExchange->pCommSvr->Disconnect(pConnArgs);

        /* Only free the pConnArgs structure it's not a bypass */
        if (pConnArgs->bByPass == FALSE)
        {
            /* Free up the ConnArgs structure allocated in CreateCommSession */
            SYS_FREE(pConnArgs);
        }
    }

    /* Free the memory allocated above */
    SYS_FREE(pSemName);

    /* For clients, notify the CWACPStack class the session is over */
    if (LnkStruct.bServer == FALSE)
    {
        CWACPEvent* pExitEvent;
        char        Buffer[EVENT_BUF_SIZE];

        memset(Buffer, 0, EVENT_BUF_SIZE);
        
        /* Raise an event to signify the end of the client session */
        sprintf(Buffer,EXIT_EXCHANGE,SYS_GETPID());
        pExitEvent = new CWACPEvent(Buffer, FALSE);
        pExitEvent->Raise();

        /* Clean up the Event object */
        delete(pExitEvent);
    }

    /* Decrement the WACPCont to track the number of connections */
    pExchange->CountMutex.Lock(INFINITE);
    pExchange->WACPCount--;
    pExchange->CountMutex.Release();

    return TRUE;
}

/*================================================================================
STATIC SendPacketWrapper: Wrapper function for SendPacket().
================================================================================*/
boolean CInterchange::SendPacketWrapper(char* pSemName, char* pSourceID, uint8* pBuf, uint32 Size)
{
    boolean RetVal = FALSE;

    TOUCH(pSemName);

    if (pThis != NULL)
    {
        RetVal = pThis->SendPacket(pSourceID, pBuf, Size);
    }

    return (RetVal);
}

/*================================================================================
STATIC RecvWACPPacket: Wrapper function for RecvWACPPacket().
================================================================================*/
uint32 CInterchange::RecvWACPPacketWrapper(char* pSemName, char** pSourceID, uint8** pBuf)
{
    uint32 RetVal = 0;

    if (pThis != NULL)
    {
        *pSourceID = pSemName;
        RetVal = pThis->RecvWACPPacket(pSourceID, pBuf);
    }

    return (RetVal);
}

/*================================================================================
STATIC MsgFuncWrapper: Wrapper function for MsgFunc().
================================================================================*/
boolean CInterchange::MsgFuncWrapper(char* pSemName, uint16 Option, void* pData, uint16 DataVal)
{
    boolean RetVal = FALSE;

    TOUCH(pSemName);

    if (pThis != NULL)
    {
        RetVal = pThis->MsgFunc(Option, pData, DataVal);
    }

    return (RetVal);
}

/*================================================================================
STATIC RecvMsgWrapper: Wrapper function for RecvMsg().pCommSvr and pNewConnection 
       are only used in Server configurations
================================================================================*/
boolean CInterchange::RecvMsgWrapper(char* pSemName, char* pSourceID, uint8* pBuf, uint32 MsgSize,
                                 void* pNewSessionArgs)
{
    boolean Result = FALSE;

    TOUCH(pSemName);

    if (pThis != NULL)
    {
        Result = pThis->RecvMsg(pSourceID, pBuf, MsgSize, pNewSessionArgs);
    }
    
    return (Result);
}

/*================================================================================
STATIC TerminateClientWrapper: Wrapper function for TerminateClient().
================================================================================*/
boolean CInterchange::TerminateClientWrapper(char* pSemName, char* pSourceID, uint16 Port)
{
    boolean Result = FALSE;

    TOUCH(pSemName);

    if (pThis != NULL) 
    {
        /* Only terminate if this is a WACP Port message type 
         * or special 0 type 
         */
        if ((Port == pThis->WACPPort) || (Port == 0))
        {
            Result = pThis->TerminateClient(pSourceID);
        }
    }
    
    return (Result);
}

/*================================================================================
CInterchange Send Packet: Wraps the passed in Buffer into a PACKET structure and
    sends the message via the CommSvr.
================================================================================*/
boolean CInterchange::SendPacket(char* pSourceID, uint8* pBuf, uint32 Size)
{
    boolean RetVal = FALSE;
    uint32 MutexResult;
    uint8* OutBuffer;
    uint32 PacketLen;
    ConnArgs* pConnArgs = NULL;

    /* Get a reference the CommSvr object based on the SemName */
    pConnArgs = LinkTable.GetEntry(pSourceID);
    if ((pConnArgs != NULL) && (pConnArgs->pCommSvr) && (pConnArgs->bConnected != FALSE))
    {
        /* Allocate room for the serialize packet Stream freed in EvaluateSend*/
        OutBuffer = GeneratePacket(&PacketLen, ((CCommSvr*)(pConnArgs->pCommSvr))->GetPortNum(pConnArgs),0, 0, pBuf, Size);
        if (OutBuffer)
        {
            MSG_LIST::iterator pList;
            boolean bFound = FALSE;
            
            /* Wait for a lock on the Send Mutex */
            MutexResult = SendMutex.Lock(INFINITE);

            /* Search for the message node that matches the SourceID */
            for (pList = OUT_list.begin() ; pList != OUT_list.end(); pList++)
            {
                if (strcmp(pList->SourceID, pSourceID) == 0)
                {
                    /* Add the message to the OUT_list message queue */
                    pList->pQueue->push((char*)OutBuffer);
                    bFound = TRUE;
                    break;
                }
            }

            if (bFound == FALSE)
            {
                AddListEntry(pSourceID, MSG_OUT_LIST, OutBuffer);
            }   

            /* Unlock the Send Mutex */
            SendMutex.Release();

            /* Raise the semaphore to notify the pSendThread */
            pSendSemaphore->Raise();

            /* Set the return value to TRUE */
            RetVal = TRUE;

        }/* if (OutBuffer) */
#if defined(TRACE_SERVER_ON)
        else
        {
            char tempString[] = {"Unable to allocate memory\nErrCount %d\nMessage Data:\n%A"};
            TraceServer.tprintf(TRACE_Type_ERROR,
                "CInterchange", "SendPacket", 0, 0, 0, 
                tempString, ++ErrCount, pBuf, Size);
        }
#endif

    }/* if ((pConnArgs != NULL) && (pConnArgs->pCommSvr)) */

    return(RetVal );
}

/*================================================================================
CInterchange RecvWACPPacket: Searches the messages in the STL list for the first
    message with the appropriate SourceID. The SourceID is found in the link 
    table via the passed in pSourceID.
================================================================================*/
uint32 CInterchange::RecvWACPPacket(char** pSourceID, uint8** pBuf)
{
    uint32 RetVal = 0;
    uint32 i;

    WACPRecvMutex.Lock(INFINITE);

    /* Check for valid pointers before continuing */
    if ((pSourceID != NULL) && (pBuf != NULL))
    {
        /* Make sure the list is not empty */
        if (!WACP_IN_list.empty())
        {
            /* Search for the packet that matches the SourceID */
            PACKET_LIST::iterator pList;
            
            /* Look for an existing entry */
            for (pList = WACP_IN_list.begin(), i=0; pList != WACP_IN_list.end();
                pList++, i++)
            {
                /* Clients running on a server baseline require the 
                 * SourceID to be updated and not passed in since
                 * the clients create the data thread before the
                 * connection is started
                 */
                if (strcmp(AppSemName, (char*)*pSourceID) == 0)
                {
                    *pSourceID = pList->pSourceID;
                }
                     
                if (strcmp(pList->pSourceID, (char*)*pSourceID) == 0)
                {
                    /* Update the data pointer */
                    *pBuf = pList->pDataBuf;

                    /* Update the return value with the size of the data pointer */
                    RetVal = pList->DataSize;

                    /* Remove the entry from the list */
                    WACP_IN_list.erase(pList);

                    /* Break out of the for loop */
                    break;
                }
            } /* for(...) */

        } /* if(!WACP_IN_list.empty()) */

    }/* if ((pSourceID != NULL) && (pBuf != NULL)) */
#if defined(DEBUG_PRINT_ERROR) && (TRACE_SERVER_ON)
    else
    {
        TraceServer.tprintf(TRACE_Type_ERROR,
                "CInterchange", "RecvWACPPacket", 0, 0, 0, 
                "ERROR!!!RecvWACPPacket: Invalid pSourceID and/or Buffer passed in\n");
    }
#endif

    /* Release the mutex */
    WACPRecvMutex.Release();

    return(RetVal );
}

/*================================================================================
CInterchange MsgFunc: 
    WACP_CONNECT: Opens a set of connections based on the parameters passed in.
    WACP_STATUS: Updated the RNDZ status of the currrent connection.
    WACP_DISCONNECT: Closes a single connection based on the SourceID in pData
    WACP_CLOSE: Closes all connections in the CLinkTable
================================================================================*/
boolean CInterchange::MsgFunc(uint16 Option, void* pData, uint16 DataVal)
{
    boolean RetVal = FALSE;

    if (ThreadStatus)
    {
        switch (Option)
        {
            case WACP_CONNECT:
                RetVal = Connect((ConnArgs**)pData, (uint8)DataVal);
                SYS_SLEEP(WAIT_COUNT);
                break;
            case WACP_STATUS:
                if (pCommSvr)
                {
                    ConnArgs* pConnArgs = LinkTable.GetEntry((char*)pData);
                    if (pConnArgs)
                    {
                        RetVal = pCommSvr->SetPortNum(pConnArgs);
                    }
                }
                break;
            case WACP_DISCONNECT:

                /* Allow time for the disconnect message to be sent */
                SYS_SLEEP(WAIT_COUNT);

                RetVal = Disconnect((char*)pData);
                break;
#if defined(DEBUG_PRINT_ERROR) && (TRACE_SERVER_ON)
            default:
                TraceServer.tprintf(TRACE_Type_ERROR,
                    "CInterchange", "SendPacket", 0, 0, 0, 
                    "ERROR Count %d!!! CInterchange MsgFunc: Error Invalid Option\n", ++ErrCount);
                break;
#endif
        }
    }

    return (RetVal);
}

/*================================================================================
CInterchange Evaluate Receive Thread 
================================================================================*/
void CInterchange::EvaluateReceive()
{
    PACKET      Pack;
    MESSAGE     Msg;
    char*       pSemName = NULL;
    ConnArgs*   pConnArgs = NULL;
    CWACPSemaphore* pDataSemaphore;
    MSG_LIST::iterator pList;
    
    /* Iterate through all connections for new messages to process */
    for (pList = IN_list.begin(); pList != IN_list.end(); pList++)
    {
        /* Process the message if the queue is not empty */
        if (pList->pQueue->empty() == FALSE)
        {
            uint8*  pTemp;
            char*   pBuf;
            int     Port;
            uint32  DataLength;

            /* Wait for the RecvBuf mutex */
            RecvMutex.Lock(INFINITE);

            /* Get the first packet Stream in the queue */
            pBuf = pList->pQueue->front();

            /* Remove the packet from the queue */
            pList->pQueue->pop();
            
            /* Release the RecvBuf Mutex */
            RecvMutex.Release();

            /* If we get an invalid pointer, then break out of the loop */
            if (pBuf == NULL)
                break;

            /* Set the Temp Pointer beyond the PREAMBLE of the byte stream */
            pTemp = (uint8*)pBuf + PREAMBLE_SIZE;

            /* Read the Packet Length */
            Pack.Header.PacketLen = ReadInt32(pTemp);
            pTemp += sizeof(int32);

            /* Read the Port */
            Pack.Header.Port = ReadInt16(pTemp);
            pTemp += sizeof(int16);

            /* Check the last Preamble Character...
             * CTL_C is the full packetization support mode.
             * CTL_L is the light mode that does not contain the SequenceNum and UUID
             */
            if (pBuf[2] == CTL_C)
            {
                /* Read the SequenceNum */
                Pack.Header.SequenceNum = ReadInt16(pTemp);
                pTemp += sizeof(int16);

                /* Read the Packet_UUDID */
                Pack.Header.Packet_UUID = ReadInt32(pTemp);
                pTemp += sizeof(int32);
                
                DataLength = Pack.Header.PacketLen - PACKET_HEADER_SIZE  - 2;
            }
            else
            {
                DataLength = Pack.Header.PacketLen - PACKET_LITE_HEADER_SIZE  - 2;
            }

            /* Allocate room for the databuffer. Released in RecvWACPPacket() or this 
            * function if the CRCs do not match
            */
            Pack.DataBuffer = (uint8*)SYS_MALLOC(DataLength);

            if (Pack.DataBuffer != NULL)
            {
                /* Get the databuffer */
                memcpy(Pack.DataBuffer, (char*)pTemp, DataLength);
                pTemp += DataLength;
            }

            /* Get the CRC */
            Pack.CRC = ReadInt16(pTemp);

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
            char tempString[] = {"SourceID %s\nMsgInCount %d\nMessage Data:\n%A"};
            TraceServer.tprintf(TRACE_Type_RECEIVE,
                "CInterchange", "EvaluateReceive", 0, 0, 0, 
                tempString, pList->SourceID, ++MsgInCount, pBuf, Pack.Header.PacketLen);
#endif
            /* Get the Port */
            Port = Pack.Header.Port;

            /* Check the SourceID in the CLinkTable */
            pConnArgs = LinkTable.GetEntry(pList->SourceID);

            /* Break out of the loop and drop the message if the entry is not found in the CLinkTable */
            if (!pConnArgs)
            {
#if defined(TRACE_SERVER_ON)
                char tempString[] = {"Unable to find CLinkTable Entry. SourceID %s\nMessage Data:\n%A"};
                TraceServer.tprintf(TRACE_Type_ERROR,
                    "CInterchange", "EvaluateReceive", 0, 0, 0, 
                    tempString, pList->SourceID, pBuf, Pack.Header.PacketLen);
#endif
                SYS_FREE(pBuf);
                SYS_FREE(Pack.DataBuffer);
                break;
            }

            /* Verify the CRC */
            if (Pack.CRC == CalculateCRC((uint8*)pBuf, Pack.Header.PacketLen - CRCSIZE))
            {
                boolean bRejectFlag = FALSE;

                /* Test that we have a valid Port number and, if RNDZ is enabled,
                * that RNDZ is complete before allowing any WACP messages.
                */
                if (pConnArgs->bRNDZDisabled == FALSE)
                {
                    bRejectFlag = (!(Port == RNDZPort || Port == WACPPort) ||
                                (((Port == WACPPort) && (pConnArgs->bRNDZStatus == FALSE))));
                }

                if (bRejectFlag)
                {
#if defined(TRACE_SERVER_ON)
                    char tempString[] = {"Unauthenticated Message. SourceID %s\nMessage Data:\n%A"};
                    TraceServer.tprintf(TRACE_Type_ERROR,
                        "CInterchange", "EvaluateReceive", 0, 0, 0, 
                        tempString, pList->SourceID, pBuf, Pack.Header.PacketLen);
#endif
                    SYS_FREE(pBuf);
                    SYS_FREE(Pack.DataBuffer);
                    return;
                }

                /* Set the data length */
                Msg.DataSize = DataLength;

                /* Set the Databuffer to the Pack DataBuffer */
                Msg.pDataBuf = Pack.DataBuffer;

                /* Set the Source ID */
                Msg.pSourceID = pList->SourceID;

                /* Check to see if we are running as a server or client */
                if (Args.bServer == (boolean)TRUE)
                {
                    /* Use the SourceID for the semaphore name for server mode */
                    pSemName = pList->SourceID;
                }        
                else
                {
                    /* Use the AppSemName for the semaphore name for client mode */
                    pSemName = AppSemName;
                }
            }/* if ( Pack.CRC == CalculateCRC(Pack)) */
            else
            {
                /*CRC mismatch! Pass message up to presentation layer*/
                
                /* Set the Source ID */
                Msg.pSourceID = pList->SourceID;

                if (pConnArgs->bSvrFlag == (boolean)TRUE)
                {
                    /* Get the EventHandle from the Link Table */
                    pSemName = pList->SourceID;
                }        
                else
                {
                    pSemName = AppSemName;
                }

                /* Set the data length */
                Msg.DataSize = sizeof(CRCError);
                
                Msg.pDataBuf = (uint8*)SYS_MALLOC(Msg.DataSize);
                if (Msg.pDataBuf != NULL)
                {
                    /* Set the CRCError in the message data buffer*/
                    memcpy(Msg.pDataBuf, CRCError, Msg.DataSize);
                } 

#if defined(TRACE_SERVER_ON)
                char tempString[] = {"CRC failure Stored CRC %d Calculated CRC %d\nErrCount %d\nMessage Data:\n%A"};
                TraceServer.tprintf(TRACE_Type_ERROR,
                    "CInterchange", "EvaluateReceive", 0, 0, 0, 
                    tempString, Pack.CRC, 
                    CalculateCRC((uint8*)pBuf, Pack.Header.PacketLen - CRCSIZE),
                    ++ErrCount, pBuf, Pack.Header.PacketLen);
#endif
                if (Pack.DataBuffer != NULL)
                {
                    /* Free the memory created above since this is a corrupted message */
                    SYS_FREE(Pack.DataBuffer);
                }
            }/* else */

            if (pSemName != NULL)
            {
                /* Wait for the WACP_IN_list mutex */
                WACPRecvMutex.Lock(INFINITE);
                
                /* Add the message to the WACP INQueue*/
                WACP_IN_list.push_back(Msg);

                /* Release the WACP_IN_list Mutex */
                WACPRecvMutex.Release();

                /* Open the named handle and signal the event */
                pDataSemaphore = new CWACPSemaphore(pSemName, 0, MAX_SEM_COUNT);

#if defined(TRACE_SERVER_ON_STATUS)
                TraceServer.tprintf(TRACE_Type_RECEIVE,
                    "CInterchange", "EvaluateReceive", 0, 0, 0, 
                    "Raising semaphore %s", pSemName);
#endif               
                pDataSemaphore->Raise();     

                delete(pDataSemaphore);
            }

            /* Free the message Buffer allocated in RecvMsg() */
            SYS_FREE(pBuf);

        }/*if (pList->pQueue->empty() == FALSE)*/

    }/* for (...) */
}

/*================================================================================
CInterchange Evaluate Send Thread 
================================================================================*/
void CInterchange::EvaluateSend()
{
    uint32  MutexResult;
    boolean Result = FALSE;
    MSG_LIST::iterator pList;
    ConnArgs* pConnArgs = NULL;

    /* Wait for the pSendMutex */
    MutexResult = SendMutex.Lock(INFINITE);

    /* Iterate through all connections for shared priority */
    for (pList = OUT_list.begin(); pList != OUT_list.end(); pList++)
    {
        /* Check to see if the queue is empty */
        if (pList->pQueue->empty() == FALSE)
        {
            /* Get the first available request Packet from Send_queue */
            char* pBuf = pList->pQueue->front();

            /* Get the pointer to the CCommSvr object */
            pConnArgs = LinkTable.GetEntry(pList->SourceID);
            if ((pConnArgs) && (pConnArgs->pCommSvr != NULL))
            {   
                /* Get the message Size */
                uint32 MsgSize = ReadInt32((uint8*)&pBuf[PREAMBLE_SIZE]); 

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
                char tempString[] = {"SourceID %s\nMsgOutCount %d\nMessage Data:\n%A"};
                TraceServer.tprintf(TRACE_Type_SEND,
                    "CInterchange", "EvaluateSend", 0, 0, 0, 
                    tempString, pList->SourceID, ++MsgOutCount, pBuf, MsgSize);
#endif
                /* Send the message */
                Result = ((CCommSvr*)(pCommSvr))->SendMsg(pConnArgs, (uint8*)pBuf, MsgSize);

            }
#if defined(TRACE_SERVER_ON)
            else
            {
                TraceServer.tprintf(TRACE_Type_ERROR,
                    "CInterchange", "EvaluateSend", 0, 0, 0, 
                    "Failed to find entry in CLinkTable %s\nErrCount %d",
                    pList->SourceID, ++ErrCount);
            }
#endif          
            /* Remove the packet Stream from the queue */
            pList->pQueue->pop();

            /* Free the memory allocated by SendPacket for the queue */
            SYS_FREE(pBuf);

            /* Check to see if the message was sent */
            if (Result == FALSE)
            {
                uint32 Count = 0;
                /* If the an error occurred, then we can assume a dead connection
                 * Delete all remaining items in the send queue
                 */

                while (pList->pQueue->empty() == FALSE)
                {
                    Count++;

                    /* Get the first available request Packet from Send_queue */
                    char* pBuf = pList->pQueue->front();

                    /* Remove the packet Stream from the queue */
                    pList->pQueue->pop();

                    /* Free the memory allocated by SendPacket for the queue */
                    SYS_FREE(pBuf);

                }/* while (pList->pQueue->empty() == FALSE) */

            }/* if (Result == FALSE) */

        }/* if (pList->pQueue->empty() == FALSE) */ 

    }/* for (pList = OUT_list.begin(); pList != OUT_list.end(); pList++) */

    /* Release the pSendMutex */
    SendMutex.Release();

    return;
}

/*================================================================================
CInterchange CreateWACPThread
================================================================================*/
boolean CInterchange::CreateWACPThread(ThreadArgs* pThreadArgs)
{
    CWACPThread TempThread;
    boolean     bResult = FALSE;

    if (ThreadStatus)
    {
        /* Create the WACP Presentation Layer */
        bResult = TempThread.StartThread(this->CreateWACPLayer, (void*)pThreadArgs, Args.ThreadPriority);
        if (bResult)
        {
            /* Increment the WACPCount to track the number of connections */
            CountMutex.Lock(INFINITE);
            WACPCount++;
            CountMutex.Release();
        }
    }
    return (bResult);
}

/*================================================================================
CInterchange CreateCommSession: Takes a pointer to a CCommSvr object and updates
    the link list of CCommSvr Objects. Once registered, the new CCommSvr thread 
    is started.
================================================================================*/
boolean CInterchange::CreateCommSession(char* pSourceID, void* pNewSessionArgs)
{
    int         Result = TRUE;
    uint16      Port;
    boolean     bNewFlag = FALSE;
    SessionArgs* pSessionArgs = (SessionArgs*)pNewSessionArgs;
    ConnArgs*    pConnArgs;

    /* Get the Port */
    Port = pSessionArgs->Port;

    /* Check for existing SourceID in Link Table */
    pConnArgs = LinkTable.GetEntry(pSourceID);

    /* Check to see if we have a new connection based on the SourceID.
     * Be sure to filter out new connection requests if a connection
     * already exists by checking the port of the new message and the
     * port of the existing connection. The following assumes an invalid
     * message and then checks for valid message scenarios.
     */
    if (pConnArgs == NULL)
    {
        /* If an entry does not exist, only allow new RNDZ Messages */
        if (Port == RNDZPort)
            bNewFlag = TRUE;
        else
            Result = FALSE;
    }

    if (bNewFlag == (boolean)TRUE)
    {
        /* Create a new CLinkTable Entry if this is a new connection */
        if (!pConnArgs)
        {
            pConnArgs = (ConnArgs*)SYS_CALLOC(sizeof(ConnArgs), 1);

            /* Check for a valid pointer */
            if (pConnArgs)
            {
                pConnArgs->StackType = pSessionArgs->StackType;
                pConnArgs->RNDZPort = RNDZPort;
                pConnArgs->WACPPort = WACPPort;
                pConnArgs->bSvrFlag = TRUE;
                pConnArgs->EncryptionType = WACP_ENCRYPTION_NONE;
                pConnArgs->Timeout = DEFAULT_RNDZ_TIMEOUT;
                pConnArgs->bByPass = pSessionArgs->bByPass;
                if (pConnArgs->bByPass == FALSE)
                {
                    pConnArgs->bConnected = TRUE;
                }
                pConnArgs->ValidationState = 0;
                pConnArgs->pConfig = pSessionArgs->pAddr; 
                memcpy(pConnArgs->SourceID, pSourceID, strlen(pSourceID)); 
            }
        }

        if (Port == RNDZPort)
        {
            /* Update the SourceID for the new connection */
            Args.pSourceID = pSourceID;
            Args.pExchange = this;

            /* Register the new CommSvr object */
            if (LinkTable.AddEntry(pConnArgs) == TRUE)
            {
                /* Create the RNDZ layer */ 
                if (CreateWACPThread(&Args))
                {
                    /* Add an entry into IN_list for the new connection */
                    AddListEntry(pSourceID, MSG_IN_LIST, NULL);

                    /* Set Result to TRUE */
                    Result = TRUE;
                }
                else
                {
                    Result = FALSE;
                }
            }
#if defined(TRACE_SERVER_ON)
            else
            {
                TraceServer.tprintf(TRACE_Type_ERROR,
                    "CInterchange", "CreateCommSession", 0, 0, 0, 
                    "LinkTable AddEntry ret. FALSE, max connections exceeded. %s connection rejected", pSourceID);
            }
#endif
        }
#if defined(TRACE_SERVER_ON)
        else
        {
            TraceServer.tprintf(TRACE_Type_ERROR,
                "CInterchange", "CreateCommSession", 0, 0, 0, 
                "Invalid Port %d from %s", Port, pSourceID);
        }
#endif

        if (Result == (boolean)TRUE)
        {
            CWACPThread     TempThread;

            if (Port == RNDZPort)
            {
                /* Wait until we get a sync event from the datathread before continuing */
                pSyncEvent->Wait(INFINITE);

                if ((pSessionArgs->bByPass == FALSE) && (ThreadStatus == TRUE))
                {
                    /* Start the new CCommSvr thread */
                    pCommSvr->Connect(pConnArgs);

                    /* Update the connection status */
                    pConnArgs->bConnected = TRUE;
                }
            }
        }
        else
        {
#if defined(TRACE_SERVER_ON)
            TraceServer.tprintf(TRACE_Type_ERROR,
                "CInterchange", "CreateCommSession", 0, 0, 0, 
                "Unauthorized connection request from %s", pSourceID);
#endif
            Result = FALSE;
        }

    }

    return (Result ? TRUE : FALSE);
}


/*================================================================================
CInterchange StartServer - Creates RNDZ and WACP listeners for both TCP and UDP
    protocols.
================================================================================*/
boolean CInterchange::StartServer(ConnArgs** pConnList, uint8 NumConnections)
{       
    boolean Result = TRUE;
    pListener = new CCommSvr(this->RecvMsgWrapper,
                                       this->TerminateClientWrapper,
                                       this->AppSemName);
    pListener->StartServer(pConnList, NumConnections);
    return(Result);
}

/*================================================================================
CInterchange StartClient
================================================================================*/
boolean CInterchange::StartClient(void)
{       
    boolean Result = FALSE;

    /* Set the thread arguements */
    Args.pExchange = this;

    /* Update the SourceID for the new connection */
    Args.pSourceID = AppSemName;
    Args.bServer = FALSE;

    /* Create the WACP thread */
    Result = CreateWACPThread(&Args);

    /* Sleep to allow the WACPThread to start */
    SYS_SLEEP(WAIT_COUNT/4);

    return (Result);
}
/*================================================================================
CInterchange Thread Status
================================================================================*/
boolean CInterchange::GetThreadStatus()
{   
    return ThreadStatus;
}

/*================================================================================
CInterchange Close: Closes all Mutex handles and sets the ThreadStatus to FALSE
================================================================================*/
uint16 CInterchange::Close()
{
    uint16 i;
    char* pSourceList[MAX_CONNECTIONS];
    
    ThreadStatus = FALSE;

    /* Get a list of all entries in CLinkTable and shut down all of them individually */
    LinkTable.GetSourceList(pSourceList);
    for (i=0; pSourceList[i] != NULL; i++)
    {
        TerminateClient(pSourceList[i]);
    }

    /* Wait until all connections have terminated */
    while(WACPCount > 0)
    {
        SYS_SLEEP(WAIT_COUNT);
    }

    /* Close the listeners */
    pListener->EndServer();

    /* Raise the events to allow the send/recv threads to run and terminate */
    pSendSemaphore->Raise();
    pRecvSemaphore->Raise();

    /* Wait for the Send/Recv Threads to terminate */
    SendThreadMutex.Lock(INFINITE);
    RecvThreadMutex.Lock(INFINITE);

    SendThreadMutex.Release();
    RecvThreadMutex.Release();

    if(pSendSemaphore != NULL)
        delete(pSendSemaphore);

    if(pRecvSemaphore != NULL)
        delete(pRecvSemaphore);

    if(pDataSemaphore != NULL)
        delete(pDataSemaphore);

    return TRUE;
}

/*================================================================================
CInterchange RecvMsg: Adds a message to the IN_queue
================================================================================*/
boolean CInterchange::RecvMsg(char* pSourceID, uint8* pBuf, uint32 MsgSize, void* pSessionArgs)
{
    int32   MutexResult;
    boolean Result = TRUE;
    char*   pNewSourceID = pSourceID;

    if (ThreadStatus == TRUE)
    {
        if (pSessionArgs != NULL)
        {
            /* Check to see if we need to create a new session */
            Result = CreateCommSession(pSourceID, pSessionArgs);

            /* If a new session is created, update the SourceID */
            if (Result == (boolean)TRUE)
            {
                pNewSourceID = ((SessionArgs*)pSessionArgs)->pSourceID;
            }
        }
        
        /* Check for a valid Buffer and Size */
        if ((ThreadStatus == TRUE) && (pBuf != NULL) && (Result == (boolean)TRUE))
        {
            char* pTemp;

            /* Allocate room for the new message, this is released in EvaluateReceive() */
            pTemp = (char*)SYS_MALLOC(MsgSize);
            if (pTemp != NULL)
            {
                boolean bMatch = FALSE;
                MSG_LIST::iterator pList;

                /* Create a copy of the message */
                memcpy(pTemp, pBuf, MsgSize);

                /* Wait for the Recv Mutex */
                MutexResult = RecvMutex.Lock(INFINITE);
        
                /* Search for the node that matches the SourceID */
                for (pList = IN_list.begin(); pList != IN_list.end(); pList++)
                {
                    if (strcmp(pList->SourceID, pNewSourceID) == 0)
                    {
                        pList->pQueue->push(pTemp);
#if defined(DEBUG_PRINT) && defined(TRACE_SERVER_ON)
                        TraceServer.tprintf(TRACE_Type_RECEIVE,
                            "CInterchange", "RecvMsg", 0, 0, 0, 
                            "Adding Msg from %s to IN list, Size=%d\n",pNewSourceID,
                            pList->pQueue->size());
#endif
                        bMatch = TRUE;
                        break;
                    }
                }

                if (bMatch == FALSE)
                {
                    /* Add an entry into IN_list for the new connection */
                    AddListEntry(pSourceID, MSG_IN_LIST, (uint8*)pTemp);
                }   
          
                /* Release the Mutex */
                RecvMutex.Release();

                /* Notify the Receiving thread to process the message */
                pRecvSemaphore->Raise();
            }
        }
    }
 
    return (Result);
}

/*================================================================================
CInterchange AddListEntry: Search for the SourceID and adds the entry from
    the list. 
================================================================================*/
void CInterchange::AddListEntry(char* pSourceID, uint8 List, uint8* pBuf)
{
    MSG_node    pNode;

    /* Create a new queue for the IN_list */
    pNode.pQueue = new MSG_QUEUE;

    if (pBuf != NULL)
    {
        pNode.pQueue->push((char*)pBuf);
    }

    memset(pNode.SourceID, 0, ADDR_BUF_SIZE);
    if (pSourceID != NULL)
    {
        memcpy(pNode.SourceID, pSourceID, strlen((char*)pSourceID));
    }

    if (List == MSG_IN_LIST)
    {
#if defined(DEBUG_PRINT) && defined(TRACE_SERVER_ON)
        TraceServer.tprintf(TRACE_Type_STATUS,
            "CInterchange", "AddListEntry", 0, 0, 0,
            "Adding Msg node for %s to IN_list on\n", pSourceID);
#endif
        RecvMutex.Lock(INFINITE);
        IN_list.push_back(pNode);
        RecvMutex.Release();
    }
    else if (List == MSG_OUT_LIST)
    {
#if defined(DEBUG_PRINT) && defined(TRACE_SERVER_ON)
        TraceServer.tprintf(TRACE_Type_STATUS, 
            "CInterchange", "AddListEntry", 0, 0, 0,
            "Adding Msg node for %s to OUT_list ond\n", pSourceID);
#endif
        SendMutex.Lock(INFINITE);
        OUT_list.push_back(pNode);
        SendMutex.Release();
    }
}
/*================================================================================
CInterchange RemoveListEntry: Search for the SourceID and remove the entry from
    the list. 
================================================================================*/
void CInterchange::DeleteListEntry(char* pSourceID)
{
    /* Find the queue for this message */
    MSG_LIST::iterator pList;
    PACKET_LIST::iterator pPackList;
    uint32 MutexResult;

    MutexResult = RecvMutex.Lock(INFINITE);

    /* Search for the node that matches the SourceID */
    for (pList = IN_list.begin(); pList != IN_list.end(); pList++)
    {
        if (strcmp(pList->SourceID, (char*)pSourceID) == 0)
        {
            /* Delete the items in the queue */
            while (pList->pQueue->empty() == FALSE)
            {
                /* Get the first available request Packet from Send_queue */
                char* pBuf = pList->pQueue->front();

                /* Remove the packet Stream from the queue */
                pList->pQueue->pop();

                /* Free the memory allocated by SendPacket for the queue */
                SYS_FREE(pBuf);
            }

            /* Delete the queue */
            delete(pList->pQueue);

#if defined(DEBUG_PRINT) && defined(TRACE_SERVER_ON)
           TraceServer.tprintf(TRACE_Type_STATUS,
               "CInterchange", "DeleteListEntry", 0, 0, 0,
               "Removing Msg node for %s to IN_list\n",pList->SourceID);
#endif
            /* Erase the entry from the list */
            IN_list.erase(pList);
            break;
        }
    }

    RecvMutex.Release();

    MutexResult = SendMutex.Lock(INFINITE);

    for (pList = OUT_list.begin(); pList != OUT_list.end(); pList++)
    {
        if (strcmp(pList->SourceID, (char*)pSourceID) == 0)
        {
            /* Delete the items in the queue */
            while (pList->pQueue->empty() == FALSE)
            {
                /* Get the first available request Packet from Send_queue */
                char* pBuf = pList->pQueue->front();

                /* Remove the packet Stream from the queue */
                pList->pQueue->pop();

                /* Free the memory allocated by SendPacket for the queue */
                SYS_FREE(pBuf);
            }

            /* Delete the queue */
            delete(pList->pQueue);

#if defined(DEBUG_PRINT) && defined(TRACE_SERVER_ON)
             TraceServer.tprintf(TRACE_Type_STATUS, 
                 "CInterchange", "DeleteListEntry", 0, 0, 0,
                 "Removing Msg node for %s to OUT_list\n",pList->SourceID);
#endif

            /* Erase the entry from the list */
            OUT_list.erase(pList);
            break;
        }
    }

    SendMutex.Release(); 

    WACPRecvMutex.Lock(INFINITE);

    for (pPackList = WACP_IN_list.begin(); pPackList != WACP_IN_list.end();)
    {
        if (strcmp(pPackList->pSourceID, (char*)pSourceID) == 0)
        {
            SYS_FREE(pPackList->pDataBuf);

            /* Erase all entries for this pSourceID from the list */
            pPackList = WACP_IN_list.erase(pPackList);
        }
        else
        {
            pPackList++;
        }
    }
    WACPRecvMutex.Release();

}

/*================================================================================
TerminateClient: Terminates the client connection
================================================================================*/
boolean CInterchange::TerminateClient(char* pSourceID)
{
    boolean Result = FALSE;
    MESSAGE Msg;
    CWACPSemaphore* pDataSemaphore;
    MSG_LIST::iterator pList;
    PACKET_LIST::iterator pPackList;

    RecvMutex.Lock(INFINITE);

    /* Search for the node that matches the SourceID */
    for (pList = IN_list.begin(); pList != IN_list.end(); pList++)
    {
        if (strcmp(pList->SourceID, pSourceID) == 0)
        {
            /* Set the data length */
            Msg.DataSize = sizeof(WacpTerminate);

            Msg.pDataBuf = (uint8*)SYS_MALLOC(Msg.DataSize);

            if (Msg.pDataBuf != NULL)
            {
                /* Set the Databuffer to the Pack DataBuffer */
                memcpy(Msg.pDataBuf, WacpTerminate, Msg.DataSize);

                /* Set the Source ID */
                Msg.pSourceID = pList->SourceID;

                /* Wait for the WACP_IN_list mutex */
                WACPRecvMutex.Lock(INFINITE);
                
                /* Remove all messages in the queue to process the termination message */
                for (pPackList = WACP_IN_list.begin(); pPackList != WACP_IN_list.end();)
                {
                    if (strcmp(pPackList->pSourceID, (char*)pSourceID) == 0)
                    {
                        SYS_FREE(pPackList->pDataBuf);

                        /* Erase the entry from the list */
                        pPackList = WACP_IN_list.erase(pPackList);
                    }
                    else
                    {
                        pPackList++;
                    }
                }                
                /* Add the message to the WACP INQueue*/
                WACP_IN_list.push_back(Msg);

                /* Release the WACP_IN_list Mutex */
                WACPRecvMutex.Release();

                /* Open a handle to the named semaphore */
                pDataSemaphore = new CWACPSemaphore(pSourceID, 0, MAX_SEM_COUNT);
                
                /* Signal the event to notify the Presentation layer there is data available */
                pDataSemaphore->Raise();     
            
                /* Clean up the DataEvent Object */
                delete(pDataSemaphore);

                Result = TRUE;
            }/* if (Msg.pDataBuf != NULL) */

        }/*if (strcmp(pList->SourceID, (char*)pSourceID) == 0)*/

    }/*for (pList = OUT_list.begin(); pList != OUT_list.end(); pList++)*/

    RecvMutex.Release();

    return Result;
}

/*================================================================================
TerminateAllClients: Terminates all client connections
================================================================================*/
void CInterchange::TerminateAllClients()
{
    MSG_LIST::iterator pList;

    /* Check for empty list */
    if (!IN_list.empty())
    {
        for (pList = IN_list.begin(); pList != IN_list.end(); pList++)
        {
           TerminateClient(pList->SourceID);
        }
    }
}


/*================================================================================
GeneratePacket: Wraps the passed in Buffer into a PACKET structure and
    adds header information based on parameters. Returns a newly malloced 
    uint8 buffer filled with the new packet.
================================================================================*/
uint8* GeneratePacket(uint32* OutSize, int16 Port, int16 SequenceNum, int32 UUID, uint8* pBuf, uint32 Size)
{
    uint8* OutBuffer = NULL;
    boolean Packetized = FALSE;

    /* Set the Size of the entire packet */
    if (Packetized)
    {
        *OutSize = PACKET_HEADER_SIZE + Size + CRCSIZE;
    }
    else
    {
        *OutSize = PACKET_LITE_HEADER_SIZE + Size + CRCSIZE;
    }

    /* Allocate room for the serialize packet Stream freed in EvaluateSend*/
    OutBuffer = (uint8*)SYS_MALLOC(*OutSize);
    
    if (OutBuffer)
    {
        uint8* pTemp = OutBuffer; 

        /* Serialize the packet header struct */
        /* Set the PREAMBLE */    
        *pTemp++ = CTL_W;
        *pTemp++ = CTL_A;

        /* TODO: Check to see if this is a piece of a packetized message */
        if (Packetized)
            *pTemp++ = CTL_C;
        else
            *pTemp++ = CTL_L;

        /* Write the Packet Length */
        WriteInt32(pTemp, *OutSize);
        pTemp += sizeof(int32);

        /* Write the Port */
        WriteInt16(pTemp, Port);
        pTemp += sizeof(int16);

        /* TODO: Check to see if this is a piece of a packetized message */
        if (Packetized)
        {
            /* Write the SequenceNum */
            WriteInt16(pTemp, SequenceNum);
            pTemp += sizeof(int16);

            /* Write the Packet_UUDID */
            WriteInt32(pTemp, UUID);
            pTemp += sizeof(int32);
        }

        /* Copy the data Stream */
        memcpy((char*)pTemp, pBuf, Size);
        pTemp += Size;
        
        /* Write the CRC */
        WriteInt16(pTemp, CalculateCRC(OutBuffer, *OutSize - CRCSIZE));

    }/* if (OutBuffer) */

    return(OutBuffer);
}

/*================================================================================
Connect: Creates the connections specified in the ConnArgs array passed in. The
    status boolean bConnected is updated with each individual connection status.
    
    Returns TRUE if all connections are opened successfully
    Returns FALSE if any one connection fails to open.
================================================================================*/
boolean CInterchange::Connect(ConnArgs** pConfig, uint8 NumConnections)
{
    boolean Result = FALSE;
    uint16 i = 0;

    if (pCommSvr != NULL)
    {
        for (i=0; i < NumConnections; i++)
        {
            if (pConfig[i]->bConnected == FALSE)
            {
                /* Start the connection */
                Result = pCommSvr->Connect(pConfig[i]);
                if (Result)
                {
                    /* Add an entry into IN_list for the new connection */
                    AddListEntry(pConfig[i]->SourceID, MSG_IN_LIST, NULL);
                }
#if defined(TRACE_SERVER_ON)
                else
                {
                    TraceServer.tprintf(TRACE_Type_ERROR,
                        "CInterchange", "Connect", 0, 0, 0,
                        "Failed to connect to %s\n", pConfig[i]->SourceID);
                }
#endif
            }
#if defined(TRACE_SERVER_ON)
            else
            {
                TraceServer.tprintf(TRACE_Type_ERROR,
                    "CInterchange", "Connect", 0, 0, 0,
                    "%s is already connected\n", pConfig[i]->SourceID);
            }
#endif
        }
    }

    return (Result);
}

/*================================================================================
Disconnect: Closes down a single connection based on the SourceID.

    Returns TRUE if the connection is successfully closed.
    Returns FALSE if the SourceID is not found or if the connection was not closed.
================================================================================*/
boolean CInterchange::Disconnect(char* pSourceID)
{
    boolean Result = FALSE;
    ConnArgs* pConnArgs = NULL;

    /* Find the entry in the CLinkTable */
    pConnArgs = LinkTable.GetEntry(pSourceID);
    if ((pConnArgs) && (pConnArgs->bConnected == (boolean)TRUE))
    {
        /* Delete the list entry */
        DeleteListEntry(pConnArgs->SourceID);

        /* Destroy the CommSvr */
        Result = pCommSvr->Disconnect(pConnArgs);

        /* Reset the RNDZ authentication */
        pConnArgs->bRNDZStatus = FALSE;

        /* Reset the connection status */
        pConnArgs->bConnected = FALSE;

        /* Free up the memory allocated in connection sequence */
        SYS_FREE(pConnArgs->pKey);
        pConnArgs->pKey = NULL;

        if (pConnArgs->pGUIDStd)
        {
            delete(pConnArgs->pGUIDStd);
            pConnArgs->pGUIDStd = NULL;
        }
    }
    else
    {
        Result = FALSE;
#if defined(TRACE_SERVER_ON)
        TraceServer.tprintf(TRACE_Type_ERROR,
            "CInterchange", "Disconnect", 0, 0, 0,
            "Unable to find %s\n", pSourceID);
#endif
    }

    return (Result);
}

