/*-D************************************************************************* 
'*   This program source file is confidential and proprietary property of
'*    Welch Allyn, Inc. Unauthorized use or reproduction is prohibited.
'****************************************************************************
'****************************************************************************
'**
'****************************************************************************
'**  ////////////////////////////////////////////////////////////////////////
'**  '// CInterchange.cpp:                                                 //
'**  '// Language       :   C++                                            //
'**  '// Application    :   WACP                                           //
'**  '// Sources        :   MOMP Standard Library                          //
'**  '///////////////////////////////////////////////////////////////////////
'*-E************************************************************************/

#if !defined(CINTERCHANGE_H)
    #define  CINTERCHANGE_H

#include "Common.h"
#include "CCommSvr.h"
#include "CWACPApp.h"
#include "CWACPMutex.h"
#include "CWACPEvent.h"
#include "CTrapObjSvr.h"
#include "CWACPThread.h"
#include "SysUtil.h"
#include <queue>
#include <list>
#include "CWACPSemaphore.h"
#include "CTraceSvr.h"
#include "CLinkTable.h"

using namespace std;

#define WACP_SEM_NAME "WACP_%d_%p"
#define EXIT_EXCHANGE "EXIT_EXCHANGE_%d"
#define CRCSIZE (2)

/* Used by AddListEntry */
#define MSG_IN_LIST     ( 0 )
#define MSG_OUT_LIST    ( 1 )

struct PACKET_HDR
{
    uint8   Preamble[ PREAMBLE_SIZE ];
    uint32  PacketLen;
    uint16  Port;
    uint16  SequenceNum;
    uint32  Packet_UUID;
};

/* PACKET_LITE_HEADER_SIZE = PREAMBLE + PacketLen + Port
 *      PREAMBLE  = sizeof(uint8) * PREAMBLE_SIZE
 *      PacketLen = sizeof(uint32)
 *      Port      = sizeof(uint16) 
 */
#define PACKET_LITE_HEADER_SIZE 9

/* PACKET_HEADER_SIZE = PREAMBLE + PacketLen + Port + SequenceNum + Packet_UUID
 *      PREAMBLE    = sizeof(uint8) * PREAMBLE_SIZE
 *      PacketLen   = sizeof(uint32)
 *      Port        = sizeof(uint16) 
 *      SequenceNum = sizeof(uint16) 
 *      Packet_UUID = sizeof(uint32)
 */
#define PACKET_HEADER_SIZE 15

struct PACKET
{
    PACKET_HDR  Header;
    uint8       *DataBuffer;
    uint16      CRC;
};

struct MESSAGE
{
    char        *pSourceID;
    uint8       *pDataBuf;
    uint32      DataSize;
};

typedef struct ThreadArgs
{
    void *pExchange;
    char *pSourceID;
    int32 ThreadPriority;
    void *pCallBackObj;
    boolean bServer;
}ThreadArgs;

typedef queue< char* > MSG_QUEUE;

struct MSG_node
{
    MSG_QUEUE   *pQueue;
    char        SourceID[ADDR_BUF_SIZE];
};

typedef list < MESSAGE > PACKET_LIST;
typedef list < MSG_node > MSG_LIST;
struct DLLStruct
{
    void     *pExchange;
    char     *pSourceID;
    char     *pAppName;
};

class CInterchange
{
public:


    CInterchange();
    ~CInterchange();

    void Init(uint16 RNDZPortNum, uint16 WACPPortNum, int32 Priority, void *pCallBackObj);

    /* Thread Functions */
    boolean StartServer(ConnArgs **pConnList, uint8 NumConnections);
    boolean StartClient(void);
    boolean CreateWACPThread(ThreadArgs *pThreadArgs);

    uint16 Close();

    boolean GetThreadStatus();
    void AddListEntry(char *pSourceID, uint8 list, uint8 *pBuf);
    void DeleteListEntry(char *pSourceID);
    boolean TerminateClient(char *pSourceID);
    void TerminateAllClients();

    /* STL List class */
    MSG_LIST        IN_list;
    MSG_LIST        OUT_list;
    PACKET_LIST     WACP_IN_list;
private:
    static CInterchange *pThis;

    /* Static Callback Function Wrappers */
    static boolean SendPacketWrapper(char *pSemName, char *pSourceID, uint8 *pBuf, uint32 Size);
    static uint32 RecvWACPPacketWrapper(char *pSemName, char **pSourceID, uint8** pBuf);
    static boolean MsgFuncWrapper(char *pSemName, uint16 Option, void *pData, uint16 DataVal);
    static boolean RecvMsgWrapper(char *pSemName, char *pSourceID, uint8* pBuf, uint32 MsgSize, void *pNewSessionArgs);
    static boolean TerminateClientWrapper(char *pSemName, char *pSourceID, uint16 Port);

    /* Callback Functions */
    boolean SendPacket(char *pSourceID, uint8 *pBuf, uint32 Size);
    uint32 RecvWACPPacket(char** pSourceID, uint8** pBuf);
    boolean MsgFunc(uint16 Option, void *pData, uint16 DataVal);
    boolean RecvMsg(char *pSourceID, uint8* pBuf, uint32 MsgSize, void *pSessionArgs);
    boolean Connect(ConnArgs **pConnArgs, uint8 NumConnections);
    boolean Disconnect(char *pSourceID);
    boolean CreateCommSession(char *pSourceID, void *pNewSessionArgs);

    /* Thread Functions */
    static uint32 WACPAPI SendingThread(void *pData);
    static uint32 WACPAPI ReceivingThread(void *pData);
    static uint32 WACPAPI CreateWACPLayer(void *pData);

    /* Send and Receive Thread functions */
    void EvaluateReceive();
    void EvaluateSend();

    /* Threads */
    CWACPThread SendThread;
    CWACPThread RecvThread;

    /* Mutex Pointers */
    CWACPMutex  WACPRecvMutex;             /* WACP Recv Queue mutex */
    CWACPMutex  SendMutex;                 /* OUT_queue mutex  */
    CWACPMutex  RecvMutex;                 /* IN_queue mutex */
    CWACPMutex  RecvThreadMutex;           /* Exit RecvThread sync mutex */
    CWACPMutex  SendThreadMutex;           /* Exit SendThread sync mutex */
    CWACPMutex  CountMutex;

    /* Thread Status var */
    volatile boolean ThreadStatus;

    /* Thread arguements*/
    ThreadArgs Args;

    /* WACP and RNDZ Ports */
    uint16 RNDZPort;
    uint16 WACPPort;

    /* Client: Base class for all outgoing connections */
    CCommSvr *pCommSvr;
    
    /* Server: Base class for all incoming connections */
    CCommSvr *pListener;

    /* Semaphores used for message notification */
    CWACPSemaphore  *pSendSemaphore;
    CWACPSemaphore  *pRecvSemaphore;
    CWACPSemaphore  *pDataSemaphore;
    char            AppSemName[EVENT_BUF_SIZE];

    /* Synchronization Event */
    CWACPEvent      *pSyncEvent;
    
    /* Reference to CLinkTable */
    CLinkTable      LinkTable;

    /* WACP Thread Count */
    volatile uint16 WACPCount;

#if defined(TRACE_SERVER_ON) || defined(TRACE_SERVER_ON_STATUS)
    /* TraceServer reference */
    CTraceSvr   TraceServer;
    uint32 ErrCount;
#endif

#if defined(TRACE_SERVER_ON) && defined(TRACE_SERVER_ON_STATUS)
    uint32 MsgInCount;
    uint32 MsgOutCount;
#endif

};

/* Helper function used to generate packets. Also used by CCommSvr for fake messages */
uint8 *GeneratePacket(uint32 *OutSize, int16 Port, int16 SequenceNum, int32 UUID, uint8 *pBuf, uint32 Size);

#endif /* #if !defined(CINTERCHANGE_H) */

/*---------------------------------------------------*/
/* End of File */





